@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: transmissioncli.c 5115 2008-02-25 21:53:53Z charles $
+ * $Id: transmissioncli.c 6185 2008-06-13 21:25:25Z charles $
  *
  * Copyright (c) 2005-2006 Transmission authors and contributors
  *
@@ -30,10 +30,11 @@
 #include <signal.h>
 
 #include <libtransmission/transmission.h>
+#include <libtransmission/bencode.h>
 #include <libtransmission/makemeta.h>
 #include <libtransmission/metainfo.h> /* tr_metainfoFree */
 #include <libtransmission/utils.h> /* tr_wait */
-
+#include <libtransmission/web.h> /* tr_webRun */
 
 /* macro to shut up "unused parameter" warnings */
 #ifdef __GNUC__
@@ -43,23 +44,25 @@
 #endif
 
 const char * USAGE =
-"Usage: %s [-car[-m]] [-dfinpsuv] [-h] file.torrent [output-dir]\n\n"
+"Usage: %s [-car[-m]] [-dfginpsuv] [-h] file.torrent [output-dir]\n\n"
 "Options:\n"
+"  -h, --help                Print this help and exit\n" 
+"  -i, --info                Print metainfo and exit\n"
+"  -s, --scrape              Print counts of seeders/leechers and exit\n"
+"  -V, --version             Print the version number and exit\n"
 "  -c, --create-from <file>  Create torrent from the specified source file.\n"
-"  -a, --announce <url> Used in conjunction with -c.\n"
-"  -r, --private        Used in conjunction with -c.\n"
-"  -m, --comment <text> Adds an optional comment when creating a torrent.\n"
-"  -d, --download <int> Maximum download rate (-1 = no limit, default = -1)\n"
-"  -f, --finish <shell script> Command you wish to run on completion\n" 
-"  -h, --help           Print this help and exit\n" 
-"  -i, --info           Print metainfo and exit\n"
-"  -n  --nat-traversal  Attempt NAT traversal using NAT-PMP or UPnP IGD\n"
-"  -p, --port <int>     Port we should listen on (default = %d)\n"
-"  -s, --scrape         Print counts of seeders/leechers and exit\n"
-"  -u, --upload <int>   Maximum upload rate (-1 = no limit, default = 20)\n"
-"  -v, --verbose <int>  Verbose level (0 to 2, default = 0)\n"
-"  -V, --version        Print the version number and exit\n"
-"  -y, --recheck        Force a recheck of the torrent data\n";
+"  -a, --announce <url>      Used in conjunction with -c.\n"
+"  -g, --config-dir <path>   Where to look for configuration files\n"
+"  -o, --output-dir <path>   Where to save downloaded data\n"
+"  -r, --private             Used in conjunction with -c.\n"
+"  -m, --comment <text>      Adds an optional comment when creating a torrent.\n"
+"  -d, --download <int>      Max download rate (-1 = no limit, default = -1)\n"
+"  -f, --finish <script>     Command you wish to run on completion\n" 
+"  -n  --nat-traversal       Attempt NAT traversal using NAT-PMP or UPnP IGD\n"
+"  -p, --port <int>          Port we should listen on (default = %d)\n"
+"  -u, --upload <int>        Maximum upload rate (-1 = no limit, default = 20)\n"
+"  -v, --verbose <int>       Verbose level (0 to 2, default = 0)\n"
+"  -y, --recheck             Force a recheck of the torrent data\n";
 
 static int           showHelp      = 0;
 static int           showInfo      = 0;
@@ -79,6 +82,7 @@ static sig_atomic_t  manualUpdate  = 0;
 
 static char          * finishCall   = NULL;
 static char          * announce     = NULL;
+static char          * configdir    = NULL;
 static char          * sourceFile   = NULL;
 static char          * comment      = NULL;
 
@@ -96,6 +100,26 @@ getStringRatio( float ratio )
     return string;
 }
 
+static int
+is_rfc2396_alnum( char ch )
+{
+    return     ( '0' <= ch && ch <= '9' )
+            || ( 'A' <= ch && ch <= 'Z' )
+            || ( 'a' <= ch && ch <= 'z' );
+}
+
+static void
+escape( char * out, const uint8_t * in, int in_len ) /* rfc2396 */
+{
+    const uint8_t *end = in + in_len;
+    while( in != end )
+        if( is_rfc2396_alnum(*in) )
+            *out++ = (char) *in++;
+        else
+            out += snprintf( out, 4, "%%%02X", (unsigned int)*in++ );
+    *out = '\0';
+}
+
 #define LINEWIDTH 80
 
 static void
@@ -106,6 +130,36 @@ torrentStateChanged( tr_torrent   * torrent UNUSED,
     system( finishCall );
 }
 
+static int leftToScrape = 0;
+
+static void
+scrapeDoneFunc( struct tr_handle    * session UNUSED,
+                long                  response_code,
+                const void          * response,
+                size_t                response_byte_count,
+                void                * host )
+{
+    tr_benc top, *files;
+
+    if( !tr_bencLoad( response, response_byte_count, &top, NULL ) 
+        && tr_bencDictFindDict( &top, "files", &files )
+        && files->val.l.count >= 2 )
+    {
+        int64_t complete=-1, incomplete=-1, downloaded=-1;
+        tr_benc * hash = &files->val.l.vals[1];
+        tr_bencDictFindInt( hash, "complete", &complete );
+        tr_bencDictFindInt( hash, "incomplete", &incomplete );
+        tr_bencDictFindInt( hash, "downloaded", &downloaded );
+        printf( "%4d seeders, %4d leechers, %5d downloads at %s\n",
+                (int)complete, (int)incomplete, (int)downloaded, (char*)host );
+        tr_bencFree( &top );
+    }
+    else
+        printf( "unable to parse response (http code %lu) at %s", response_code, (char*)host );
+
+    --leftToScrape;
+}
+
 int
 main( int argc, char ** argv )
 {
@@ -139,45 +193,94 @@ main( int argc, char ** argv )
         return EXIT_FAILURE;
     }
 
+    /* don't bind the port if we're just running the CLI 
+     * to get metainfo or to create a torrent */
+    if( showInfo || showScrape || ( sourceFile != NULL ) )
+        bindPort = -1;
+
+    if( configdir == NULL )
+        configdir = strdup( tr_getDefaultConfigDir( ) );
+
     /* Initialize libtransmission */
-    h = tr_initFull( "cli",
-                     1,                       /* pex enabled */
-                     natTraversal,            /* nat enabled */
-                     bindPort,                /* public port */
-                     TR_ENCRYPTION_PREFERRED, /* encryption mode */
-                     uploadLimit >= 0,        /* use upload speed limit? */
-                     uploadLimit,             /* upload speed limit */
-                     downloadLimit >= 0,      /* use download speed limit? */
-                     downloadLimit,           /* download speed limit */
-                     512,                     /* globalPeerLimit */
-                     verboseLevel + 1,        /* messageLevel */
-                     0 );                     /* is message queueing enabled? */
+    h = tr_initFull( configdir,
+                     "cli",                         /* tag */
+                     1,                             /* pex enabled */
+                     natTraversal,                  /* nat enabled */
+                     bindPort,                      /* public port */
+                     TR_ENCRYPTION_PREFERRED,       /* encryption mode */
+                     uploadLimit >= 0,              /* use upload speed limit? */
+                     uploadLimit,                   /* upload speed limit */
+                     downloadLimit >= 0,            /* use download speed limit? */
+                     downloadLimit,                 /* download speed limit */
+                     TR_DEFAULT_GLOBAL_PEER_LIMIT,
+                     verboseLevel + 1,              /* messageLevel */
+                     0,                             /* is message queueing enabled? */
+                     0,                             /* use the blocklist? */
+                     TR_DEFAULT_PEER_SOCKET_TOS );
 
     if( sourceFile && *sourceFile ) /* creating a torrent */
     {
-        int ret;
+        int err;
         tr_metainfo_builder * builder = tr_metaInfoBuilderCreate( h, sourceFile );
         tr_makeMetaInfo( builder, torrentPath, announce, comment, isPrivate );
         while( !builder->isDone ) {
             tr_wait( 1000 );
             printf( "." );
         }
-        ret = !builder->failed;
+        err = builder->result;
         tr_metaInfoBuilderFree( builder );
-        return ret;
+        return err;
     }
 
     ctor = tr_ctorNew( h );
     tr_ctorSetMetainfoFromFile( ctor, torrentPath );
-    tr_ctorSetPaused( ctor, TR_FORCE, 0 );
+    tr_ctorSetPaused( ctor, TR_FORCE, showScrape );
     tr_ctorSetDestination( ctor, TR_FORCE, savePath );
 
+    if( showScrape )
+    {
+        tr_info info;
+
+        if( !tr_torrentParse( h, ctor, &info ) )
+        {
+            int i;
+            const time_t start = time( NULL );
+            for( i=0; i<info.trackerCount; ++i )
+            {
+                if( info.trackers[i].scrape )
+                {
+                    const char * scrape = info.trackers[i].scrape;
+                    char escaped[SHA_DIGEST_LENGTH*3 + 1];
+                    char url[2048], *host;
+                    escape( escaped, info.hash, SHA_DIGEST_LENGTH );
+                    snprintf( url, sizeof(url), "%s%cinfo_hash=%s",
+                                                scrape,
+                                                strchr(scrape,'?')?'&':'?',
+                                                escaped );
+                    tr_httpParseURL( scrape, -1, &host, NULL, NULL );
+                    ++leftToScrape;
+                    tr_webRun( h, url, scrapeDoneFunc, host );
+                }
+            }
+
+            fprintf( stderr, "scraping %d trackers:\n", leftToScrape );
+
+            while( leftToScrape>0 && ((time(NULL)-start)<20) )
+                tr_wait( 250 );
+        }
+        goto cleanup;
+    }
+
+
     if( showInfo )
     {
         tr_info info;
 
         if( !tr_torrentParse( h, ctor, &info ) )
         {
+            int prevTier = -1;
+            tr_file_index_t ff;
+
             printf( "hash:\t" );
             for( i=0; i<SHA_DIGEST_LENGTH; ++i )
                 printf( "%02x", info.hash[i] );
@@ -185,15 +288,12 @@ main( int argc, char ** argv )
 
             printf( "name:\t%s\n", info.name );
 
-            for( i=0; i<info.trackerTiers; ++i ) {
-                int j;
-                printf( "tracker tier #%d:\n", ( i+1 ) );
-                for( j=0; j<info.trackerList[i].count; ++j ) {
-                    const tr_tracker_info * tracker = &info.trackerList[i].list[j];
-                    printf( "\taddress:\t%s:%d\n", tracker->address, tracker->port );
-                    printf( "\tannounce:\t%s\n", tracker->announce );
-                    printf( "\n" );
+            for( i=0; i<info.trackerCount; ++i ) {
+                if( prevTier != info.trackers[i].tier ) {
+                    prevTier = info.trackers[i].tier;
+                    printf( "\ntracker tier #%d:\n", (prevTier+1) );
                 }
+                printf( "\tannounce:\t%s\n", info.trackers[i].announce );
             }
 
             printf( "size:\t%"PRIu64" (%"PRIu64" * %d + %"PRIu64")\n",
@@ -208,8 +308,8 @@ main( int argc, char ** argv )
                 printf( "private flag set\n" );
 
             printf( "file(s):\n" );
-            for( i=0; i<info.fileCount; ++i )
-                printf( "\t%s (%"PRIu64")\n", info.files[i].name, info.files[i].length );
+            for( ff=0; ff<info.fileCount; ++ff )
+                printf( "\t%s (%"PRIu64")\n", info.files[ff].name, info.files[ff].length );
 
             tr_metainfoFree( &info );
         }
@@ -228,31 +328,6 @@ main( int argc, char ** argv )
         return EXIT_FAILURE;
     }
 
-    if( showScrape )
-    {
-        printf( "Scraping, Please wait...\n" );
-        const tr_stat * stats;
-        
-        uint64_t start = tr_date();
-        
-        do
-        {
-            stats = tr_torrentStat( tor );
-            if( stats == NULL || tr_date() - start > 20000 )
-            {
-                printf( "Scrape failed.\n" );
-                goto cleanup;
-            }
-            tr_wait( 2000 );
-        }
-        while( stats->completedFromTracker == -1 || stats->leechers == -1 || stats->seeders == -1 );
-        
-        printf( "%d seeder(s), %d leecher(s), %d download(s).\n",
-            stats->seeders, stats->leechers, stats->completedFromTracker );
-
-        goto cleanup;
-    }
-
     signal( SIGINT, sigHandler );
     signal( SIGHUP, sigHandler );
 
@@ -263,7 +338,7 @@ main( int argc, char ** argv )
     {
         char string[LINEWIDTH];
         int  chars = 0;
-        const tr_stat * s;
+        const struct tr_stat * s;
 
         tr_wait( 1000 );
 
@@ -288,7 +363,7 @@ main( int argc, char ** argv )
         if( recheckData )
         {
             recheckData = 0;
-            tr_torrentRecheck( tor );
+            tr_torrentVerify( tor );
         }
 
         s = tr_torrentStat( tor );
@@ -367,83 +442,53 @@ parseCommandLine( int argc, char ** argv )
 {
     for( ;; )
     {
-        static struct option long_options[] =
-          { { "help",     no_argument,          NULL, 'h' },
-            { "info",     no_argument,          NULL, 'i' },
-            { "scrape",   no_argument,          NULL, 's' },
-            { "private",  no_argument,          NULL, 'r' },
-            { "version",  no_argument,          NULL, 'V' },
-            { "verbose",  required_argument,    NULL, 'v' },
-            { "port",     required_argument,    NULL, 'p' },
-            { "upload",   required_argument,    NULL, 'u' },
-            { "download", required_argument,    NULL, 'd' },
-            { "finish",   required_argument,    NULL, 'f' },
-            { "create-from", required_argument, NULL, 'c' },
-            { "comment",  required_argument,    NULL, 'm' },
-            { "announce", required_argument,    NULL, 'a' },
-            { "nat-traversal", no_argument,     NULL, 'n' },
-            { "recheck",  no_argument,          NULL, 'y' },
-            { "output-dir", required_argument,  NULL, 'o' },
+        static const struct option long_options[] = {
+            { "announce",      required_argument, NULL, 'a' },
+            { "create-from",   required_argument, NULL, 'c' },
+            { "download",      required_argument, NULL, 'd' },
+            { "finish",        required_argument, NULL, 'f' },
+            { "config-dir",    required_argument, NULL, 'g' },
+            { "help",          no_argument,       NULL, 'h' },
+            { "info",          no_argument,       NULL, 'i' },
+            { "comment",       required_argument, NULL, 'm' },
+            { "nat-traversal", no_argument,       NULL, 'n' },
+            { "output-dir",    required_argument, NULL, 'o' },
+            { "port",          required_argument, NULL, 'p' },
+            { "private",       no_argument,       NULL, 'r' },
+            { "scrape",        no_argument,       NULL, 's' },
+            { "upload",        required_argument, NULL, 'u' },
+            { "verbose",       required_argument, NULL, 'v' },
+            { "version",       no_argument,       NULL, 'V' },
+            { "recheck",       no_argument,       NULL, 'y' },
             { 0, 0, 0, 0} };
-
-        int c, optind = 0;
-        c = getopt_long( argc, argv, "hisrVv:p:u:d:f:c:m:a:no:y",
-                         long_options, &optind );
+        int optind = 0;
+        int c = getopt_long( argc, argv,
+                             "a:c:d:f:g:him:no:p:rsu:v:Vy",
+                             long_options, &optind );
         if( c < 0 )
         {
             break;
         }
         switch( c )
         {
-            case 'h':
-                showHelp = 1;
-                break;
-            case 'i':
-                showInfo = 1;
-                break;
-            case 's':
-                showScrape = 1;
-                break;
-            case 'r':
-                isPrivate = 1;
-                break;
-            case 'v':
-                verboseLevel = atoi( optarg );
-                break;
-            case 'V':
-                showVersion = 1;
-                break;
-            case 'p':
-                bindPort = atoi( optarg );
-                break;
-            case 'u':
-                uploadLimit = atoi( optarg );
-                break;
-            case 'd':
-                downloadLimit = atoi( optarg );
-                break;
-            case 'f':
-                finishCall = optarg;
-                break;
-            case 'c':
-                sourceFile = optarg;
-                break;
-            case 'm':
-                comment = optarg;
-                break;
-            case 'a':
-                announce = optarg;
-                break;
-            case 'n':
-                natTraversal = 1;
-                break;
-            case 'y':
-                recheckData = 1;
-                break;
-            case 'o':
-                savePath = optarg;
-            default:
-                return 1;
+            case 'a': announce = optarg; break;
+            case 'c': sourceFile = optarg; break;
+            case 'd': downloadLimit = atoi( optarg ); break;
+            case 'f': finishCall = optarg; break;
+            case 'g': configdir = strdup( optarg ); break;
+            case 'h': showHelp = 1; break;
+            case 'i': showInfo = 1; break;
+            case 'm': comment = optarg; break;
+            case 'n': natTraversal = 1; break;
+            case 'o': savePath = optarg;
+            case 'p': bindPort = atoi( optarg ); break;
+            case 'r': isPrivate = 1; break;
+            case 's': showScrape = 1; break;
+            case 'u': uploadLimit = atoi( optarg ); break;
+            case 'v': verboseLevel = atoi( optarg ); break;
+            case 'V': showVersion = 1; break;
+            case 'y': recheckData = 1; break;
+            default: return 1;
         }
     }
 
