@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: client.c 3578 2007-10-26 03:43:27Z joshe $
+ * $Id: client.c 5646 2008-04-19 00:41:32Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -35,11 +35,12 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <libtransmission/bsdtree.h>
 #include <libtransmission/bencode.h>
 #include <libtransmission/ipcparse.h>
 #include <libtransmission/trcompat.h>
+#include <libtransmission/version.h>
 
+#include "bsdtree.h"
 #include "bsdqueue.h"
 #include "client.h"
 #include "errors.h"
@@ -121,12 +122,8 @@ client_init( struct event_base * base )
         return -1;
     }
 
-    if( 0 > ipc_addmsg( gl_tree, IPC_MSG_INFO, infomsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_STAT, statmsg ) )
-    {
-        return -1;
-    }
-
+    ipc_addmsg( gl_tree, IPC_MSG_INFO, infomsg );
+    ipc_addmsg( gl_tree, IPC_MSG_STAT, statmsg );
     ipc_setdefmsg( gl_tree, defmsg );
 
     return 0;
@@ -139,9 +136,9 @@ client_new_sock( const char * path )
     int                fd;
     struct con       * con;
 
-    assert( NULL != gl_base );
+    assert( gl_base );
+    assert( path );
     assert( 0 > gl_proxy );
-    assert( NULL != path );
 
     gl_proxy = 0;
 
@@ -172,7 +169,6 @@ client_new_sock( const char * path )
     con->ipc = ipc_newcon( gl_tree );
     if( NULL == con->ipc )
     {
-        mallocmsg( sizeof *con->ipc );
         close( fd );
         free( con );
     }
@@ -295,7 +291,6 @@ client_new_cmd( char * const * cmd )
     con->ipc = ipc_newcon( gl_tree );
     if( NULL == con->ipc )
     {
-        mallocmsg( sizeof *con->ipc );
         bufferevent_free( con->evin );
         bufferevent_free( con->evout );
         free( con );
@@ -560,33 +555,30 @@ addintlistreq( enum ipc_msg which, size_t len, const int * list )
 int
 client_start( size_t len, const int * list )
 {
-    enum ipc_msg id;
-
-    id = ( NULL == list ? IPC_MSG_STARTALL : IPC_MSG_START );
-
+    const enum ipc_msg id = list ? IPC_MSG_START : IPC_MSG_STARTALL;
     return addintlistreq( id, len, list );
 }
 
 int
 client_stop( size_t len, const int * list )
 {
-    enum ipc_msg id;
-
-    id = ( NULL == list ? IPC_MSG_STOPALL : IPC_MSG_STOP );
-
+    const enum ipc_msg id = list ? IPC_MSG_STOP : IPC_MSG_STOPALL;
     return addintlistreq( id, len, list );
 }
 
 int
 client_remove( size_t len, const int * list )
 {
-    enum ipc_msg id;
-
-    id = ( NULL == list ? IPC_MSG_REMOVEALL : IPC_MSG_REMOVE );
-
+    const enum ipc_msg id = list ? IPC_MSG_REMOVE : IPC_MSG_REMOVEALL;
     return addintlistreq( id, len, list );
 }
 
+int
+client_verify( size_t len, const int * list )
+{
+    return list ? addintlistreq( IPC_MSG_VERIFY, len, list ) : -1;
+}
+
 int
 client_list( cl_infofunc func )
 {
@@ -731,7 +723,7 @@ canread( struct bufferevent * evin, void * arg )
         return;
     }
 
-    res = ipc_parse( con->ipc, buf, len, con );
+    res = ipc_handleMessages( con->ipc, buf, len, con );
     if( 0 > res )
     {
         switch( errno )
@@ -765,7 +757,7 @@ flushreqs( struct con * con )
     benc_val_t       pk, * val;
     struct stritem * jj;
 
-    if( !HASVERS( con->ipc ) )
+    if( !ipc_hasvers( con->ipc ) )
     {
         return;
     }
@@ -802,7 +794,7 @@ flushreqs( struct con * con )
                         tr_bencInitStr( tr_bencListAdd( val ),
                                         jj->str, -1, 1 );
                     }
-                    buf = ipc_mkval( &pk, &buflen );
+                    buf = ipc_serialize( &pk, &buflen );
                     SAFEBENCFREE( &pk );
                 }
                 SAFEFREESTRLIST( req->strs );
@@ -813,7 +805,7 @@ flushreqs( struct con * con )
                 {
                     tr_bencInitStr( tr_bencDictAdd( val, "data" ),
                                     req->buf, req->listlen, 1 );
-                    buf = ipc_mkval( &pk, &buflen );
+                    buf = ipc_serialize( &pk, &buflen );
                     SAFEBENCFREE( &pk );
                 }
                 SAFEFREE( req->buf );
@@ -833,6 +825,7 @@ flushreqs( struct con * con )
             case IPC_MSG_START:
             case IPC_MSG_STOP:
             case IPC_MSG_REMOVE:
+            case IPC_MSG_VERIFY:
                 val = ipc_initval( con->ipc, req->id, -1, &pk, TYPE_LIST );
                 if( NULL != val && !tr_bencListReserve( val, req->listlen ) )
                 {
@@ -841,15 +834,15 @@ flushreqs( struct con * con )
                         tr_bencInitInt( tr_bencListAdd( val ),
                                         req->numlist[ii] );
                     }
-                    buf = ipc_mkval( &pk, &buflen );
+                    buf = ipc_serialize( &pk, &buflen );
                     SAFEBENCFREE( &pk );
                 }
                 SAFEFREE( req->numlist );
                 break;
             case IPC_MSG_GETINFOALL:
             case IPC_MSG_GETSTATALL:
-                buf = ipc_mkgetinfo( con->ipc, &buflen, req->id, req->tag,
-                                     req->types, NULL );
+                buf = ipc_createInfoRequest( con->ipc, &buflen, req->id, req->tag,
+                                             req->types, NULL );
                 break;
             default:
                 assert( 0 );
@@ -914,7 +907,6 @@ void
 infomsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
          void * arg UNUSED )
 {
-    benc_val_t   * dict;
     int            ii;
     struct cl_info inf;
     int64_t        id;
@@ -922,10 +914,8 @@ infomsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
 
     assert( IPC_MSG_INFO == msgid );
 
-    if( TYPE_LIST != list->type )
-    {
+    if( !tr_bencIsList( list ) )
         return;
-    }
 
     memset( &key, 0, sizeof key );
     key.tag = tag;
@@ -938,11 +928,10 @@ infomsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
 
     for( ii = 0; list->val.l.count > ii; ii++ )
     {
-        dict = &list->val.l.vals[ii];
-        if( TYPE_DICT != dict->type )
-        {
+        tr_benc * dict = &list->val.l.vals[ii];
+
+        if( !tr_bencIsDict( dict ) )
             continue;
-        }
 
         id       = getinfoint( msgid, dict, IPC_INF_ID,   -1   );
         inf.name = getinfostr( msgid, dict, IPC_INF_NAME, NULL );
@@ -966,7 +955,6 @@ void
 statmsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
          void * arg UNUSED )
 {
-    benc_val_t   * dict;
     int            ii;
     int64_t        id;
     struct cl_stat st;
@@ -974,10 +962,8 @@ statmsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
 
     assert( IPC_MSG_STAT == msgid );
 
-    if( TYPE_LIST != list->type )
-    {
+    if( !tr_bencIsList( list ) )
         return;
-    }
 
     memset( &key, 0, sizeof key );
     key.tag = tag;
@@ -990,11 +976,10 @@ statmsg( enum ipc_msg msgid, benc_val_t * list, int64_t tag,
 
     for( ii = 0; list->val.l.count > ii; ii++ )
     {
-        dict   = &list->val.l.vals[ii];
-        if( TYPE_DICT != dict->type )
-        {
+        tr_benc * dict = &list->val.l.vals[ii];
+
+        if( !tr_bencIsDict( dict ) )
             continue;
-        }
 
         id           = getinfoint( msgid, dict, IPC_ST_ID,        -1   );
         st.state     = getinfostr( msgid, dict, IPC_ST_STATE,     NULL );
@@ -1060,11 +1045,11 @@ defmsg( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg UNUSED )
 void
 cbdone( struct resp * resp )
 {
-    if( NULL != resp->infocb )
+    if( resp->infocb )
     {
         resp->infocb( NULL );
     }
-    else if( NULL != resp->statcb )
+    else if( resp->statcb )
     {
         resp->statcb( NULL );
     }
@@ -1073,29 +1058,15 @@ cbdone( struct resp * resp )
 int64_t
 getinfoint( enum ipc_msg msgid, benc_val_t * dict, int type, int64_t defval  )
 {
-    benc_val_t * val;
-
-    val = tr_bencDictFind( dict, ipc_infoname( msgid, type ) );
-
-    if( NULL != val && TYPE_INT == val->type )
-    {
-        return val->val.i;
-    }
-
-    return defval;
+    const char * key = ipc_infoname( msgid, type );
+    benc_val_t * val = tr_bencDictFind( dict, key );
+    return tr_bencIsInt( val ) ? val->val.i : defval;
 }
 
 char *
 getinfostr( enum ipc_msg msgid, benc_val_t * dict, int type, char * defval  )
 {
-    benc_val_t * val;
-
-    val = tr_bencDictFind( dict, ipc_infoname( msgid, type ) );
-
-    if( NULL != val && TYPE_STR == val->type )
-    {
-        return val->val.s.s ;
-    }
-
-    return defval;
+    const char * key = ipc_infoname( msgid, type );
+    benc_val_t * val = tr_bencDictFind( dict, key );
+    return tr_bencIsString( val ) ? val->val.s.s : defval;
 }
