@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: daemon.c 4390 2007-12-31 01:19:35Z livings124 $
+ * $Id: daemon.c 5758 2008-05-06 19:02:05Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -32,7 +32,6 @@
 #include <assert.h>
 #include <errno.h>
 #include <event.h>
-#include <fcntl.h>
 #include <getopt.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -42,6 +41,8 @@
 #include <unistd.h>
 
 #include <libtransmission/trcompat.h>
+#include <libtransmission/platform.h>
+#include <libtransmission/utils.h>
 #include <libtransmission/version.h>
 
 #include "errors.h"
@@ -50,16 +51,14 @@
 #include "torrents.h"
 
 static void usage       ( const char *, ... );
-static void readargs    ( int, char **, int *, int *, char **, char ** );
-static int  trylocksock ( const char * );
-static int  getlock     ( const char * );
+static void readargs    ( int, char **, int *, int *, char **, char **, char ** );
+static int  trylocksock ( const char * configdir, const char * sockpath );
 static int  getsock     ( const char * );
 static void exitcleanup ( void );
 static void setupsigs   ( struct event_base * );
 static void gotsig      ( int, short, void * );
 static int  savepid     ( const char * );
 
-static int  gl_lockfd               = -1;
 static char gl_lockpath[MAXPATHLEN] = "";
 static int  gl_sockfd               = -1;
 static char gl_sockpath[MAXPATHLEN] = "";
@@ -70,23 +69,24 @@ main( int argc, char ** argv )
 {
     struct event_base * evbase;
     int                 nofork, debug, sockfd;
-    char              * sockpath, * pidfile;
+    char              * configdir, * sockpath, * pidfile;
 
     setmyname( argv[0] );
-    readargs( argc, argv, &nofork, &debug, &sockpath, &pidfile );
+    readargs( argc, argv, &nofork, &debug, &configdir, &sockpath, &pidfile );
 
-    if( !nofork )
-    {
-        if( 0 > daemon( 1, 0 ) )
-        {
+    if( !nofork ) {
+        if( 0 > daemon( 1, 0 ) ) {
             errnomsg( "failed to daemonize" );
             exit( 1 );
         }
         errsyslog( 1 );
     }
 
+    if( configdir == NULL )
+        configdir = (char*) tr_getDefaultConfigDir( );
+
     atexit( exitcleanup );
-    sockfd = trylocksock( sockpath );
+    sockfd = trylocksock( configdir, sockpath );
     if( 0 > sockfd )
     {
         exit( 1 );
@@ -98,7 +98,7 @@ main( int argc, char ** argv )
 
     evbase = event_init();
     setupsigs( evbase );
-    torrent_init( evbase );
+    torrent_init( configdir, evbase );
     server_init( evbase );
     server_debug( debug );
     server_listen( sockfd );
@@ -130,6 +130,7 @@ usage( const char * msg, ... )
   "\n"
   "  -d --debug                Print data send and received, implies -f\n"
   "  -f --foreground           Run in the foreground and log to stderr\n"
+  "  -g --config-dir <path>    Where to look for configuration files\n"
   "  -h --help                 Display this message and exit\n"
   "  -p --pidfile <path>       Save the process id in a file at <path>\n"
   "  -s --socket <path>        Place the socket file at <path>\n"
@@ -140,14 +141,15 @@ usage( const char * msg, ... )
 }
 
 void
-readargs( int argc, char ** argv, int * nofork, int * debug, char ** sock,
-          char ** pidfile )
+readargs( int argc, char ** argv, int * nofork, int * debug,
+          char ** configdir, char ** sock, char ** pidfile )
 {
-    char optstr[] = "dfhp:s:";
+    char optstr[] = "dfg:hp:s:";
     struct option longopts[] =
     {
         { "debug",              no_argument,       NULL, 'd' },
         { "foreground",         no_argument,       NULL, 'f' },
+        { "config-dir",         required_argument, NULL, 'g' },
         { "help",               no_argument,       NULL, 'h' },
         { "pidfile",            required_argument, NULL, 'p' },
         { "socket",             required_argument, NULL, 's' },
@@ -159,55 +161,63 @@ readargs( int argc, char ** argv, int * nofork, int * debug, char ** sock,
     *debug     = 0;
     *sock      = NULL;
     *pidfile   = NULL;
+    *configdir = NULL;
 
-    while( 0 <= ( opt = getopt_long( argc, argv, optstr, longopts, NULL ) ) )
-    {
-        switch( opt )
-        {
-            case 'd':
-                *debug = 1;
-                /* FALLTHROUGH */
-            case 'f':
-                *nofork = 1;
-                break;
-            case 'p':
-                *pidfile = optarg;
-                break;
-            case 's':
-                *sock   = optarg;
-                break;
-            default:
-                usage( NULL );
-                break;
+    while( 0 <= ( opt = getopt_long( argc, argv, optstr, longopts, NULL ) ) ) {
+        switch( opt ) {
+            case 'd': *debug = 1; /* FALLTHROUGH */
+            case 'f': *nofork = 1; break;
+            case 'g': *configdir = strdup( optarg ); break;
+            case 'p': *pidfile = optarg; break;
+            case 's': *sock   = optarg; break;
+            default: usage( NULL ); break;
         }
     }
 }
 
-int
-trylocksock( const char * sockpath )
+static int
+getlock( const char * filename )
 {
-    char path[MAXPATHLEN];
-    int  fd;
+    const int state = tr_lockfile( filename );
+    const int success = state == TR_LOCKFILE_SUCCESS;
 
-    confpath( path, sizeof path, NULL, CONF_PATH_TYPE_DAEMON );
-    if( 0 > mkdir( path, 0777 ) && EEXIST != errno )
+    if( !success ) switch( state ) {
+        case TR_LOCKFILE_EOPEN:
+            errnomsg( "failed to open file: %s", filename );
+            break;
+        case TR_LOCKFILE_ELOCK:
+            errmsg( "another copy of %s is already running", getmyname() );
+            break;
+        default:
+            errmsg( "unhandled tr_lockfile error: %d", state );
+            break;
+    }
+
+    return success;
+}
+
+
+int
+trylocksock( const char * configdir, const char * sockpath )
+{
+    int  fd;
+    char path[MAXPATHLEN];
+
+    confpath( path, sizeof path, configdir, NULL, CONF_PATH_TYPE_DAEMON );
+    if( tr_mkdirp( path, 0777 ) && EEXIST != errno )
     {
         errnomsg( "failed to create directory: %s", path );
         return -1;
     }
 
-    confpath( path, sizeof path, CONF_FILE_LOCK, 0 );
-    fd = getlock( path );
-    if( 0 > fd )
-    {
+    confpath( path, sizeof path, configdir, CONF_FILE_LOCK, 0 );
+    if( !getlock( path ) )
         return -1;
-    }
-    gl_lockfd = fd;
     strlcpy( gl_lockpath, path, sizeof gl_lockpath );
 
-    if( NULL == sockpath )
+    if( !sockpath )
     {
-        confpath( path, sizeof path, CONF_FILE_SOCKET, 0 );
+        confpath( path, sizeof path, configdir, CONF_FILE_SOCKET, 0 );
         sockpath = path;
     }
     fd = getsock( sockpath );
@@ -221,46 +231,6 @@ trylocksock( const char * sockpath )
     return fd;
 }
 
-int
-getlock( const char * path )
-{
-    struct flock lk;
-    int          fd;
-    char         pid[64];
-
-    fd = open( path, O_RDWR | O_CREAT, 0666 );
-    if( 0 > fd )
-    {
-        errnomsg( "failed to open file: %s", path );
-        return -1;
-    }
-
-    memset( &lk, 0, sizeof lk );
-    lk.l_start  = 0;
-    lk.l_len    = 0;
-    lk.l_type   = F_WRLCK;
-    lk.l_whence = SEEK_SET;
-    if( 0 > fcntl( fd, F_SETLK, &lk ) )
-    {
-        if( EAGAIN == errno )
-        {
-            errmsg( "another copy of %s is already running", getmyname() );
-        }
-        else
-        {
-            errnomsg( "failed to obtain lock on file: %s", path );
-        }
-        close( fd );
-        return -1;
-    }
-
-    ftruncate( fd, 0 );
-    snprintf( pid, sizeof pid, "%i\n", getpid() );
-    write( fd, pid, strlen( pid ) );
-
-    return fd;
-}
-
 int
 getsock( const char * path )
 {
@@ -305,11 +275,9 @@ exitcleanup( void )
     {
         unlink( gl_pidfile );
     }
-    if( 0 <= gl_lockfd )
-    {
+
+    if( *gl_lockpath )
         unlink( gl_lockpath );
-        close( gl_lockfd );
-    }
 }
 
 void
