@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: misc.c 2514 2007-07-27 02:25:28Z charles $
+ * $Id: misc.c 5721 2008-04-29 03:36:14Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -27,6 +27,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <libgen.h> /* basename */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,25 +35,19 @@
 
 #include <libtransmission/transmission.h>
 #include <libtransmission/trcompat.h>
+#include <libtransmission/utils.h> /* tr_loadFile() */
 
 #include "errors.h"
 #include "misc.h"
 
-static void pushdir ( char *, const char *, size_t );
-
 static char gl_myname[256];
 
 void
 setmyname( const char * argv0 )
 {
-    const char * name;
-
-    name = strrchr( argv0, '/' );
-    if( NULL == name || '\0' == *(++name) )
-    {
-        name = argv0;
-    }
-    strlcpy( gl_myname, name, sizeof gl_myname );
+    char tmp[MAX_PATH_LENGTH];
+    strlcpy( tmp, argv0, sizeof( tmp ) );
+    strlcpy( gl_myname, basename( tmp ), sizeof( gl_myname ) );
 }
 
 const char *
@@ -62,65 +57,39 @@ getmyname( void )
 }
 
 void
-pushdir( char * path, const char * file, size_t size )
+confpath( char               * buf,
+          size_t               len,
+          const char         * configDir,
+          const char         * file,
+          enum confpathtype    type )
 {
-    size_t off;
-
-    off = strlen( path );
-    if( 0 < off && off + 1 < size && '/' != path[off-1] )
-    {
-        path[off]   = '/';
-        path[off+1] = '\0';
-    }
-    strlcat( path, file, size );
-}
-
-void
-confpath( char * buf, size_t len, const char * file, enum confpathtype type )
-{
-    strlcpy( buf, tr_getPrefsDirectory(), len );
-
     switch( type )
     {
         case CONF_PATH_TYPE_DAEMON:
-            pushdir( buf, "daemon", len );
+            tr_buildPath( buf, len, configDir, "daemon", file, NULL );
             break;
         case CONF_PATH_TYPE_GTK:
-            pushdir( buf, "gtk", len );
+            tr_buildPath( buf, len, configDir, "gtk", file, NULL );
             break;
         case CONF_PATH_TYPE_OSX:
+            tr_buildPath( buf, len, configDir, file, NULL );
             break;
         default:
             assert( 0 );
             break;
     }
-
-    if( NULL != file )
-    {
-        pushdir( buf, file, len );
-    }
 }
 
 void
 absolutify( char * buf, size_t len, const char * path )
 {
-    size_t off;
-
-    if( '/' == path[0] )
-    {
+    if( *path == '/' )
         strlcpy( buf, path, len );
-        return;
+    else {
+        char cwd[MAX_PATH_LENGTH];
+        getcwd( cwd, sizeof( cwd ) );
+        tr_buildPath( buf, len, cwd, path, NULL );
     }
-
-    getcwd( buf, len );
-    off = strlen( buf );
-    if( 0 < off && len > off + 1 && '/' != buf[off] )
-    {
-        buf[off] = '/';
-        off++;
-        buf[off] = '\0';
-    }
-    strlcat( buf, path, len );
 }
 
 int
@@ -132,7 +101,7 @@ writefile( const char * name, uint8_t * buf, ssize_t len )
     fd = open( name, O_WRONLY | O_CREAT | O_TRUNC, 0666 );
     if( 0 > fd )
     {
-        errnomsg( "failed to open %s for writing", name );
+        errnomsg( "Couldn't open \"%1$s\": %2$s", name, tr_strerror( errno ) );
         return -1;
     }
 
@@ -154,54 +123,9 @@ writefile( const char * name, uint8_t * buf, ssize_t len )
 }
 
 uint8_t *
-readfile( const char * name, size_t * len )
+readfile( const char * filename, size_t * len )
 {
-    struct stat sb;
-    int         fd;
-    uint8_t   * buf;
-    ssize_t     res;
-
-    fd = open( name, O_RDONLY );
-    if( 0 > fd )
-    {
-        if( ENOENT != errno )
-        {
-            errnomsg( "failed to open %s for reading", name );
-        }
-        return NULL;
-    }
-
-    if( 0 > fstat( fd, &sb ) )
-    {
-        errnomsg( "failed to stat %s", name );
-        return NULL;
-    }
-
-    buf = malloc( sb.st_size );
-    if( NULL == buf )
-    {
-        mallocmsg( sb.st_size );
-        return NULL;
-    }
-
-    res = read( fd, buf, sb.st_size );
-    if( 0 > res )
-    {
-        errnomsg( "failed to read from %s", name );
-        free( buf );
-        return NULL;
-    }
-    if( res < sb.st_size )
-    {
-        errmsg( "failed to read all data from %s", name );
-        free( buf );
-        return NULL;
-    }
-
-    close( fd );
-    *len = res;
-
-    return buf;
+    return tr_loadFile( filename, len );
 }
 
 #ifndef HAVE_DAEMON
