@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: server.c 4305 2007-12-24 03:48:42Z livings124 $
+ * $Id: server.c 5766 2008-05-07 11:57:23Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -36,10 +36,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <libtransmission/bsdtree.h>
 #include <libtransmission/bencode.h>
 #include <libtransmission/ipcparse.h>
+#include <libtransmission/utils.h> /* tr_free */
 
+#include "bsdtree.h"
 #include "errors.h"
 #include "misc.h"
 #include "server.h"
@@ -100,41 +101,39 @@ server_init( struct event_base * base )
         return -1;
     }
 
-    if( 0 > ipc_addmsg( gl_tree, IPC_MSG_ADDMANYFILES, addmsg1 ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_ADDONEFILE,   addmsg2 ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_AUTOMAP,      intmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_AUTOSTART,    intmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_CRYPTO,       strmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_DOWNLIMIT,    intmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_DIR,          strmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETAUTOMAP,   prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETAUTOSTART, prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETCRYPTO,    prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETDOWNLIMIT, prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETDIR,       prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETINFO,      infomsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETINFOALL,   infomsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETPEX,       prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETPORT,      prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETSTAT,      infomsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETSTATALL,   infomsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETUPLIMIT,   prefmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_GETSUP,       supmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_LOOKUP,       lookmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_NOOP,         noopmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_PEX,          intmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_PORT,         intmsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_QUIT,         quitmsg ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_REMOVE,       tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_REMOVEALL,    tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_START,        tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_STARTALL,     tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_STOP,         tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_STOPALL,      tormsg  ) ||
-        0 > ipc_addmsg( gl_tree, IPC_MSG_UPLIMIT,      intmsg  ) )
-    {
-        return -1;
-    }
+    ipc_addmsg( gl_tree, IPC_MSG_ADDMANYFILES, addmsg1 );
+    ipc_addmsg( gl_tree, IPC_MSG_ADDONEFILE,   addmsg2 );
+    ipc_addmsg( gl_tree, IPC_MSG_AUTOMAP,      intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_AUTOSTART,    intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_CRYPTO,       strmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_DOWNLIMIT,    intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_DIR,          strmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_GETAUTOMAP,   prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETAUTOSTART, prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETCRYPTO,    prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETDOWNLIMIT, prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETDIR,       prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETINFO,      infomsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETINFOALL,   infomsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETPEX,       prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETPORT,      prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETSTAT,      infomsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETSTATALL,   infomsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETUPLIMIT,   prefmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_GETSUP,       supmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_LOOKUP,       lookmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_NOOP,         noopmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_PEX,          intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_PORT,         intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_QUIT,         quitmsg );
+    ipc_addmsg( gl_tree, IPC_MSG_REMOVE,       tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_REMOVEALL,    tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_START,        tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_STARTALL,     tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_STOP,         tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_STOPALL,      tormsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_UPLIMIT,      intmsg  );
+    ipc_addmsg( gl_tree, IPC_MSG_VERIFY,       tormsg  );
 
     ipc_setdefmsg( gl_tree, defmsg );
 
@@ -247,7 +246,6 @@ newclient( int fd, short event UNUSED, void * arg )
         client->ipc = ipc_newcon( gl_tree );
         if( NULL == client->ipc )
         {
-            mallocmsg( sizeof *client->ipc );
             close( clfd );
             free( client );
             return;
@@ -361,7 +359,7 @@ doread( struct bufferevent * ev, void * arg )
         return;
     }
 
-    res = ipc_parse( client->ipc, buf, len, client );
+    res = ipc_handleMessages( client->ipc, buf, len, client );
 
     if( gl_exiting )
     {
@@ -390,7 +388,7 @@ doread( struct bufferevent * ev, void * arg )
     }
 }
 
-int
+static int
 queuemsg( struct client * client, uint8_t * buf, size_t buflen )
 {
     if( NULL == buf )
@@ -419,6 +417,16 @@ queuemsg( struct client * client, uint8_t * buf, size_t buflen )
     return 0;
 }
 
+static int
+queuepkmsg( struct client * client, tr_benc * pk )
+{
+    size_t buflen;
+    uint8_t * buf = ipc_serialize( pk, &buflen );
+    int ret = queuemsg( client, buf, buflen );
+    tr_free( buf );
+    return ret;
+}
+
 int
 msgresp( struct client * client, int64_t tag, enum ipc_msg id )
 {
@@ -460,12 +468,10 @@ void
 addmsg1( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 {
     struct client * client = arg;
-    benc_val_t      pk, * added, * file;
+    benc_val_t      pk, * added;
     int             ii, tor;
-    size_t          buflen;
-    uint8_t       * buf;
 
-    if( NULL == val || TYPE_LIST != val->type )
+    if( !tr_bencIsList( val ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
@@ -481,17 +487,15 @@ addmsg1( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 
     for( ii = 0; ii < val->val.l.count; ii++ )
     {
-        file = &val->val.l.vals[ii];
-        if( TYPE_STR != file->type )
-        {
+        tr_benc * file = &val->val.l.vals[ii];
+        if( !tr_bencIsString( file ) )
             continue;
-        }
+
         /* XXX need to somehow inform client of skipped or failed files */
         tor = torrent_add_file( file->val.s.s, NULL, -1 );
         if( TORRENT_ID_VALID( tor ) )
         {
-            const tr_info * inf = torrent_info( tor );
-            if( 0 > ipc_addinfo( added, tor, inf, 0 ) )
+            if( 0 > ipc_addinfo( added, tor, torrent_handle( tor ), 0 ) )
             {
                 errnomsg( "failed to build message" );
                 tr_bencFree( &pk );
@@ -501,10 +505,8 @@ addmsg1( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
         }
     }
 
-    buf = ipc_mkval( &pk, &buflen );
+    queuepkmsg( client, &pk );
     tr_bencFree( &pk );
-    queuemsg( client, buf, buflen );
-    free( buf );
 }
 
 void
@@ -513,23 +515,20 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
     struct client * client = arg;
     benc_val_t    * val, pk;
     int             tor, start;
-    size_t          buflen;
-    uint8_t       * buf;
     const char    * dir;
 
-    if( NULL == dict || TYPE_DICT != dict->type )
+    if( !tr_bencIsDict( dict ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
     }
 
     val   = tr_bencDictFind( dict, "directory" );
-    dir   = ( NULL == val || TYPE_STR != val->type ? NULL : val->val.s.s );
+    dir   = tr_bencIsString( val ) ? val->val.s.s : NULL;
     val   = tr_bencDictFind( dict, "autostart" );
-    start = ( NULL == val || TYPE_INT != val->type ? -1 :
-              ( val->val.i ? 1 : 0 ) );
+    start = tr_bencIsInt( val ) ? (val->val.i!=0) : -1;
     val   = tr_bencDictFind( dict, "data" );
-    if( NULL != val && TYPE_STR == val->type )
+    if( tr_bencIsString( val ) )
     {
         /* XXX detect duplicates and return a message indicating so */
         tor = torrent_add_data( ( uint8_t * )val->val.s.s, val->val.s.i,
@@ -538,7 +537,7 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
     else
     {
         val = tr_bencDictFind( dict, "file" );
-        if( NULL == val || TYPE_STR != val->type )
+        if( !tr_bencIsString( val ) )
         {
             msgresp( client, tag, IPC_MSG_BAD );
             return;
@@ -549,7 +548,6 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
 
     if( TORRENT_ID_VALID( tor ) )
     {
-        const tr_info * inf;
         val = ipc_initval( client->ipc, IPC_MSG_INFO, tag, &pk, TYPE_LIST );
         if( NULL == val )
         {
@@ -557,18 +555,16 @@ addmsg2( enum ipc_msg id UNUSED, benc_val_t * dict, int64_t tag, void * arg )
             byebye( client->ev, EVBUFFER_EOF, NULL );
             return;
         }
-        inf = torrent_info( tor );
-        if( 0 > ipc_addinfo( val, tor, inf, 0 ) )
+        if( 0 > ipc_addinfo( val, tor, torrent_handle( tor ), 0 ) )
         {
             errnomsg( "failed to build message" );
             tr_bencFree( &pk );
             byebye( client->ev, EVBUFFER_EOF, NULL );
             return;
         }
-        buf = ipc_mkval( &pk, &buflen );
+
+        queuepkmsg( client, &pk );
         tr_bencFree( &pk );
-        queuemsg( client, buf, buflen );
-        free( buf );
     }
     else
     {
@@ -589,7 +585,7 @@ intmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     struct client * client = arg;
     int             num;
 
-    if( NULL == val || TYPE_INT != val->type )
+    if( !tr_bencIsInt( val ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
@@ -629,7 +625,7 @@ strmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 {
     struct client * client = arg;
 
-    if( NULL == val || TYPE_STR != val->type )
+    if( !tr_bencIsString( val ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
@@ -638,18 +634,18 @@ strmsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     switch( id )
     {
         case IPC_MSG_CRYPTO:
-            if(!strcasecmp(val->val.s.s, "plaintext"))
-                torrent_set_encryption(TR_PLAINTEXT_PREFERRED);
-            else if(!strcasecmp(val->val.s.s, "preferred"))
-                torrent_set_encryption(TR_ENCRYPTION_PREFERRED);
-            else if(!strcasecmp(val->val.s.s, "required"))
-                torrent_set_encryption(TR_ENCRYPTION_REQUIRED);
-            else
-            {
+            if( !strcasecmp( val->val.s.s, "required" ) )
+                torrent_set_encryption( TR_ENCRYPTION_REQUIRED );
+            else if( !strcasecmp( val->val.s.s, "preferred" ) )
+                torrent_set_encryption( TR_ENCRYPTION_PREFERRED );
+            else if( !strcasecmp( val->val.s.s, "tolerated" ) )
+                torrent_set_encryption( TR_PLAINTEXT_PREFERRED );
+            else {
                 msgresp(client, tag, IPC_MSG_BAD);
                 return;
             }
             break;
+
         case IPC_MSG_DIR:
             torrent_set_directory( val->val.s.s );
             break;
@@ -665,8 +661,6 @@ void
 infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
 {
     struct client * client = arg;
-    uint8_t       * buf;
-    size_t          buflen;
     benc_val_t      pk, * pkinf, * typelist, * idlist, * idval;
     int             all, types, ii, tor;
     void          * iter;
@@ -707,7 +701,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     /* add info/status for all torrents */
     if( all )
     {
-        if( NULL == val || TYPE_LIST != val->type )
+        if( !tr_bencIsList( val ) )
         {
             msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
@@ -729,7 +723,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     /* add info/status for the requested IDs */
     else
     {
-        if( NULL == val || TYPE_DICT != val->type )
+        if( !tr_bencIsDict( val ) )
         {
             msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
@@ -737,8 +731,7 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
         }
         typelist = tr_bencDictFind( val, "type" );
         idlist   = tr_bencDictFind( val, "id" );
-        if( NULL == typelist || TYPE_LIST != typelist->type ||
-            NULL == idlist   || TYPE_LIST != idlist->type )
+        if( !tr_bencIsList(typelist) || !tr_bencIsList(idlist) )
         {
             msgresp( client, tag, IPC_MSG_BAD );
             tr_bencFree( &pk );
@@ -763,25 +756,22 @@ infomsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
         }
     }
 
-    /* generate packet data and send it */
-    buf = ipc_mkval( &pk, &buflen );
+    queuepkmsg( client, &pk );
     tr_bencFree( &pk );
-    queuemsg( client, buf, buflen );
-    free( buf );
 }
 
 int
 addinfo( benc_val_t * list, int id, int types )
 {
-    const tr_info * inf = torrent_info( id );
-    return inf ? ipc_addinfo( list, id, inf, types ) : 0;
+    tr_torrent * tor = torrent_handle( id );
+    return tor ? ipc_addinfo( list, id, tor, types ) : 0;
 }
 
 int
 addstat( benc_val_t * list, int id, int types )
 {
-    const tr_stat * st = torrent_stat( id );
-    return st ? ipc_addstat( list, id, st, types ) : 0;
+    tr_torrent * tor = torrent_handle( id );
+    return tor ? ipc_addstat( list, id, tor, types ) : 0;
 }
 
 void
@@ -814,6 +804,10 @@ tormsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
         case IPC_MSG_STOP:
             func = torrent_stop;
             break;
+        case IPC_MSG_VERIFY:
+            all = 0;
+            func = torrent_verify;
+            break;
         default:
             assert( 0 );
             return;
@@ -835,7 +829,7 @@ tormsg( enum ipc_msg id, benc_val_t * val, int64_t tag, void * arg )
     /* remove/start/stop requested list of torrents */
     else
     {
-        if( NULL == val || TYPE_LIST != val->type )
+        if( !tr_bencIsList( val ) )
         {
             msgresp( client, tag, IPC_MSG_BAD );
             return;
@@ -858,13 +852,11 @@ void
 lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 {
     struct client * client = arg;
-    uint8_t       * buf;
-    size_t          buflen;
     int             ii;
     benc_val_t    * hash, pk, * pkinf;
     int64_t         found;
 
-    if( NULL == val || TYPE_LIST != val->type )
+    if( !tr_bencIsList( val ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
@@ -880,10 +872,8 @@ lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 
     for( ii = 0; val->val.l.count > ii; ii++ )
     {
-        const tr_info * inf;
         hash = &val->val.l.vals[ii];
-        if( NULL == hash || TYPE_STR != hash->type ||
-            SHA_DIGEST_LENGTH * 2 != hash->val.s.i )
+        if( !tr_bencIsString(hash) || SHA_DIGEST_LENGTH * 2 != hash->val.s.i )
         {
             tr_bencFree( &pk );
             msgresp( client, tag, IPC_MSG_BAD );
@@ -894,9 +884,7 @@ lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
         {
             continue;
         }
-        inf = torrent_info( found );
-        assert( NULL != inf );
-        if( 0 > ipc_addinfo( pkinf, found, inf, IPC_INF_HASH ) )
+        if( 0 > ipc_addinfo( pkinf, found, torrent_handle( found ), IPC_INF_HASH ) )
         {
             errnomsg( "failed to build message" );
             tr_bencFree( &pk );
@@ -905,10 +893,8 @@ lookmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
         }
     }
 
-    buf = ipc_mkval( &pk, &buflen );
+    queuepkmsg( client, &pk );
     tr_bencFree( &pk );
-    queuemsg( client, buf, buflen );
-    free( buf );
 }
 
 void
@@ -930,20 +916,11 @@ prefmsg( enum ipc_msg id, benc_val_t * val UNUSED, int64_t tag, void * arg )
                              torrent_get_autostart() );
             break;
         case IPC_MSG_GETCRYPTO:
-            switch(torrent_get_encryption())
-            {
-                case TR_PLAINTEXT_PREFERRED:
-                    strval = "plaintext";
-                    break;
-                case TR_ENCRYPTION_PREFERRED:
-                    strval = "preferred";
-                    break;
-                case TR_ENCRYPTION_REQUIRED:
-                    strval = "required";
-                    break;
-                default:
-                    assert(0);
-                    return;
+            switch(torrent_get_encryption()) {
+                case TR_ENCRYPTION_REQUIRED:  strval = "required"; break;
+                case TR_ENCRYPTION_PREFERRED: strval = "preferred"; break;
+                case TR_PLAINTEXT_PREFERRED:  strval = "tolerated"; break;
+                default: assert(0); return;
             }
             buf = ipc_mkstr(client->ipc, &buflen, IPC_MSG_CRYPTO, tag, strval);
             break;
@@ -980,13 +957,11 @@ void
 supmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 {
     struct client  * client = arg;
-    uint8_t        * buf;
-    size_t           buflen;
     int              ii;
-    benc_val_t       pk, *pkval, * name;
+    benc_val_t       pk, *pkval;
     enum ipc_msg     found;
 
-    if( NULL == val || TYPE_LIST != val->type )
+    if( !tr_bencIsList( val ) )
     {
         msgresp( client, tag, IPC_MSG_BAD );
         return;
@@ -1010,8 +985,8 @@ supmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
 
     for( ii = 0; val->val.l.count > ii; ii++ )
     {
-        name = &val->val.l.vals[ii];
-        if( NULL == name || TYPE_STR != name->type )
+        tr_benc * name = &val->val.l.vals[ii];
+        if( !tr_bencIsString( name ) )
         {
             tr_bencFree( &pk );
             msgresp( client, tag, IPC_MSG_BAD );
@@ -1026,8 +1001,6 @@ supmsg( enum ipc_msg id UNUSED, benc_val_t * val, int64_t tag, void * arg )
                         name->val.s.s, name->val.s.i, 1 );
     }
 
-    buf = ipc_mkval( &pk, &buflen );
+    queuepkmsg( client, &pk );
     tr_bencFree( &pk );
-    queuemsg( client, buf, buflen );
-    free( buf );
 }
