@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: torrents.c 4987 2008-02-08 16:51:33Z charles $
+ * $Id: torrents.c 5779 2008-05-08 19:34:12Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -39,11 +39,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <libtransmission/bsdtree.h>
 #include <libtransmission/bencode.h>
 #include <libtransmission/transmission.h>
 #include <libtransmission/trcompat.h>
 
+#include "bsdtree.h"
 #include "errors.h"
 #include "misc.h"
 #include "torrents.h"
@@ -65,17 +65,11 @@ RB_HEAD( tortree, tor );
 RB_HEAD( hashtree, tor );
 
 static struct tor * opentor    ( const char *, const char *, uint8_t *, size_t,
-                                  const char * );
+                                 const char *, int start );
 static void         closetor   ( struct tor *, int );
 static void         starttimer ( int );
 static void         timerfunc  ( int, short, void * );
-static int          loadstate  ( void );
 static int          savestate  ( void );
-static int          toridcmp   ( struct tor *, struct tor * );
-static int          torhashcmp ( struct tor *, struct tor * );
-static struct tor * idlookup   ( int );
-static struct tor * hashlookup ( const uint8_t * );
-static struct tor * iterate    ( struct tor * );
 
 static struct event_base * gl_base      = NULL;
 static tr_handle         * gl_handle    = NULL;
@@ -97,48 +91,113 @@ static int                 gl_downlimit = -1;
 static char                gl_dir[MAXPATHLEN];
 static tr_encryption_mode  gl_crypto    = TR_ENCRYPTION_PREFERRED;
 
-RB_GENERATE_STATIC( tortree, tor, idlinks, toridcmp )
+static int
+torhashcmp( struct tor * left, struct tor * right )
+{
+    return memcmp( left->hash, right->hash, sizeof left->hash );
+}
+
 RB_GENERATE_STATIC( hashtree, tor, hashlinks, torhashcmp )
+
 INTCMP_FUNC( toridcmp, tor, id )
+RB_GENERATE_STATIC( tortree, tor, idlinks, toridcmp )
 
 void
-torrent_init( struct event_base * base )
+torrent_init( const char * configdir, struct event_base * base )
 {
-    assert( NULL == gl_handle && NULL == gl_base );
+    tr_benc state, * torrents;
+    int have_state;
+    assert( !gl_handle && !gl_base );
 
-    gl_base   = base;
-    gl_handle = tr_init( "daemon" );
-
-    confpath( gl_state, sizeof gl_state, CONF_FILE_STATE, 0 );
-    strlcpy( gl_newstate, gl_state, sizeof gl_state );
-    strlcat( gl_newstate, ".new", sizeof gl_state );
+    confpath( gl_state, sizeof gl_state, configdir, CONF_FILE_STATE, 0 );
+    snprintf( gl_newstate, sizeof( gl_newstate ), "%s.new", gl_state );
     absolutify( gl_dir, sizeof gl_dir, "." );
 
-    loadstate();
+    /* initialize the session variables */
+    if(( have_state = !tr_bencLoadFile( gl_state, &state )))
+    {
+        int64_t i;
+        const char * str;
+
+        if( tr_bencDictFindInt( &state, "autostart", &i ) )
+            gl_autostart = i != 0;
+        if( tr_bencDictFindInt( &state, "port", &i ) && ( 0<i ) && ( i<=0xffff ) )
+            gl_port = i;
+        if( tr_bencDictFindInt( &state, "default-pex", &i ) )
+            gl_pex = i != 0;
+        if( tr_bencDictFindInt( &state, "port-mapping", &i ) )
+            gl_mapping = i != 0;
+        if( tr_bencDictFindInt( &state, "upload-limit", &i ) )
+            gl_uplimit = i;
+        if( tr_bencDictFindInt( &state, "download-limit", &i ) )
+            gl_downlimit = i;
+        if( tr_bencDictFindStr( &state, "default-directory", &str ) )
+            strlcpy( gl_dir, str, sizeof gl_dir );
+        if( tr_bencDictFindStr( &state, "encryption-mode", &str ) ) {
+            if( !strcmp( str, "required" ) )
+                gl_crypto = TR_ENCRYPTION_REQUIRED;
+            else
+                gl_crypto = TR_ENCRYPTION_PREFERRED;
+        }
+    }
+
+    /* start the session */
+    gl_base = base;
+    gl_handle = tr_initFull( configdir, "daemon", gl_pex,
+                             gl_mapping, gl_port,
+                             gl_crypto,
+                             gl_uplimit >= 0, gl_uplimit,
+                             gl_downlimit >= 0, gl_downlimit,
+                             TR_DEFAULT_GLOBAL_PEER_LIMIT,
+                             TR_MSG_INF, 0,
+                             0, /* is the blocklist enabled? */
+                             TR_DEFAULT_PEER_SOCKET_TOS );
+
+    /* now load the torrents */
+    if( have_state && tr_bencDictFindList( &state, "torrents", &torrents ) )
+    {
+        int i, n;
+        for( i=0, n=tr_bencListSize(torrents); i<n; ++i )
+        {
+            int start;
+            int64_t paused;
+            const char * directory = NULL;
+            const char * hash = NULL;
+
+            tr_benc * dict = tr_bencListChild( torrents, i );
+            if( !tr_bencIsDict( dict ) ||
+                !tr_bencDictFindStr( dict, "directory", &directory ) ||
+                !tr_bencDictFindStr( dict, "hash", &hash ) )
+                continue;
+
+            if( tr_bencDictFindInt( dict, "paused", &paused ) )
+                start = !paused;
+            else
+                start = gl_autostart;
+
+            opentor( NULL, hash, NULL, 0, directory, start );
+        }
+    }
+
+    /* cleanup */
+    if( have_state )
+        tr_bencFree( &state );
 }
 
 int
-torrent_add_file( const char * path, const char * dir, int autostart )
+torrent_add_file( const char * path, const char * dir, int start )
 {
     struct tor * tor;
 
-    assert( NULL != gl_handle );
+    assert( gl_handle );
     assert( !gl_exiting );
 
-    tor = opentor( path, NULL, NULL, 0, dir );
-    if( NULL == tor )
-    {
-        return -1;
-    }
+    if( start < 0 )
+        start = gl_autostart;
 
-    if( 0 > autostart )
-    {
-        autostart = gl_autostart;
-    }
-    if( autostart )
-    {
-        tr_torrentStart( tor->tor );
-    }
+    tor = opentor( path, NULL, NULL, 0, dir, start );
+    if( !tor )
+        return -1;
 
     savestate();
 
@@ -146,51 +205,48 @@ torrent_add_file( const char * path, const char * dir, int autostart )
 }
 
 int
-torrent_add_data( uint8_t * data, size_t size, const char * dir, int autostart )
+torrent_add_data( uint8_t * data, size_t size, const char * dir, int start )
 {
     struct tor * tor;
 
-    assert( NULL != gl_handle );
+    assert( gl_handle );
     assert( !gl_exiting );
 
-    tor = opentor( NULL, NULL, data, size, dir );
-    if( NULL == tor )
-    {
-        return -1;
-    }
+    if( start < 0 )
+        start = gl_autostart;
 
-    if( 0 > autostart )
-    {
-        autostart = gl_autostart;
-    }
-    if( autostart )
-    {
-        tr_torrentStart( tor->tor );
-    }
+    tor = opentor( NULL, NULL, data, size, dir, start );
+    if( !tor )
+        return -1;
 
     savestate();
 
     return tor->id;
 }
 
+static struct tor *
+idlookup( int id )
+{
+    struct tor * found = NULL;
+
+    if( gl_handle && !gl_exiting )
+    {
+        struct tor key;
+        memset( &key, 0, sizeof key );
+        key.id = id;
+        found = RB_FIND( tortree, &gl_tree, &key );
+    }
+
+    return found;
+}
+
 void
 torrent_start( int id )
 {
-    struct tor * tor;
-
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
-
-    tor = idlookup( id );
-    if( tor != NULL )
-    {
-        const tr_stat  * st = tr_torrentStat( tor->tor );
-
-        if( !TR_STATUS_IS_ACTIVE( st->status ) )
-        {
-            tr_torrentStart( tor->tor );
-            savestate();
-        }
+    struct tor * tor = idlookup( id );
+    if( tor && !TR_STATUS_IS_ACTIVE( tr_torrentStat( tor->tor )->status ) ) {
+        tr_torrentStart( tor->tor );
+        savestate();
     }
 
 }
@@ -198,72 +254,60 @@ torrent_start( int id )
 void
 torrent_stop( int id )
 {
-    struct tor * tor;
-
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
-
-    tor = idlookup( id );
-    if( tor != NULL )
-    {
-        const tr_stat  * st = tr_torrentStat( tor->tor );
-
-        if( TR_STATUS_IS_ACTIVE( st->status ) )
-        {
-            tr_torrentStop( tor->tor );
-            savestate();
-        }
+    struct tor * tor = idlookup( id );
+    if( tor && TR_STATUS_IS_ACTIVE( tr_torrentStat( tor->tor )->status ) ) {
+        tr_torrentStop( tor->tor );
+        savestate( );
     }
 }
 
+void
+torrent_verify( int id )
+{
+    struct tor * tor = idlookup( id );
+    if( tor )
+        tr_torrentVerify( tor->tor );
+}
+
 void
 torrent_remove( int id )
 {
-    struct tor * tor;
-
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
-
-    tor = idlookup( id );
-    if( tor != NULL )
-    {
+    struct tor * tor = idlookup( id );
+    if( tor ) {
         closetor( tor, 1 );
         savestate();
     }
 }
 
+tr_torrent *
+torrent_handle( int id )
+{
+    const struct tor * tor = idlookup( id );
+    return tor ? tor->tor : NULL;
+}
+   
 const tr_info *
 torrent_info( int id )
 {
-    struct tor * tor;
-
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
-
-    tor = idlookup( id );
-    if( NULL == tor )
-    {
-        return NULL;
-    }
-
-    return tr_torrentInfo( tor->tor );
+    return tr_torrentInfo( torrent_handle( id ) );
 }
 
 const tr_stat *
 torrent_stat( int id )
 {
-    struct tor * tor;
+    return tr_torrentStat( torrent_handle( id ) );
+}
 
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
+static struct tor *
+hashlookup( const uint8_t * hash )
+{
+    struct tor key, * found;
 
-    tor = idlookup( id );
-    if( NULL == tor )
-    {
-        return NULL;
-    }
+    memset( &key, 0, sizeof key );
+    memcpy( key.hash, hash, sizeof key.hash );
+    found = RB_FIND( hashtree, &gl_hashes, &key );
 
-    return tr_torrentStat( tor->tor );
+    return found;
 }
 
 int
@@ -297,22 +341,25 @@ torrent_lookup( const uint8_t * hashstr )
     return tor->id;
 }
 
-void *
-torrent_iter( void * iter, int * id )
+static struct tor *
+iterate( struct tor * tor )
 {
-    struct tor * tor = iter;
+    struct tor * next = NULL;
 
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
+    if( gl_handle && !gl_exiting )
+        next = tor ? RB_NEXT( tortree, &gl_tree, tor )
+                   : RB_MIN( tortree, &gl_tree );
 
-    tor = iterate( tor );
+    return next;
+}
 
-    if( NULL != tor )
-    {
-        *id = tor->id;
-    }
-
-    return tor;
+void *
+torrent_iter( void * cur, int * id )
+{
+    struct tor * next = iterate( cur );
+    if( next )
+        *id = next->id;
+    return next;
 }
 
 void
@@ -471,17 +518,19 @@ torrent_get_encryption(void)
 }
 
 struct tor *
-opentor( const char * path, const char * hash, uint8_t * data, size_t size,
-         const char * dir )
+opentor( const char * path,
+         const char * hash,
+         uint8_t    * data,
+         size_t       size,
+         const char * dir, 
+         int          start )
 {
     struct tor * tor, * found;
     int          errcode;
     const tr_info  * inf;
     tr_ctor        * ctor;
 
-    assert( ( NULL != path && NULL == hash && NULL == data ) ||
-            ( NULL == path && NULL != hash && NULL == data ) ||
-            ( NULL == path && NULL == hash && NULL != data ) );
+    assert( (path?1:0) + (hash?1:0) + (data?1:0) == 1 );
 
     /* XXX should probably wrap around back to 1 and avoid duplicates */
     if( INT_MAX == gl_lastid )
@@ -502,7 +551,7 @@ opentor( const char * path, const char * hash, uint8_t * data, size_t size,
         dir = gl_dir;
 
     ctor = tr_ctorNew( gl_handle );
-    tr_ctorSetPaused( ctor, TR_FORCE, 1 );
+    tr_ctorSetPaused( ctor, TR_FORCE, !start );
     tr_ctorSetDestination( ctor, TR_FORCE, dir );
     if( path != NULL )
         tr_ctorSetMetainfoFromFile( ctor, path );
@@ -613,10 +662,10 @@ starttimer( int callnow )
 static void
 timerfunc( int fd UNUSED, short event UNUSED, void * arg UNUSED )
 {
-    struct tor       * tor, * next;
-    tr_handle_status * hs;
-    int                stillmore;
-    struct timeval     tv;
+    struct tor             * tor, * next;
+    const tr_handle_status * hs;
+    int                      stillmore;
+    struct timeval           tv;
 
     /* true if we've still got live torrents... */
     stillmore = tr_torrentCount( gl_handle ) != 0;
@@ -657,213 +706,45 @@ timerfunc( int fd UNUSED, short event UNUSED, void * arg UNUSED )
     }
 }
 
-int
-loadstate( void )
-{
-    uint8_t   *  buf;
-    size_t       len;
-    benc_val_t   top, * num, * str, * list, * dict;
-    int          ii;
-    struct tor * tor;
-    const char * dir;
-
-    buf = readfile( gl_state, &len );
-    if( NULL == buf )
-    {
-        return -1;
-    }
-
-    if( tr_bencLoad( buf, len, &top, NULL ) )
-    {
-        free( buf );
-        errmsg( "failed to load bencoded data from %s", gl_state );
-        return -1;
-    }
-    free( buf );
-
-    num = tr_bencDictFind( &top, "autostart" );
-    if( NULL != num && TYPE_INT == num->type )
-    {
-        gl_autostart = ( num->val.i ? 1 : 0 );
-    }
-
-    num = tr_bencDictFind( &top, "port" );
-    if( NULL != num && TYPE_INT == num->type &&
-        0 < num->val.i && 0xffff > num->val.i )
-    {
-        gl_port = num->val.i;
-    }
-    tr_setBindPort( gl_handle, gl_port );
-
-    num = tr_bencDictFind( &top, "default-pex" );
-    if( NULL != num && TYPE_INT == num->type )
-    {
-        gl_pex = ( num->val.i ? 1 : 0 );
-    }
-
-    num = tr_bencDictFind( &top, "port-mapping" );
-    if( NULL != num && TYPE_INT == num->type )
-    {
-        gl_mapping = ( num->val.i ? 1 : 0 );
-    }
-    tr_natTraversalEnable( gl_handle, gl_mapping );
-
-    num = tr_bencDictFind( &top, "upload-limit" );
-    if( NULL != num && TYPE_INT == num->type )
-    {
-        gl_uplimit = num->val.i;
-    }
-    tr_setGlobalSpeedLimit( gl_handle, TR_UP, gl_uplimit );
-    tr_setUseGlobalSpeedLimit( gl_handle, TR_UP, gl_uplimit > 0 );
-
-    num = tr_bencDictFind( &top, "download-limit" );
-    if( NULL != num && TYPE_INT == num->type )
-    {
-        gl_downlimit = num->val.i;
-    }
-    tr_setGlobalSpeedLimit( gl_handle, TR_DOWN, gl_downlimit );
-    tr_setUseGlobalSpeedLimit( gl_handle, TR_DOWN, gl_downlimit > 0 );
-
-    str = tr_bencDictFind( &top, "default-directory" );
-    if( NULL != str && TYPE_STR == str->type )
-    {
-        strlcpy( gl_dir, str->val.s.s, sizeof gl_dir );
-    }
-
-    str = tr_bencDictFind( &top, "encryption-mode" );
-    if( NULL != str && TYPE_STR == str->type )
-    {
-        if(!strcasecmp(str->val.s.s, "preferred"))
-            gl_crypto = TR_ENCRYPTION_PREFERRED;
-        else if(!strcasecmp(str->val.s.s, "required"))
-            gl_crypto = TR_ENCRYPTION_REQUIRED;
-    }
-
-    tr_setEncryptionMode(gl_handle, gl_crypto);
-
-    list = tr_bencDictFind( &top, "torrents" );
-    if( NULL == list || TYPE_LIST != list->type )
-    {
-        return 0;
-    }
-
-    for( ii = 0; ii < list->val.l.count; ii++ )
-    {
-        dict = &list->val.l.vals[ii];
-        if( TYPE_DICT != dict->type )
-        {
-            continue;
-        }
-
-        str = tr_bencDictFind( dict, "directory" );
-        dir = ( NULL != str && TYPE_STR == str->type ? str->val.s.s : NULL );
-
-        str = tr_bencDictFind( dict, "hash" );
-        if( NULL == str || TYPE_STR != str->type ||
-            2 * SHA_DIGEST_LENGTH != str->val.s.i )
-        {
-            continue;
-        }
-
-        tor = opentor( NULL, str->val.s.s, NULL, 0, dir );
-        if( NULL == tor )
-        {
-            continue;
-        }
-
-        num = tr_bencDictFind( dict, "pex" );
-        if( NULL != num && TYPE_INT == num->type )
-        {
-            fprintf( stderr, "warning: obsolete command 'pex'\n" );
-        }
-
-        num = tr_bencDictFind( dict, "paused" );
-        if( NULL != num && TYPE_INT == num->type && !num->val.i )
-        {
-            tr_torrentStart( tor->tor );
-        }
-    }
-
-    return 0;
-}
-
 int
 savestate( void )
 {
-    benc_val_t   top, * list, * tor;
+    benc_val_t   top, * list;
     struct tor * ii;
-    uint8_t    * buf;
-    int          len;
-
-    tr_bencInit( &top, TYPE_DICT );
-    if( tr_bencDictReserve( &top, 9 ) )
-    {
-      nomem:
-        tr_bencFree( &top );
-        errmsg( "failed to save state: failed to allocate memory" );
-        return -1;
-    }
-    tr_bencInitInt( tr_bencDictAdd( &top, "autostart" ),      gl_autostart );
-    tr_bencInitInt( tr_bencDictAdd( &top, "port" ),           gl_port );
-    tr_bencInitInt( tr_bencDictAdd( &top, "default-pex" ),    gl_pex );
-    tr_bencInitInt( tr_bencDictAdd( &top, "port-mapping" ),   gl_mapping );
-    tr_bencInitInt( tr_bencDictAdd( &top, "upload-limit" ),   gl_uplimit );
-    tr_bencInitInt( tr_bencDictAdd( &top, "download-limit" ), gl_downlimit );
-    tr_bencInitStr( tr_bencDictAdd( &top, "default-directory" ),
-                    gl_dir, -1, 1 );
-    if(TR_ENCRYPTION_REQUIRED == gl_crypto)
-        tr_bencInitStr(tr_bencDictAdd(&top, "encryption-mode"), "required", -1, 1);
-    else
-        tr_bencInitStr(tr_bencDictAdd(&top, "encryption-mode"), "preferred", -1, 1);
-    list = tr_bencDictAdd( &top, "torrents" );
-    tr_bencInit( list, TYPE_LIST );
-
-    len = 0;
+    int          torrentCount;
+ 
+    torrentCount = 0;
     RB_FOREACH( ii, tortree, &gl_tree )
-    {
-        len++;
-    }
-    if( tr_bencListReserve( list, len ) )
-    {
-        goto nomem;
-    }
+        ++torrentCount;
+
+    tr_bencInitDict( &top, 9 );
+    tr_bencDictAddInt( &top, "autostart",         gl_autostart );
+    tr_bencDictAddInt( &top, "port",              gl_port );
+    tr_bencDictAddInt( &top, "default-pex",       gl_pex );
+    tr_bencDictAddInt( &top, "port-mapping",      gl_mapping );
+    tr_bencDictAddInt( &top, "upload-limit",      gl_uplimit );
+    tr_bencDictAddInt( &top, "download-limit",    gl_downlimit );
+    tr_bencDictAddStr( &top, "default-directory", gl_dir );
+    tr_bencDictAddStr( &top, "encryption-mode",   TR_ENCRYPTION_REQUIRED == gl_crypto
+                                                  ? "required" : "preferred" );
+    list = tr_bencDictAddList( &top, "torrents", torrentCount );
 
     RB_FOREACH( ii, tortree, &gl_tree )
     {
-        const tr_info * inf;
-        const tr_stat * st;
-        tor = tr_bencListAdd( list );
-        assert( NULL != tor );
-        tr_bencInit( tor, TYPE_DICT );
-        inf    = tr_torrentInfo( ii->tor );
-        st     = tr_torrentStat( ii->tor );
-        if( tr_bencDictReserve( tor, 3 ) )
-        {
-            goto nomem;
-        }
-        tr_bencInitStr( tr_bencDictAdd( tor, "hash" ),
-                        inf->hashString, 2 * SHA_DIGEST_LENGTH, 1 );
-        tr_bencInitInt( tr_bencDictAdd( tor, "paused" ),
-                        !TR_STATUS_IS_ACTIVE( st->status ) );
-        tr_bencInitStr( tr_bencDictAdd( tor, "directory" ),
-                        tr_torrentGetFolder( ii->tor ), -1, 1 );
+        const tr_info * inf = tr_torrentInfo( ii->tor );
+        const tr_stat * st  = tr_torrentStat( ii->tor );
+        tr_benc * tor = tr_bencListAddDict( list, 3 );
+        tr_bencDictAddStr( tor, "hash", inf->hashString );
+        tr_bencDictAddInt( tor, "paused", !TR_STATUS_IS_ACTIVE( st->status ) );
+        tr_bencDictAddStr( tor, "directory", tr_torrentGetFolder( ii->tor ) );
     }
 
-    buf = ( uint8_t * )tr_bencSave( &top, &len );
-    SAFEBENCFREE( &top );
-    if( NULL == buf )
+    if( tr_bencSaveFile( gl_newstate, &top ) )
     {
-        errnomsg( "failed to save state: bencoding failed" );
+        errnomsg( "failed to save state: failed to write to %s", gl_newstate );
         return -1;
     }
 
-    if( 0 > writefile( gl_newstate, buf, len ) )
-    {
-        free( buf );
-        return -1;
-    }
-    free( buf );
-
     if( 0 > rename( gl_newstate, gl_state ) )
     {
         errnomsg( "failed to save state: failed to rename %s to %s",
@@ -873,51 +754,3 @@ savestate( void )
 
     return 0;
 }
-
-int
-torhashcmp( struct tor * left, struct tor * right )
-{
-    return memcmp( left->hash, right->hash, sizeof left->hash );
-}
-
-struct tor *
-idlookup( int id )
-{
-    struct tor key, * found;
-
-    memset( &key, 0, sizeof key );
-    key.id = id;
-    found = RB_FIND( tortree, &gl_tree, &key );
-
-    return found;
-}
-
-struct tor *
-hashlookup( const uint8_t * hash )
-{
-    struct tor key, * found;
-
-    memset( &key, 0, sizeof key );
-    memcpy( key.hash, hash, sizeof key.hash );
-    found = RB_FIND( hashtree, &gl_hashes, &key );
-
-    return found;
-}
-
-struct tor *
-iterate( struct tor * tor )
-{
-    assert( NULL != gl_handle );
-    assert( !gl_exiting );
-
-    if( NULL == tor )
-    {
-        tor = RB_MIN( tortree, &gl_tree );
-    }
-    else
-    {
-        tor = RB_NEXT( tortree, &gl_tree, tor );
-    }
-
-    return tor;
-}
