@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: torrents.h 3111 2007-09-20 20:14:13Z charles $
+ * $Id: torrents.h 5646 2008-04-19 00:41:32Z charles $
  *
  * Copyright (c) 2007 Joshua Elsasser
  *
@@ -31,14 +31,16 @@
 
 struct event_base;
 
-void         torrent_init                ( struct event_base * );
+void         torrent_init                ( const char * configDir, struct event_base * );
 int          torrent_add_file            ( const char *, const char *, int );
 int          torrent_add_data            ( uint8_t *, size_t, const char *, int );
 void         torrent_start               ( int );
 void         torrent_stop                ( int );
+void         torrent_verify              ( int );
 void         torrent_remove              ( int );
-const tr_info  * torrent_info          ( int );
-const tr_stat  * torrent_stat          ( int );
+tr_torrent     * torrent_handle          ( int );
+const tr_info  * torrent_info            ( int );
+const tr_stat  * torrent_stat            ( int );
 int          torrent_lookup              ( const uint8_t * );
 void       * torrent_iter                ( void *, int * );
 
@@ -57,5 +59,7 @@ void         torrent_set_downlimit       ( int );
 int          torrent_get_downlimit       ( void );
 void         torrent_set_directory       ( const char * );
 const char * torrent_get_directory       ( void );
+void         torrent_set_encryption      ( tr_encryption_mode );
+tr_encryption_mode torrent_get_encryption      ( void );
 
 #endif /* TR_DAEMON_TORRENTS_H */
