@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.c 4404 2008-01-01 17:20:20Z charles $
+ * $Id: conf.c 5753 2008-05-06 11:04:16Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -28,9 +28,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include <glib.h>
@@ -39,6 +36,7 @@
 
 #include <libtransmission/transmission.h>
 #include <libtransmission/bencode.h>
+#include <libtransmission/platform.h>
 
 #include "conf.h"
 #include "util.h"
@@ -59,7 +57,7 @@ cf_init(const char *dir, char **errstr)
         return TRUE;
 
     if( errstr != NULL )
-        *errstr = g_strdup_printf( _("Failed to create the directory %s:\n%s"),
+        *errstr = g_strdup_printf( _( "Couldn't create \"%1$s\": %2$s" ),
                                    gl_confdir, g_strerror(errno) );
 
     return FALSE;
@@ -72,47 +70,27 @@ cf_init(const char *dir, char **errstr)
 ***/
 
 /* errstr may be NULL, this might be called before GTK is initialized */
-static int
-lockfile(const char *file, char **errstr)
+static gboolean
+lockfile(const char * filename, char **errstr)
 {
-    int fd;
-    struct flock lk;
+    const int state = tr_lockfile( filename );
+    const gboolean success = state == TR_LOCKFILE_SUCCESS;
 
-    if( errstr )
-        *errstr = NULL;
-
-    fd = open( file, O_RDWR | O_CREAT, 0666 );
-    if( fd < 0 )
-    { 
-        const int savederr = errno;
-        if( errstr )
-            *errstr = g_strdup_printf(
-                          _("Failed to open the file %s for writing:\n%s"),
-                          file, g_strerror( errno ) );
-        errno = savederr;
-        return -1;
+    if( errstr ) switch( state ) {
+        case TR_LOCKFILE_EOPEN:
+        *errstr = g_strdup_printf( _( "Couldn't open \"%1$s\": %2$s" ),
+                                   filename, g_strerror( errno ) );
+            break;
+        case TR_LOCKFILE_ELOCK:
+            *errstr = g_strdup_printf( _( "%s is already running." ),
+                                       g_get_application_name( ) );
+            break;
+        case TR_LOCKFILE_SUCCESS:
+            *errstr = NULL;
+            break;
     }
 
-    memset( &lk, 0,  sizeof( lk ) );
-    lk.l_start = 0;
-    lk.l_len = 0;
-    lk.l_type = F_WRLCK;
-    lk.l_whence = SEEK_SET;
-    if( -1 == fcntl( fd, F_SETLK, &lk ) )
-    {
-        const int savederr = errno;
-        if( errstr )
-            *errstr = ( errno == EAGAIN )
-                ? g_strdup_printf( _( "Another copy of %s is already running." ),
-                                   g_get_application_name( ) )
-                : g_strdup_printf( _( "Failed to lock the file %s:\n%s" ),
-                                  file, g_strerror( errno ) );
-        close( fd );
-        errno = savederr;
-        return -1;
-    }
-
-    return fd;
+    return success;
 }
 
 static char*
@@ -125,8 +103,10 @@ getLockFilename( void )
 static void
 cf_removelocks( void )
 {
-    g_unlink( gl_lockpath );
-    g_free( gl_lockpath );
+    if( gl_lockpath ) {
+      g_unlink( gl_lockpath );
+      g_free( gl_lockpath );
+    }
 }
 
 /* errstr may be NULL, this might be called before GTK is initialized */
@@ -134,12 +114,12 @@ gboolean
 cf_lock( char ** errstr )
 {
     char * path = getLockFilename( );
-    int fd = lockfile( path, errstr );
-    if( fd >= 0 )
+    const gboolean didLock = lockfile( path, errstr );
+    if( didLock )
         gl_lockpath = g_strdup( path );
     g_atexit( cf_removelocks );
     g_free( path );
-    return fd >= 0;
+    return didLock;
 }
 
 char*
@@ -198,6 +178,14 @@ gboolean
 pref_flag_get ( const char * key ) {
     return g_key_file_get_boolean( getPrefsKeyFile( ), GROUP, key, NULL );
 }
+gboolean
+pref_flag_eval( pref_flag_t val, const char * key ) {
+    switch( val ) {
+        case PREF_FLAG_TRUE: return TRUE;
+        case PREF_FLAG_FALSE: return FALSE;
+        default: return pref_flag_get( key );
+    }
+}
 void
 pref_flag_set( const char * key, gboolean value ) {
     g_key_file_set_boolean( getPrefsKeyFile( ), GROUP, key, value );
@@ -272,7 +260,14 @@ static char*
 getCompat08PrefsFilename( void )
 {
     assert( gl_confdir != NULL );
-    return g_build_filename( gl_confdir, "prefs", NULL );
+    return g_build_filename( g_get_home_dir(), ".transmission", "gtk", "prefs", NULL );
+}
+
+static char*
+getCompat09PrefsFilename( void )
+{
+    assert( gl_confdir != NULL );
+    return g_build_filename( g_get_home_dir(), ".transmission", "gtk", "prefs.ini", NULL );
 }
 
 static void
@@ -300,9 +295,9 @@ translate_08_to_09( const char* oldfile, const char* newfile )
     GString * out = g_string_new( NULL );
     gchar * contents = NULL;
     gsize contents_len = 0;
-    benc_val_t top;
+    tr_benc top;
 
-    memset( &top, 0, sizeof(benc_val_t) );
+    memset( &top, 0, sizeof(tr_benc) );
 
     if( g_file_get_contents( oldfile, &contents, &contents_len, NULL )
         && !tr_bencLoad( contents, contents_len, &top, NULL )
@@ -311,7 +306,7 @@ translate_08_to_09( const char* oldfile, const char* newfile )
         unsigned int i;
         g_string_append( out, "\n[general]\n" );
         for ( i=0; i<G_N_ELEMENTS(pref_table); ++i ) {
-            const benc_val_t * val = tr_bencDictFind( &top, pref_table[i].oldkey );
+            const tr_benc * val = tr_bencDictFind( &top, pref_table[i].oldkey );
             if( val != NULL ) {
                 const char * valstr = val->val.s.s;
                 if( !strcmp( valstr, "yes" ) ) valstr = "true";
@@ -331,11 +326,17 @@ cf_check_older_configs( void )
 {
     char * cfn = getPrefsFilename( );
     char * cfn08 = getCompat08PrefsFilename( );
+    char * cfn09 = getCompat09PrefsFilename( );
+
+    if( !g_file_test( cfn,   G_FILE_TEST_IS_REGULAR )
+      && g_file_test( cfn09, G_FILE_TEST_IS_REGULAR ) )
+      g_rename( cfn09, cfn );
 
     if( !g_file_test( cfn,   G_FILE_TEST_IS_REGULAR )
       && g_file_test( cfn08, G_FILE_TEST_IS_REGULAR ) )
         translate_08_to_09( cfn08, cfn );
 
+    g_free( cfn09 );
     g_free( cfn08 );
     g_free( cfn );
 }
