@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: dialogs.h 1654 2007-04-04 00:32:58Z joshe $
+ * $Id: dialogs.h 5276 2008-03-18 01:22:11Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,33 +25,16 @@
 #ifndef TG_PREFS_H
 #define TG_PREFS_H
 
-#include "tr_backend.h"
-#include "tr_torrent.h"
+#include "tr-core.h"
+#include "tr-torrent.h"
 #include "util.h"
 
-GtkWidget *
-makeprefwindow(GtkWindow *parent, TrBackend *back);
-
-/* set various things based on saved prefs */
-void
-applyprefs(TrBackend *back);
-
-/* show the "add a torrent" dialog */
-void
-makeaddwind(GtkWindow *parent, add_torrents_func_t addfunc, void *cbdata);
-
-/* show the info window for a torrent */
-void
-makeinfowind( GtkWindow * parent, GtkTreeModel * model, GtkTreePath * path,
-              TrTorrent * tor );
-
-/* prompt for a download directory for torrents, then add them */
-void
-promptfordir( GtkWindow * parent, add_torrents_func_t addfunc, void *cbdata,
-              GList * files, guint flags );
-
 /* prompt if the user wants to quit, calls func with cbdata if they do */
-void
-askquit( GtkWindow * parent, callbackfunc_t func, void * cbdata );
+void askquit( TrCore*, GtkWindow* parent, callbackfunc_t func, void* cbdata );
+
+void confirmRemove( GtkWindow * parent,
+                    TrCore    * core,
+                    GSList    * gtorrents,
+                    gboolean    doDelete );
 
 #endif /* TG_PREFS_H */
