@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id: makemeta-ui.c 4849 2008-01-28 04:27:20Z charles $
+ * $Id: makemeta-ui.c 5765 2008-05-07 04:06:07Z charles $
  */
 
 #include <string.h>
@@ -24,6 +24,9 @@
 
 #define UPDATE_INTERVAL_MSEC 200
 
+#define BUILDER_KEY "builder"
+#define UI_KEY "ui"
+
 typedef struct
 {
     GtkWidget * size_lb;
@@ -44,8 +47,7 @@ MakeMetaUI;
 static void
 freeMetaUI( gpointer p )
 {
-    MakeMetaUI * ui = (MakeMetaUI *) p;
-    tr_metaInfoBuilderFree( ui->builder );
+    MakeMetaUI * ui = p;
     memset( ui, ~0, sizeof(MakeMetaUI) );
     g_free( ui );
 }
@@ -65,7 +67,7 @@ setIsBuilding( MakeMetaUI * ui, gboolean isBuilding )
     ui->isBuilding = isBuilding;
 
     if( ui->builder != NULL )
-        ui->builder->failed = FALSE;
+        ui->builder->result = TR_MAKEMETA_OK;
 
     if( !isBuilding )
         gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( ui->progressbar ), 0 );
@@ -79,7 +81,7 @@ refresh_cb ( gpointer user_data )
     int denom;
     char buf[1024];
     double fraction;
-    MakeMetaUI * ui = (MakeMetaUI *) user_data;
+    MakeMetaUI * ui = user_data;
     GtkProgressBar * p = GTK_PROGRESS_BAR( ui->progressbar );
 
     denom = ui->builder->pieceCount ? ui->builder->pieceCount : 1;
@@ -90,21 +92,41 @@ refresh_cb ( gpointer user_data )
 
     if( ui->builder->isDone )
     {
-        if( ui->builder->failed )
-        {
-            const char * reason = ui->builder->abortFlag
-                ? _("Torrent creation aborted.")
-                : _("Torrent creation failed.");
+        char * txt = NULL;
 
-            gtk_progress_bar_set_text( p, reason );
-            gtk_progress_bar_set_fraction( p, 0 );
-        }
-        else
+        switch( ui->builder->result )
         {
-            gtk_progress_bar_set_text( p, _("Torrent Created") );
+            case TR_MAKEMETA_OK:
+                txt = g_strdup( _( "Torrent created" ) );
+                break;
+
+            case TR_MAKEMETA_URL:
+                txt = g_strdup_printf( _( "Torrent creation failed: %s" ), _( "Invalid URL" ) );
+                break;
+
+            case TR_MAKEMETA_CANCELLED:
+                txt = g_strdup_printf( _( "Torrent creation cancelled" ) );
+                break;
+
+            case TR_MAKEMETA_IO_READ: {
+                char * tmp = g_strdup_printf( _( "Couldn't read \"%1$s\": %2$s" ), ui->builder->errfile, g_strerror( ui->builder->my_errno ) );
+                txt = g_strdup_printf( _( "Torrent creation failed: %s" ), tmp );
+                g_free( tmp  );
+                break;
+            }
+
+            case TR_MAKEMETA_IO_WRITE: {
+                char * tmp = g_strdup_printf( _( "Couldn't create \"%1$s\": %2$s" ), ui->builder->errfile, g_strerror( ui->builder->my_errno ) );
+                txt = g_strdup_printf( _( "Torrent creation failed: %s" ), tmp );
+                g_free( tmp  );
+                break;
+            }
         }
 
+        gtk_progress_bar_set_fraction( p, ui->builder->result==TR_MAKEMETA_OK ? 1.0 : 0.0 );
+        gtk_progress_bar_set_text( p, txt );
         setIsBuilding( ui, FALSE );
+        g_free( txt );
     }
 
     return !ui->builder->isDone;
@@ -119,7 +141,7 @@ remove_tag (gpointer tag)
 static void
 response_cb( GtkDialog* d, int response, gpointer user_data )
 {
-    MakeMetaUI * ui = (MakeMetaUI*) user_data;
+    MakeMetaUI * ui = user_data;
     char *tmp;
     char buf[1024];
     guint tag;
@@ -170,69 +192,90 @@ response_cb( GtkDialog* d, int response, gpointer user_data )
 ***/
 
 static void
-file_selection_changed_cb( GtkFileChooser *chooser, gpointer user_data )
+refreshFromBuilder( MakeMetaUI * ui )
 {
-    MakeMetaUI * ui = (MakeMetaUI *) user_data;
-    char * filename;
     char sizeStr[128];
     char buf[MAX_PATH_LENGTH];
-    uint64_t totalSize=0;
-    int fileCount=0, pieceCount=0, pieceSize=0;
+    tr_metainfo_builder * builder = ui->builder;
+    const char * filename = builder ? builder->top : NULL;
 
-    if( ui->builder ) {
-        tr_metaInfoBuilderFree( ui->builder );
-        ui->builder = NULL;
-    }
-
-    filename = gtk_file_chooser_get_filename( chooser );
     if( !filename )
-        *buf = '\0';
-    else {
-        ui->builder = tr_metaInfoBuilderCreate( ui->handle, filename );
+        g_snprintf( buf, sizeof( buf ), _( "No files selected" ) );
+    else
         g_snprintf( buf, sizeof(buf), "%s.torrent (%d%%)", filename, 0 );
-        g_free( filename );
-        fileCount = ui->builder->fileCount;
-        totalSize = ui->builder->totalSize;
-        pieceCount = ui->builder->pieceCount;
-        pieceSize = ui->builder->pieceSize;
-    }
     gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar ), buf );
     refreshButtons( ui );
 
-    tr_strlsize( sizeStr, totalSize, sizeof(sizeStr) );
-    g_snprintf( buf, sizeof(buf), "<i>%s; %d %s</i>",
-                sizeStr, fileCount,
-                ngettext("File", "Files", fileCount) );
+    if( !filename )
+        g_snprintf( buf, sizeof( buf ), _( "<i>No files selected</i>" ) );
+    else {
+        tr_strlsize( sizeStr, builder->totalSize, sizeof(sizeStr) );
+        g_snprintf( buf, sizeof( buf ),
+                    /* %1$s is the torrent size
+                       %2$'d is its number of files */
+                    ngettext( "<i>%1$s; %2$'d File</i>",
+                              "<i>%1$s; %2$'d Files</i>", builder->fileCount ),
+                    sizeStr, builder->fileCount );
+    }
     gtk_label_set_markup ( GTK_LABEL(ui->size_lb), buf );
 
-    tr_strlsize( sizeStr, pieceSize, sizeof(sizeStr) );
-    g_snprintf( buf, sizeof(buf), "<i>%d %s @ %s</i>",
-                pieceCount,
-                ngettext("Piece", "Pieces", pieceCount),
-                sizeStr );
+    if( !filename )
+        *buf = '\0';
+    else {
+        tr_strlsize( sizeStr, builder->pieceSize, sizeof(sizeStr) );
+        g_snprintf( buf, sizeof( buf ),
+                    /* %1$'s is number of pieces;
+                       %2$s is how big each piece is */
+                    ngettext( "<i>%1$'d Piece @ %2$s</i>",
+                              "<i>%1$'d Pieces @ %2$s</i>",
+                              builder->pieceCount ),
+                    builder->pieceCount, sizeStr );
+    }
     gtk_label_set_markup ( GTK_LABEL(ui->pieces_lb), buf );
 }
 
 static void
-file_chooser_shown_cb( GtkWidget *w, gpointer folder_toggle )
+onSelectionChanged( GtkFileChooser * chooser, gpointer gui )
 {
-    const gboolean isFolder = gtk_toggle_button_get_active( folder_toggle );
-    gtk_file_chooser_set_action (GTK_FILE_CHOOSER(w), isFolder
-        ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER
-        : GTK_FILE_CHOOSER_ACTION_OPEN );
+    MakeMetaUI * ui = gui;
+    char * filename = gtk_file_chooser_get_filename( chooser );
+    tr_metainfo_builder * builder = tr_metaInfoBuilderCreate( ui->handle, filename );
+
+    g_object_set_data_full( G_OBJECT( chooser ),
+                            BUILDER_KEY,
+                            builder,
+                            (GDestroyNotify)tr_metaInfoBuilderFree );
+    ui->builder = builder;
+
+    refreshFromBuilder( ui );
+
+    g_free( filename );
 }
 
+static void
+onFileModeToggled( GtkToggleButton * t, gpointer w )
+{
+    const gboolean active = gtk_toggle_button_get_active( t );
+    gtk_widget_set_sensitive( w, active );
+    if( active )
+    {
+        MakeMetaUI * ui = g_object_get_data( G_OBJECT( w ), UI_KEY );
+        ui->builder = g_object_get_data( G_OBJECT( w ), BUILDER_KEY );
+        refreshFromBuilder( ui );
+    }
+}
+    
 GtkWidget*
 make_meta_ui( GtkWindow * parent, tr_handle * handle )
 {
+    GSList * group;
     int row = 0;
-    GtkWidget *d, *t, *w, *h, *rb_file, *rb_dir;
+    GtkWidget *l, *d, *t, *w, *h;
     GtkBox * main_vbox;
-    char name[256];
     MakeMetaUI * ui = g_new0 ( MakeMetaUI, 1 );
     ui->handle = handle;
 
-    d = gtk_dialog_new_with_buttons( _("Create New Torrent"),
+    d = gtk_dialog_new_with_buttons( _("New Torrent"),
                                      parent,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -246,34 +289,27 @@ make_meta_ui( GtkWindow * parent, tr_handle * handle )
 
     t = hig_workarea_create ();
 
-    hig_workarea_add_section_title (t, &row, _("Files"));
-    hig_workarea_add_section_spacer (t, row, 3);
+    hig_workarea_add_section_title (t, &row, _( "Content" ));
 
-        g_snprintf( name, sizeof(name), "%s:", _("File _Type"));
-        h = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
-        w = rb_dir = gtk_radio_button_new_with_mnemonic( NULL, _("Directory"));
-        gtk_box_pack_start ( GTK_BOX(h), w, FALSE, FALSE, 0 );
-        w = rb_file = gtk_radio_button_new_with_mnemonic_from_widget( GTK_RADIO_BUTTON(w), _("Single File") );
-        gtk_box_pack_start ( GTK_BOX(h), w, FALSE, FALSE, 0 );
-        hig_workarea_add_row (t, &row, name, h, NULL);
+        l = gtk_radio_button_new_with_mnemonic( NULL, _( "_Single File:" ) );
+        w = gtk_file_chooser_button_new( NULL, GTK_FILE_CHOOSER_ACTION_OPEN );
+        g_object_set_data( G_OBJECT( w ), UI_KEY, ui );
+        hig_workarea_add_row_w( t, &row, l, w, NULL );
+        group = gtk_radio_button_get_group( GTK_RADIO_BUTTON( l ) );
+        g_signal_connect( l, "toggled", G_CALLBACK(onFileModeToggled), w );
+        g_signal_connect( w, "selection-changed", G_CALLBACK(onSelectionChanged), ui );
 
-        g_snprintf( name, sizeof(name), "%s:", _("_File"));
+        l = gtk_radio_button_new_with_mnemonic( group, _( "_Folder:" ) );
+        w = gtk_file_chooser_button_new( NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER );
+        g_object_set_data( G_OBJECT( w ), UI_KEY, ui );
+        hig_workarea_add_row_w( t, &row, l, w, NULL );
+        g_signal_connect( l, "toggled", G_CALLBACK(onFileModeToggled), w );
+        g_signal_connect( w, "selection-changed", G_CALLBACK(onSelectionChanged), ui );
+        gtk_widget_set_sensitive( w, FALSE );
 
-        w = gtk_file_chooser_dialog_new ( NULL,
-                                          NULL,
-                                          GTK_FILE_CHOOSER_ACTION_OPEN,
-                                          GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-                                          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                                          NULL );
-        g_signal_connect( w, "map", G_CALLBACK(file_chooser_shown_cb), rb_dir );
-        w = gtk_file_chooser_button_new_with_dialog( w );
-        g_signal_connect( w, "selection-changed", G_CALLBACK(file_selection_changed_cb), ui );
-        hig_workarea_add_row (t, &row, name, w, NULL);
-
-        g_snprintf( name, sizeof(name), "<i>%s</i>", _("No Files Selected"));
         h = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
         w = ui->size_lb = gtk_label_new (NULL);
-        gtk_label_set_markup ( GTK_LABEL(w), name );
+        gtk_label_set_markup ( GTK_LABEL(w), _( "<i>No files selected</i>" ) );
         gtk_box_pack_start( GTK_BOX(h), w, FALSE, FALSE, 0 );
         w = ui->pieces_lb = gtk_label_new (NULL);
         gtk_box_pack_end( GTK_BOX(h), w, FALSE, FALSE, 0 );
@@ -284,20 +320,16 @@ make_meta_ui( GtkWindow * parent, tr_handle * handle )
         
 
     hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title (t, &row, _("Torrent"));
-    hig_workarea_add_section_spacer (t, row, 3);
+    hig_workarea_add_section_title (t, &row, _("Details"));
 
-        g_snprintf( name, sizeof(name), _("Private to this Tracker") );
-        w = ui->private_check = hig_workarea_add_wide_checkbutton( t, &row, name, FALSE );
+        w = ui->private_check = hig_workarea_add_wide_checkbutton( t, &row, _( "_Private to this tracker" ), FALSE );
 
-        g_snprintf( name, sizeof(name), "%s:", _("Announce _URL"));
         w = ui->announce_entry = gtk_entry_new( );
         gtk_entry_set_text(GTK_ENTRY(w), "http://");
-        hig_workarea_add_row (t, &row, name, w, NULL );
+        hig_workarea_add_row (t, &row, _( "Announce _URL:" ), w, NULL );
 
-        g_snprintf( name, sizeof(name), "%s:", _("Commen_t"));
         w = ui->comment_entry = gtk_entry_new( );
-        hig_workarea_add_row (t, &row, name, w, NULL );
+        hig_workarea_add_row (t, &row, _( "Commen_t:" ), w, NULL );
 
     hig_workarea_finish( t, &row );
     gtk_box_pack_start_defaults( main_vbox, t );
@@ -307,7 +339,7 @@ make_meta_ui( GtkWindow * parent, tr_handle * handle )
     gtk_container_set_border_width( GTK_CONTAINER( w ), GUI_PAD_BIG );
     
         ui->progressbar = gtk_progress_bar_new( );
-        gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar), _( "No Files Selected" ) );
+        gtk_progress_bar_set_text( GTK_PROGRESS_BAR( ui->progressbar), _( "No files selected" ) );
         gtk_container_add( GTK_CONTAINER( w ), ui->progressbar );
 
     gtk_box_pack_start( main_vbox, w, FALSE, FALSE, GUI_PAD_BIG );
