@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  * 
- * $Id:$
+ * $Id: stats.h 5126 2008-02-26 21:08:51Z charles $
  */
 
 #ifndef __TR_GTK_STATS_H__
@@ -15,7 +15,7 @@
 
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
-#include "tr_core.h"
+#include "tr-core.h"
 
 GtkWidget* stats_dialog_create( GtkWindow * parent,
                                 TrCore    * core );
