@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: bencode.c 4892 2008-02-01 15:50:12Z charles $
+ * $Id: bencode.c 5772 2008-05-08 03:25:21Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -33,6 +33,7 @@
 
 #include "transmission.h"
 #include "bencode.h"
+#include "list.h"
 #include "ptrarray.h"
 #include "utils.h" /* tr_new(), tr_free() */
 
@@ -40,41 +41,27 @@
 ***
 **/
 
-static int
-isType( const benc_val_t * val, int type )
+int
+tr_bencIsType( const tr_benc * val, int type )
 {
     return ( ( val != NULL ) && ( val->type == type ) );
 }
 
-#define isInt(v)    ( isType( ( v ), TYPE_INT ) )
-#define isString(v) ( isType( ( v ), TYPE_STR ) )
-#define isList(v)   ( isType( ( v ), TYPE_LIST ) )
-#define isDict(v)   ( isType( ( v ), TYPE_DICT ) )
-
 static int
-isContainer( const benc_val_t * val )
+isContainer( const tr_benc * val )
 {
-    return isList(val) || isDict(val);
+    return tr_bencIsList(val) || tr_bencIsDict(val);
 }
 static int
-isSomething( const benc_val_t * val )
+isSomething( const tr_benc * val )
 {
-    return isContainer(val) || isInt(val) || isString(val);
+    return isContainer(val) || tr_bencIsInt(val) || tr_bencIsString(val);
 }
 
-benc_val_t*
-tr_bencListGetNthChild( benc_val_t * val, int i )
-{
-    benc_val_t * ret = NULL;
-    if( isList( val ) && ( i >= 0 ) && ( i < val->val.l.count ) )
-        ret = val->val.l.vals + i;
-    return ret;
-}
-
-
-/**
-***
-**/
+/***
+****  tr_bencParse()
+****  tr_bencLoad()
+***/
 
 /**
  * The initial i and trailing e are beginning and ending delimiters.
@@ -167,7 +154,7 @@ tr_bencParseStr( const uint8_t  * buf,
 #define LIST_SIZE 8 /* number of items to increment list/dict buffer by */
 
 static int
-makeroom( benc_val_t * val, int count )
+makeroom( tr_benc * val, int count )
 {
     assert( TYPE_LIST == val->type || TYPE_DICT == val->type );
 
@@ -176,7 +163,7 @@ makeroom( benc_val_t * val, int count )
         /* We need a bigger boat */
         const int len = val->val.l.alloc + count +
             ( count % LIST_SIZE ? LIST_SIZE - ( count % LIST_SIZE ) : 0 );
-        void * new = realloc( val->val.l.vals, len * sizeof( benc_val_t ) );
+        void * new = realloc( val->val.l.vals, len * sizeof( tr_benc ) );
         if( NULL == new )
             return 1;
 
@@ -187,10 +174,10 @@ makeroom( benc_val_t * val, int count )
     return 0;
 }
 
-static benc_val_t*
-getNode( benc_val_t * top, tr_ptrArray * parentStack, int type )
+static tr_benc*
+getNode( tr_benc * top, tr_ptrArray * parentStack, int type )
 {
-    benc_val_t * parent;
+    tr_benc * parent;
 
     assert( top != NULL );
     assert( parentStack != NULL );
@@ -219,7 +206,7 @@ getNode( benc_val_t * top, tr_ptrArray * parentStack, int type )
 int
 tr_bencParse( const void     * buf_in,
               const void     * bufend_in,
-              benc_val_t     * top,
+              tr_benc        * top,
               const uint8_t ** setme_end )
 {
     int err;
@@ -239,7 +226,7 @@ tr_bencParse( const void     * buf_in,
             int64_t val;
             const uint8_t * end;
             int err;
-            benc_val_t * node;
+            tr_benc * node;
 
             if(( err = tr_bencParseInt( buf, bufend, &end, &val )))
                 return err;
@@ -256,7 +243,7 @@ tr_bencParse( const void     * buf_in,
         }
         else if( *buf=='l' ) /* list */
         {
-            benc_val_t * node = getNode( top, parentStack, TYPE_LIST );
+            tr_benc * node = getNode( top, parentStack, TYPE_LIST );
             if( !node )
                 return TR_ERROR;
             tr_bencInit( node, TYPE_LIST );
@@ -265,7 +252,7 @@ tr_bencParse( const void     * buf_in,
         }
         else if( *buf=='d' ) /* dict */
         {
-            benc_val_t * node = getNode( top, parentStack, TYPE_DICT );
+            tr_benc * node = getNode( top, parentStack, TYPE_DICT );
             if( !node )
                 return TR_ERROR;
             tr_bencInit( node, TYPE_DICT );
@@ -274,13 +261,13 @@ tr_bencParse( const void     * buf_in,
         }
         else if( *buf=='e' ) /* end of list or dict */
         {
-            benc_val_t * node;
+            tr_benc * node;
             ++buf;
             if( tr_ptrArrayEmpty( parentStack ) )
                 return TR_ERROR;
 
             node = tr_ptrArrayBack( parentStack );
-            if( isDict( node ) && ( node->val.l.count % 2 ) )
+            if( tr_bencIsDict( node ) && ( node->val.l.count % 2 ) )
                 return TR_ERROR; /* odd # of children in dict */
 
             tr_ptrArrayPop( parentStack );
@@ -293,7 +280,7 @@ tr_bencParse( const void     * buf_in,
             uint8_t * str;
             size_t str_len;
             int err;
-            benc_val_t * node;
+            tr_benc * node;
 
             if(( err = tr_bencParseStr( buf, bufend, &end, &str, &str_len )))
                 return err;
@@ -326,7 +313,7 @@ tr_bencParse( const void     * buf_in,
 int
 tr_bencLoad( const void  * buf_in,
              int           buflen,
-             benc_val_t  * setme_benc,
+             tr_benc     * setme_benc,
              char       ** setme_end )
 {
     const uint8_t * buf = buf_in;
@@ -337,15 +324,17 @@ tr_bencLoad( const void  * buf_in,
     return ret;
 }
 
-benc_val_t *
-tr_bencDictFind( benc_val_t * val, const char * key )
+/***
+****
+***/
+
+tr_benc *
+tr_bencDictFind( tr_benc * val, const char * key )
 {
     int len, ii;
 
-    if( val->type != TYPE_DICT )
-    {
+    if( !tr_bencIsDict( val ) )
         return NULL;
-    }
 
     len = strlen( key );
     
@@ -363,52 +352,125 @@ tr_bencDictFind( benc_val_t * val, const char * key )
     return NULL;
 }
 
-benc_val_t*
-tr_bencDictFindType( benc_val_t * val, const char * key, int type )
+tr_benc*
+tr_bencDictFindType( tr_benc * val, const char * key, int type )
 {
-    benc_val_t * ret = tr_bencDictFind( val, key );
+    tr_benc * ret = tr_bencDictFind( val, key );
     return ret && ret->type == type ? ret : NULL;
 }
 
-int64_t
-tr_bencGetInt ( const benc_val_t * val )
-{
-    assert( isInt( val ) );
-    return val->val.i;
-}
-
-benc_val_t *
-tr_bencDictFindFirst( benc_val_t * val, ... )
+tr_benc *
+tr_bencDictFindFirst( tr_benc * val, ... )
 {
     const char * key;
-    benc_val_t * ret;
+    tr_benc * ret;
     va_list      ap;
 
     ret = NULL;
     va_start( ap, val );
-    while( ( key = va_arg( ap, const char * ) ) )
-    {
-        ret = tr_bencDictFind( val, key );
-        if( NULL != ret )
-        {
+    while(( key = va_arg( ap, const char * )))
+        if(( ret = tr_bencDictFind( val, key )))
             break;
-        }
-    }
     va_end( ap );
 
     return ret;
 }
 
-char *
-tr_bencStealStr( benc_val_t * val )
+int
+tr_bencListSize( const tr_benc * list )
 {
-    assert( TYPE_STR == val->type );
-    val->val.s.nofree = 1;
-    return val->val.s.s;
+    return tr_bencIsList( list ) ? list->val.l.count : 0;
 }
 
+tr_benc*
+tr_bencListChild( tr_benc * val, int i )
+{
+    tr_benc * ret = NULL;
+    if( tr_bencIsList( val ) && ( i >= 0 ) && ( i < val->val.l.count ) )
+        ret = val->val.l.vals + i;
+    return ret;
+}
+
+int
+tr_bencGetInt ( const tr_benc * val, int64_t * setme )
+{
+    const int success = tr_bencIsInt( val );
+    if( success )
+        *setme = val->val.i ;
+    return success;
+}
+
+int
+tr_bencGetStr( const tr_benc * val, const char ** setme )
+{
+    const int success = tr_bencIsString( val );
+    if( success )
+        *setme = val->val.s.s;
+    return success;
+}
+
+int
+tr_bencDictFindInt( tr_benc * dict, const char * key, int64_t * setme )
+{
+    int found = FALSE;
+    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_INT );
+    if( child )
+        found = tr_bencGetInt( child, setme );
+    return found;
+}
+
+int
+tr_bencDictFindDouble( tr_benc * dict, const char * key, double * setme )
+{
+    const char * str;
+    const int success = tr_bencDictFindStr( dict, key, &str );
+    if( success )
+        *setme = strtod( str, NULL );
+    return success;
+}
+
+int
+tr_bencDictFindList( tr_benc * dict, const char * key, tr_benc ** setme )
+{
+    int found = FALSE;
+    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_LIST );
+    if( child ) {
+        *setme = child;
+        found = TRUE;
+    }
+    return found;
+}
+
+int
+tr_bencDictFindDict( tr_benc * dict, const char * key, tr_benc ** setme )
+{
+    int found = FALSE;
+    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_DICT );
+    if( child ) {
+        *setme = child;
+        found = TRUE;
+    }
+    return found;
+}
+
+int
+tr_bencDictFindStr( tr_benc * dict, const char * key, const char ** setme )
+{
+    int found = FALSE;
+    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
+    if( child ) {
+        *setme = child->val.s.s;
+        found = TRUE;
+    }
+    return found;
+}
+
+/***
+****
+***/
+
 void
-_tr_bencInitStr( benc_val_t * val, char * str, int len, int nofree )
+_tr_bencInitStr( tr_benc * val, char * str, int len, int nofree )
 {
     tr_bencInit( val, TYPE_STR );
     val->val.s.s      = str;
@@ -420,8 +482,18 @@ _tr_bencInitStr( benc_val_t * val, char * str, int len, int nofree )
     val->val.s.i = len;
 }
 
+void
+tr_bencInitRaw( tr_benc * val, const void * src, size_t byteCount )
+{
+    tr_bencInit( val, TYPE_STR );
+    val->val.s.i = byteCount;
+    val->val.s.s = tr_new( char, byteCount );
+    val->val.s.nofree = 0;
+    memcpy( val->val.s.s, src, byteCount );
+}
+
 int
-tr_bencInitStrDup( benc_val_t * val, const char * str )
+tr_bencInitStrDup( tr_benc * val, const char * str )
 {
     char * newStr = tr_strdup( str );
     if( newStr == NULL )
@@ -432,34 +504,50 @@ tr_bencInitStrDup( benc_val_t * val, const char * str )
 }
 
 void
-tr_bencInitInt( benc_val_t * val, int64_t num )
+tr_bencInitInt( tr_benc * val, int64_t num )
 {
     tr_bencInit( val, TYPE_INT );
     val->val.i = num;
 }
 
 int
-tr_bencListReserve( benc_val_t * val, int count )
+tr_bencInitList( tr_benc * val, int reserveCount )
 {
-    assert( TYPE_LIST == val->type );
+    tr_bencInit( val, TYPE_LIST );
+    return tr_bencListReserve( val, reserveCount );
+}
 
+int
+tr_bencListReserve( tr_benc * val, int count )
+{
+    assert( tr_bencIsList( val ) );
     return makeroom( val, count );
 }
 
 int
-tr_bencDictReserve( benc_val_t * val, int count )
+tr_bencInitDict( tr_benc * val, int reserveCount )
 {
-    assert( TYPE_DICT == val->type );
+    tr_bencInit( val, TYPE_DICT );
+    return tr_bencDictReserve( val, reserveCount );
+}
 
+int
+tr_bencDictReserve( tr_benc * val, int count )
+{
+    assert( tr_bencIsDict( val ) );
     return makeroom( val, count * 2 );
 }
 
-benc_val_t *
-tr_bencListAdd( benc_val_t * list )
+tr_benc *
+tr_bencListAdd( tr_benc * list )
 {
-    benc_val_t * item;
+    tr_benc * item;
+
+    assert( tr_bencIsList( list ) );
+
+    if( list->val.l.count == list->val.l.alloc )
+        tr_bencListReserve( list, LIST_SIZE );
 
-    assert( isList( list ) );
     assert( list->val.l.count < list->val.l.alloc );
 
     item = &list->val.l.vals[list->val.l.count];
@@ -468,13 +556,41 @@ tr_bencListAdd( benc_val_t * list )
 
     return item;
 }
-
-benc_val_t *
-tr_bencDictAdd( benc_val_t * dict, const char * key )
+tr_benc *
+tr_bencListAddInt( tr_benc * list, int64_t val )
 {
-    benc_val_t * keyval, * itemval;
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitInt( node, val );
+    return node;
+}
+tr_benc *
+tr_bencListAddStr( tr_benc * list, const char * val )
+{
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitStrDup( node, val );
+    return node;
+}
+tr_benc*
+tr_bencListAddList( tr_benc * list, int reserveCount )
+{
+    tr_benc * child = tr_bencListAdd( list );
+    tr_bencInitList( child, reserveCount );
+    return child;
+}
+tr_benc*
+tr_bencListAddDict( tr_benc * list, int reserveCount )
+{
+    tr_benc * child = tr_bencListAdd( list );
+    tr_bencInitDict( child, reserveCount );
+    return child;
+}
 
-    assert( isDict( dict ) );
+tr_benc *
+tr_bencDictAdd( tr_benc * dict, const char * key )
+{
+    tr_benc * keyval, * itemval;
+
+    assert( tr_bencIsDict( dict ) );
     assert( dict->val.l.count + 2 <= dict->val.l.alloc );
 
     keyval = dict->val.l.vals + dict->val.l.count++;
@@ -485,7 +601,48 @@ tr_bencDictAdd( benc_val_t * dict, const char * key )
 
     return itemval;
 }
-
+tr_benc*
+tr_bencDictAddInt( tr_benc * dict, const char * key, int64_t val )
+{
+    tr_benc * child = tr_bencDictAdd( dict, key );
+    tr_bencInitInt( child, val );
+    return child;
+}
+tr_benc*
+tr_bencDictAddStr( tr_benc * dict, const char * key, const char * val )
+{
+    tr_benc * child = tr_bencDictAdd( dict, key );
+    tr_bencInitStrDup( child, val );
+    return child;
+}
+tr_benc*
+tr_bencDictAddDouble( tr_benc * dict, const char * key, double d )
+{
+    char buf[128];
+    snprintf( buf, sizeof( buf ), "%f", d );
+    return tr_bencDictAddStr( dict, key, buf );
+}
+tr_benc*
+tr_bencDictAddList( tr_benc * dict, const char * key, int reserveCount )
+{
+    tr_benc * child = tr_bencDictAdd( dict, key );
+    tr_bencInitList( child, reserveCount );
+    return child;
+}
+tr_benc*
+tr_bencDictAddDict( tr_benc * dict, const char * key, int reserveCount )
+{
+    tr_benc * child = tr_bencDictAdd( dict, key );
+    tr_bencInitDict( child, reserveCount );
+    return child;
+}
+tr_benc*
+tr_bencDictAddRaw( tr_benc * dict, const char * key, const void * src, size_t len )
+{
+    tr_benc * child = tr_bencDictAdd( dict, key );
+    tr_bencInitRaw( child, src, len );
+    return child;
+}
 
 /***
 ****  BENC WALKING
@@ -507,7 +664,7 @@ compareKeyIndex( const void * va, const void * vb )
 
 struct SaveNode
 {
-    const benc_val_t * val;
+    const tr_benc * val;
     int valIsVisited;
     int childCount;
     int childIndex;
@@ -515,14 +672,14 @@ struct SaveNode
 };
 
 static struct SaveNode*
-nodeNewDict( const benc_val_t * val )
+nodeNewDict( const tr_benc * val )
 {
     int i, j;
     int nKeys;
     struct SaveNode * node;
     struct KeyIndex * indices;
 
-    assert( isDict( val ) );
+    assert( tr_bencIsDict( val ) );
 
     nKeys = val->val.l.count / 2;
     node = tr_new0( struct SaveNode, 1 );
@@ -548,12 +705,12 @@ nodeNewDict( const benc_val_t * val )
 }
 
 static struct SaveNode*
-nodeNewList( const benc_val_t * val )
+nodeNewList( const tr_benc * val )
 {
     int i, n;
     struct SaveNode * node;
 
-    assert( isList( val ) );
+    assert( tr_bencIsList( val ) );
 
     n = val->val.l.count;
     node = tr_new0( struct SaveNode, 1 );
@@ -567,7 +724,7 @@ nodeNewList( const benc_val_t * val )
 }
 
 static struct SaveNode*
-nodeNewLeaf( const benc_val_t * val )
+nodeNewLeaf( const tr_benc * val )
 {
     struct SaveNode * node;
 
@@ -579,13 +736,13 @@ nodeNewLeaf( const benc_val_t * val )
 }
 
 static struct SaveNode*
-nodeNew( const benc_val_t * val )
+nodeNew( const tr_benc * val )
 {
     struct SaveNode * node;
 
-    if( val->type == TYPE_LIST )
+    if( tr_bencIsList( val ) )
         node = nodeNewList( val );
-    else if( val->type == TYPE_DICT )
+    else if( tr_bencIsDict( val ) )
         node = nodeNewDict( val );
     else
         node = nodeNewLeaf( val );
@@ -593,7 +750,7 @@ nodeNew( const benc_val_t * val )
     return node;
 }
 
-typedef void (*BencWalkFunc)( const benc_val_t * val, void * user_data );
+typedef void (*BencWalkFunc)( const tr_benc * val, void * user_data );
 
 struct WalkFuncs
 {
@@ -610,7 +767,7 @@ struct WalkFuncs
  * attack via maliciously-crafted bencoded data. (#667)
  */
 static void
-bencWalk( const benc_val_t   * top,
+bencWalk( const tr_benc   * top,
           struct WalkFuncs   * walkFuncs,
           void               * user_data )
 {
@@ -620,7 +777,7 @@ bencWalk( const benc_val_t   * top,
     while( !tr_ptrArrayEmpty( stack ) )
     {
         struct SaveNode * node = tr_ptrArrayBack( stack );
-        const benc_val_t * val;
+        const tr_benc * val;
 
         if( !node->valIsVisited )
         {
@@ -668,7 +825,7 @@ bencWalk( const benc_val_t   * top,
 
             default:
                 /* did caller give us an uninitialized val? */
-                tr_err( "Invalid benc type %d", val->type );
+                tr_err( _( "Invalid metadata" ) );
                 break;
         }
     }
@@ -681,34 +838,34 @@ bencWalk( const benc_val_t   * top,
 ****/
 
 static void
-saveIntFunc( const benc_val_t * val, void * evbuf )
+saveIntFunc( const tr_benc * val, void * evbuf )
 {
-    evbuffer_add_printf( evbuf, "i%"PRId64"e", tr_bencGetInt(val) );
+    evbuffer_add_printf( evbuf, "i%"PRId64"e", val->val.i );
 }
 static void
-saveStringFunc( const benc_val_t * val, void * vevbuf )
+saveStringFunc( const tr_benc * val, void * vevbuf )
 {
     struct evbuffer * evbuf = vevbuf;
-    evbuffer_add_printf( evbuf, "%i:", val->val.s.i );
+    evbuffer_add_printf( evbuf, "%d:", val->val.s.i );
     evbuffer_add( evbuf, val->val.s.s, val->val.s.i );
 }
 static void
-saveDictBeginFunc( const benc_val_t * val UNUSED, void * evbuf )
+saveDictBeginFunc( const tr_benc * val UNUSED, void * evbuf )
 {
     evbuffer_add_printf( evbuf, "d" );
 }
 static void
-saveListBeginFunc( const benc_val_t * val UNUSED, void * evbuf )
+saveListBeginFunc( const tr_benc * val UNUSED, void * evbuf )
 {
     evbuffer_add_printf( evbuf, "l" );
 }
 static void
-saveContainerEndFunc( const benc_val_t * val UNUSED, void * evbuf )
+saveContainerEndFunc( const tr_benc * val UNUSED, void * evbuf )
 {
     evbuffer_add_printf( evbuf, "e" );
 }
 char*
-tr_bencSave( const benc_val_t * top, int * len )
+tr_bencSave( const tr_benc * top, int * len )
 {
     char * ret;
     struct WalkFuncs walkFuncs;
@@ -733,22 +890,22 @@ tr_bencSave( const benc_val_t * top, int * len )
 ***/
 
 static void
-freeDummyFunc( const benc_val_t * val UNUSED, void * buf UNUSED  )
+freeDummyFunc( const tr_benc * val UNUSED, void * buf UNUSED  )
 {
 }
 static void
-freeStringFunc( const benc_val_t * val, void * freeme )
+freeStringFunc( const tr_benc * val, void * freeme )
 {
     if( !val->val.s.nofree )
         tr_ptrArrayAppend( freeme, val->val.s.s );
 }
 static void
-freeContainerBeginFunc( const benc_val_t * val, void * freeme )
+freeContainerBeginFunc( const tr_benc * val, void * freeme )
 {
     tr_ptrArrayAppend( freeme, val->val.l.vals );
 }
 void
-tr_bencFree( benc_val_t * val )
+tr_bencFree( tr_benc * val )
 {
     if( val != NULL )
     {
@@ -782,14 +939,14 @@ printLeadingSpaces( struct WalkPrint * data )
     fprintf( data->out, "%*.*s", width, width, " " );
 }
 static void
-printIntFunc( const benc_val_t * val, void * vdata )
+printIntFunc( const tr_benc * val, void * vdata )
 {
     struct WalkPrint * data = vdata;
     printLeadingSpaces( data );
-    fprintf( data->out, "int:  %"PRId64"\n", tr_bencGetInt(val) );
+    fprintf( data->out, "int:  %"PRId64"\n", val->val.i );
 }
 static void
-printStringFunc( const benc_val_t * val, void * vdata )
+printStringFunc( const tr_benc * val, void * vdata )
 {
     int ii;
     struct WalkPrint * data = vdata;
@@ -809,7 +966,7 @@ printStringFunc( const benc_val_t * val, void * vdata )
     fprintf( data->out, "\n" );
 }
 static void
-printListBeginFunc( const benc_val_t * val UNUSED, void * vdata )
+printListBeginFunc( const tr_benc * val UNUSED, void * vdata )
 {
     struct WalkPrint * data = vdata;
     printLeadingSpaces( data );
@@ -817,7 +974,7 @@ printListBeginFunc( const benc_val_t * val UNUSED, void * vdata )
     ++data->depth;
 }
 static void
-printDictBeginFunc( const benc_val_t * val UNUSED, void * vdata )
+printDictBeginFunc( const tr_benc * val UNUSED, void * vdata )
 {
     struct WalkPrint * data = vdata;
     printLeadingSpaces( data );
@@ -825,13 +982,13 @@ printDictBeginFunc( const benc_val_t * val UNUSED, void * vdata )
     ++data->depth;
 }
 static void
-printContainerEndFunc( const benc_val_t * val UNUSED, void * vdata )
+printContainerEndFunc( const tr_benc * val UNUSED, void * vdata )
 {
     struct WalkPrint * data = vdata;
     --data->depth;
 }
 void
-tr_bencPrint( benc_val_t * val )
+tr_bencPrint( const tr_benc * val )
 {
     struct WalkFuncs walkFuncs;
     struct WalkPrint walkPrint;
@@ -846,3 +1003,154 @@ tr_bencPrint( benc_val_t * val )
     walkPrint.depth = 0;
     bencWalk( val, &walkFuncs, &walkPrint );
 }
+
+/***
+****
+***/
+
+struct ParentState
+{
+    int type;
+    int index;
+};
+ 
+struct phpWalk
+{
+    tr_list * parents;
+    struct evbuffer * out;
+};
+
+static void
+phpChildFunc( struct phpWalk * data )
+{
+    if( data->parents )
+    {
+        struct ParentState * parentState = data->parents->data;
+
+        if( parentState->type == TYPE_LIST )
+            evbuffer_add_printf( data->out, "i:%d;", parentState->index++ );
+    }
+}
+
+static void
+phpPushParent( struct phpWalk * data, int type )
+{
+    struct ParentState * parentState = tr_new( struct ParentState, 1 );
+    parentState->type = type;
+    parentState->index = 0;
+    tr_list_prepend( &data->parents, parentState );
+}
+
+static void
+phpPopParent( struct phpWalk * data )
+{
+    tr_free( tr_list_pop_front( &data->parents ) );
+}
+
+static void
+phpIntFunc( const tr_benc * val, void * vdata )
+{
+    struct phpWalk * data = vdata;
+    phpChildFunc( data );
+    evbuffer_add_printf( data->out, "i:%"PRId64";", val->val.i );
+}
+static void
+phpStringFunc( const tr_benc * val, void * vdata )
+{
+    struct phpWalk * data = vdata;
+    phpChildFunc( data );
+    evbuffer_add_printf( data->out, "s:%d:\"%s\";", val->val.s.i, val->val.s.s );
+}
+static void
+phpDictBeginFunc( const tr_benc * val, void * vdata )
+{
+    struct phpWalk * data = vdata;
+    phpChildFunc( data );
+    phpPushParent( data, TYPE_DICT );
+    evbuffer_add_printf( data->out, "a:%d:{", val->val.l.count/2 );
+}
+static void
+phpListBeginFunc( const tr_benc * val, void * vdata )
+{
+    struct phpWalk * data = vdata;
+    phpChildFunc( data );
+    phpPushParent( data, TYPE_LIST );
+    evbuffer_add_printf( data->out, "a:%d:{", val->val.l.count );
+}
+static void
+phpContainerEndFunc( const tr_benc * val UNUSED, void * vdata )
+{
+    struct phpWalk * data = vdata;
+    phpPopParent( data );
+    evbuffer_add_printf( data->out, "}" );
+}
+char*
+tr_bencSaveAsSerializedPHP( const tr_benc * top, int * len )
+{
+    char * ret;
+    struct WalkFuncs walkFuncs;
+    struct phpWalk data;
+
+    data.out = evbuffer_new( );
+    data.parents = NULL;
+
+    walkFuncs.intFunc = phpIntFunc;
+    walkFuncs.stringFunc = phpStringFunc;
+    walkFuncs.dictBeginFunc = phpDictBeginFunc;
+    walkFuncs.listBeginFunc = phpListBeginFunc;
+    walkFuncs.containerEndFunc = phpContainerEndFunc;
+
+    bencWalk( top, &walkFuncs, &data );
+    
+    if( len != NULL )
+        *len = EVBUFFER_LENGTH( data.out );
+    ret = tr_strndup( (char*) EVBUFFER_DATA( data.out ), EVBUFFER_LENGTH( data.out ) );
+    evbuffer_free( data.out );
+    return ret;
+}
+
+/***
+****
+***/
+
+int
+tr_bencSaveFile( const char * filename,  const tr_benc * b )
+{
+    int err = TR_OK;
+    int len;
+    char * content = tr_bencSave( b, &len );
+    FILE * out = NULL;
+
+    out = fopen( filename, "wb+" );
+    if( !out )
+    {
+        tr_err( _( "Couldn't open \"%1$s\": %2$s" ),
+                filename, tr_strerror( errno ) );
+        err = TR_EINVALID;
+    }
+    else if( fwrite( content, sizeof( char ), len, out ) != (size_t)len )
+    {
+        tr_err( _( "Couldn't save file \"%1$s\": %2$s" ),
+                filename, tr_strerror( errno ) );
+        err = TR_EINVALID;
+    }
+
+    if( !err )
+        tr_dbg( "tr_bencSaveFile saved \"%s\"", filename );
+    tr_free( content );
+    if( out )
+        fclose( out );
+    return err;
+}
+
+int
+tr_bencLoadFile( const char * filename, tr_benc * b )
+{
+    int ret;
+    size_t contentLen;
+    uint8_t * content = tr_loadFile( filename, &contentLen );
+    ret = content ? tr_bencLoad( content, contentLen, b, NULL )
+                  : TR_ERROR_IO_OTHER;
+    tr_free( content );
+    return ret;
+}
