@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: completion.h 4404 2008-01-01 17:20:20Z charles $
+ * $Id: completion.h 5682 2008-04-24 15:25:01Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -40,20 +40,23 @@ uint64_t         tr_cpHaveTotal( const tr_completion * );
 uint64_t         tr_cpHaveValid( const tr_completion * );
 uint64_t         tr_cpLeftUntilComplete( const tr_completion * );
 uint64_t         tr_cpLeftUntilDone( const tr_completion * );
+uint64_t         tr_cpSizeWhenDone( const tr_completion * );
 float            tr_cpPercentComplete( const tr_completion * );
 float            tr_cpPercentDone( const tr_completion * );
 void             tr_cpInvalidateDND ( tr_completion * );
+void             tr_cpGetAmountDone( const tr_completion *, float * tab, int tabCount );
 
 /* Pieces */
-int              tr_cpPieceIsComplete( const tr_completion *, int piece );
-void             tr_cpPieceAdd( tr_completion *, int piece );
-void             tr_cpPieceRem( tr_completion *, int piece );
+int              tr_cpPieceIsComplete( const tr_completion *, tr_piece_index_t piece );
+void             tr_cpPieceAdd( tr_completion *, tr_piece_index_t piece );
+void             tr_cpPieceRem( tr_completion *, tr_piece_index_t piece );
 
 /* Blocks */
-int              tr_cpBlockIsComplete( const tr_completion *, int block );
-void             tr_cpBlockAdd( tr_completion *, int block );
-void             tr_cpBlockBitfieldSet( tr_completion *, struct tr_bitfield * );
-float            tr_cpPercentBlocksInPiece( const tr_completion * cp, int piece );
+int              tr_cpBlockIsComplete( const tr_completion *, tr_block_index_t block );
+void             tr_cpBlockAdd( tr_completion *, tr_block_index_t block );
+tr_errno         tr_cpBlockBitfieldSet( tr_completion *, struct tr_bitfield * );
+int              tr_cpMissingBlocksInPiece( const tr_completion * cp, tr_piece_index_t piece );
+
 
 const struct tr_bitfield * tr_cpPieceBitfield( const tr_completion* );
 const struct tr_bitfield * tr_cpBlockBitfield( const tr_completion * );
