@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fastresume.c 4885 2008-02-01 01:38:57Z charles $
+ * $Id: fastresume.c 5764 2008-05-06 20:31:05Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -44,22 +44,19 @@
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
+#include <stdlib.h> /* calloc */
+#include <string.h> /* strcpy, memset, memcmp */
 
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-
-#include <event.h>
+#include <sys/stat.h> /* stat */
+#include <unistd.h> /* unlink */
 
 #include "transmission.h"
 #include "completion.h"
 #include "fastresume.h"
-#include "internal.h" /* tr_torrentInitFileDLs */
 #include "peer-mgr.h"
 #include "platform.h"
+#include "resume.h" /* TR_FR_ bitwise enum */
 #include "torrent.h"
 #include "utils.h"
 
@@ -119,15 +116,16 @@ enum
 #define FR_MTIME_LEN( t ) \
   ( sizeof(tr_time_t) * (t)->info.fileCount )
 #define FR_BLOCK_BITFIELD_LEN( t ) \
-  ( ( (t)->blockCount + 7 ) / 8 )
+  ( ( (t)->blockCount + 7u ) / 8u )
 #define FR_PROGRESS_LEN( t ) \
   ( FR_MTIME_LEN( t ) + FR_BLOCK_BITFIELD_LEN( t ) )
 #define FR_SPEED_LEN (2 * (sizeof(uint16_t) + sizeof(uint8_t) ) )
 
+#if 0
 static void
 fastResumeFileName( char * buf, size_t buflen, const tr_torrent * tor, int tag )
 {
-    const char * cacheDir = tr_getCacheDirectory ();
+    const char * cacheDir = tr_getResumeDir( tor->handle );
     const char * hash = tor->info.hashString;
 
     if( !tag )
@@ -141,6 +139,7 @@ fastResumeFileName( char * buf, size_t buflen, const tr_torrent * tor, int tag )
         tr_buildPath( buf, buflen, cacheDir, base, NULL );
     }
 }
+#endif
 
 static tr_time_t*
 getMTimes( const tr_torrent * tor, int * setme_n )
@@ -167,6 +166,7 @@ getMTimes( const tr_torrent * tor, int * setme_n )
     return m;
 }
 
+#if 0
 static void
 fastResumeWriteData( uint8_t       id,
                      const void  * data,
@@ -191,8 +191,8 @@ tr_fastResumeSave( const tr_torrent * tor )
 
     fastResumeFileName( path, sizeof path, tor, 1 );
     file = fopen( path, "wb+" );
-    if( NULL == file ) {
-        tr_err( "Couldn't open '%s' for writing", path );
+    if( !file ) {
+        tr_torerr( tor, _( "Couldn't open \"%1$s\": %2$s" ), path, tr_strerror( errno ) );
         return;
     }
     
@@ -207,7 +207,7 @@ tr_fastResumeSave( const tr_torrent * tor )
     }
 
     /* Write progress data */
-    if (1) {
+    {
         int i, n;
         tr_time_t * mtimes;
         uint8_t * buf = malloc( FR_PROGRESS_LEN( tor ) );
@@ -327,9 +327,8 @@ tr_fastResumeSave( const tr_torrent * tor )
     }
 
     fclose( file );
-
-    /*tr_dbg( "Wrote resume file for '%s'", tor->info.name );*/
 }
+#endif
 
 /***
 ****
@@ -405,11 +404,11 @@ parseProgress( tr_torrent     * tor,
                const uint8_t  * buf,
                uint32_t         len )
 {
-    int i;
     uint64_t ret = 0;
    
     if( len == FR_PROGRESS_LEN( tor ) )
     {
+        int i;
         int n;
         tr_bitfield bitfield;
 
@@ -423,7 +422,7 @@ parseProgress( tr_torrent     * tor,
                 tr_torrentSetFileChecked( tor, i, TRUE );
             else {
                 tr_torrentSetFileChecked( tor, i, FALSE );
-                tr_dbg( "File '%s' recheck needed", tor->info.files[i].name );
+                tr_tordbg( tor, "Torrent needs to be verified" );
             }
         }
         free( curMTimes );
@@ -432,16 +431,22 @@ parseProgress( tr_torrent     * tor,
         memset( &bitfield, 0, sizeof bitfield );
         bitfield.len = FR_BLOCK_BITFIELD_LEN( tor );
         bitfield.bits = (uint8_t*) walk;
-        tr_cpBlockBitfieldSet( tor->completion, &bitfield );
-
-        ret = TR_FR_PROGRESS;
+        if( !tr_cpBlockBitfieldSet( tor->completion, &bitfield ) )
+            ret = TR_FR_PROGRESS;
+        else {
+            tr_torrentUncheck( tor );
+            tr_tordbg( tor, "Torrent needs to be verified" );
+        }
     }
 
     /* the files whose mtimes are wrong,
        remove from completion pending a recheck... */
-    for( i=0; i<tor->info.pieceCount; ++i )
-        if( !tr_torrentIsPieceChecked( tor, i ) )
-            tr_cpPieceRem( tor->completion, i );
+    {
+        tr_piece_index_t i;
+        for( i=0; i<tor->info.pieceCount; ++i )
+            if( !tr_torrentIsPieceChecked( tor, i ) )
+                tr_cpPieceRem( tor->completion, i );
+    }
 
     return ret;
 }
@@ -455,8 +460,8 @@ parsePriorities( tr_torrent * tor, const uint8_t * buf, uint32_t len )
     {
         const size_t n = tor->info.fileCount;
         const size_t len = 2 * n;
-        int *dnd = NULL, dndCount = 0;
-        int *dl = NULL, dlCount = 0;
+        tr_file_index_t *dnd = NULL, dndCount = 0;
+        tr_file_index_t *dl = NULL, dlCount = 0;
         size_t i;
         const uint8_t * walk = buf;
 
@@ -473,8 +478,8 @@ parsePriorities( tr_torrent * tor, const uint8_t * buf, uint32_t len )
         }
 
         /* set the dnd flags */
-        dl = tr_new( int, len );
-        dnd = tr_new( int, len );
+        dl = tr_new( tr_file_index_t, len );
+        dnd = tr_new( tr_file_index_t, len );
         for( i=0; i<n; ++i )
             if( *walk++ == 't' ) /* 't' means the DND flag is true */
                 dnd[dndCount++] = i;
@@ -546,7 +551,7 @@ parsePeers( tr_torrent * tor, const uint8_t * buf, uint32_t len )
             tr_peerMgrAddPex( tor->handle->peerMgr, tor->info.hash, TR_PEER_FROM_CACHE, &pex );
         }
 
-        tr_dbg( "found %i peers in resume file", count );
+        tr_tordbg( tor, "Loaded %d peers from resume file", count );
         ret = TR_FR_PEERS;
     }
 
@@ -580,7 +585,11 @@ parseVersion1( tr_torrent * tor, const uint8_t * buf, const uint8_t * end,
         readBytes( &id, &buf, sizeof(id) );
         readBytes( &len, &buf, sizeof(len) );
 
-        if( fieldsToLoad & internalIdToPublicBitfield( id ) ) switch( id )
+        if( buf + len > end )
+        {
+            tr_torerr( tor, "Resume file seems to be corrupt.  Skipping." );
+        }
+        else if( fieldsToLoad & internalIdToPublicBitfield( id ) ) switch( id )
         {
             case FR_ID_DOWNLOADED:   ret |= parseDownloaded( tor, buf, len ); break;
             case FR_ID_UPLOADED:     ret |= parseUploaded( tor, buf, len ); break;
@@ -592,7 +601,7 @@ parseVersion1( tr_torrent * tor, const uint8_t * buf, const uint8_t * end,
             case FR_ID_PEERS:        ret |= parsePeers( tor, buf, len ); break;
             case FR_ID_MAX_PEERS:    ret |= parseConnections( tor, buf, len ); break;
             case FR_ID_DESTINATION:  ret |= parseDestination( tor, buf, len ); break;
-            default:                 tr_dbg( "Skipping unknown resume code %d", (int)id ); break;
+            default:                 tr_tordbg( tor, "Skipping unknown resume code %d", (int)id ); break;
         }
 
         buf += len;
@@ -606,7 +615,7 @@ loadResumeFile( const tr_torrent * tor, size_t * len )
 {
     uint8_t * ret = NULL;
     char path[MAX_PATH_LENGTH];
-    const char * cacheDir = tr_getCacheDirectory ();
+    const char * cacheDir = tr_getResumeDir( tor->handle );
     const char * hash = tor->info.hashString;
 
     if( !ret && tor->handle->tag )
@@ -635,7 +644,8 @@ fastResumeLoadImpl ( tr_torrent   * tor,
     uint8_t * buf = loadResumeFile( tor, &size );
 
     if( !buf )
-        tr_inf( "Couldn't read resume file for '%s'", tor->info.name );
+        /* %s is the torrent name */
+        tr_torinf( tor, _( "Couldn't read resume file" ) );
     else {
         const uint8_t * walk = buf;
         const uint8_t * end = walk + size;
@@ -645,7 +655,8 @@ fastResumeLoadImpl ( tr_torrent   * tor,
             if( version == 1 )
                 ret |= parseVersion1 ( tor, walk, end, fieldsToLoad );
             else
-                tr_inf( "Unsupported resume file %d for '%s'", version, tor->info.name );
+                /* %s is the torrent name */
+                tr_torinf( tor, _( "Couldn't read resume file" ) );
         }
 
         tr_free( buf );
@@ -654,59 +665,30 @@ fastResumeLoadImpl ( tr_torrent   * tor,
     return ret;
 }
 
-static uint64_t
-setFromCtor( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor, int mode )
-{
-    uint64_t ret = 0;
-
-    if( fields & TR_FR_RUN ) {
-        uint8_t isPaused;
-        if( !tr_ctorGetIsPaused( ctor, mode, &isPaused ) ) {
-            tor->isRunning = !isPaused;
-            ret |= TR_FR_RUN;
-        }
-    }
-
-    if( fields & TR_FR_MAX_PEERS ) 
-        if( !tr_ctorGetMaxConnectedPeers( ctor, mode, &tor->maxConnectedPeers ) )
-            ret |= TR_FR_MAX_PEERS;
-
-    if( fields & TR_FR_DESTINATION ) {
-        const char * destination;
-        if( !tr_ctorGetDestination( ctor, mode, &destination ) ) {
-            ret |= TR_FR_DESTINATION;
-            tr_free( tor->destination );
-            tor->destination = tr_strdup( destination );
-        }
-    }
-
-    return ret;
-}
-
-static uint64_t
-useManditoryFields( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor )
-{
-    return setFromCtor( tor, fields, ctor, TR_FORCE );
-}
-
-static uint64_t
-useFallbackFields( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor )
-{
-    return setFromCtor( tor, fields, ctor, TR_FALLBACK );
-}
-
 uint64_t
 tr_fastResumeLoad( tr_torrent     * tor,
-                   uint64_t         fieldsToLoad,
-                   const tr_ctor  * ctor )
+                   uint64_t         fieldsToLoad )
 {
-    uint64_t ret = 0;
+    return fastResumeLoadImpl( tor, fieldsToLoad );
+}
 
-    ret |= useManditoryFields( tor, fieldsToLoad, ctor );
-    fieldsToLoad &= ~ret;
-    ret |= fastResumeLoadImpl( tor, fieldsToLoad );
-    fieldsToLoad &= ~ret;
-    ret |= useFallbackFields( tor, fieldsToLoad, ctor );
+void
+tr_fastResumeRemove( const tr_torrent * tor )
+{
+    char path[MAX_PATH_LENGTH];
+    const char * cacheDir = tr_getResumeDir( tor->handle );
+    const char * hash = tor->info.hashString;
 
-    return ret;
+    if( tor->handle->tag )
+    {
+        char base[1024];
+        snprintf( base, sizeof(base), "%s-%s", hash, tor->handle->tag );
+        tr_buildPath( path, sizeof(path), cacheDir, base, NULL );
+        unlink( path );
+    }
+    else
+    {
+        tr_buildPath( path, sizeof(path), cacheDir, hash, NULL );
+        unlink( path );
+    }
 }
