@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fastresume.h 4854 2008-01-28 17:53:32Z charles $
+ * $Id: fastresume.h 5641 2008-04-18 12:47:13Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -25,27 +25,12 @@
 #ifndef TR_FAST_RESUME_H
 #define TR_FAST_RESUME_H
 
-void tr_fastResumeSave( const tr_torrent * tor );
-
-enum
-{
-  TR_FR_DOWNLOADED   = (1<<0),
-  TR_FR_UPLOADED     = (1<<1),
-  TR_FR_CORRUPT      = (1<<2),
-  TR_FR_PEERS        = (1<<3),
-  TR_FR_PROGRESS     = (1<<4),
-  TR_FR_PRIORITY     = (1<<5),
-  TR_FR_SPEEDLIMIT   = (1<<6),
-  TR_FR_RUN          = (1<<7),
-  TR_FR_DESTINATION  = (1<<8),
-  TR_FR_MAX_PEERS    = (1<<9)
-};
-
 /**
  * Returns a bitwise-or'ed set of the data loaded from fastresume
  */
 uint64_t tr_fastResumeLoad( tr_torrent          * tor,
-                            uint64_t              fieldsToLoad,
-                            const tr_ctor       * ctor );
+                            uint64_t              fieldsToLoad );
+
+void tr_fastResumeRemove( const tr_torrent * tor );
 
 #endif
