@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 4774 2008-01-21 01:56:54Z charles $
+ * $Id: handshake.c 5723 2008-04-29 16:57:16Z charles $
  */
 
 #include <assert.h>
@@ -16,7 +16,6 @@
 #include <limits.h> /* UCHAR_MAX */
 #include <string.h>
 #include <stdio.h>
-#include <libgen.h> /* basename */
 
 #include <event.h>
 
@@ -26,6 +25,7 @@
 #include "crypto.h"
 #include "handshake.h"
 #include "peer-io.h"
+#include "peer-mgr.h"
 #include "torrent.h"
 #include "trevent.h"
 #include "utils.h"
@@ -133,32 +133,7 @@ enum
 ***
 **/
 
-static void
-myDebug( const char * file, int line, const tr_handshake * handshake, const char * fmt, ... )
-{
-    FILE * fp = tr_getLog( );
-    if( fp != NULL )
-    {
-        va_list args;
-        char timestr[64];
-        struct evbuffer * buf = evbuffer_new( );
-        char * myfile = tr_strdup( file );
-
-        evbuffer_add_printf( buf, "[%s] %s: ",
-                             tr_getLogTimeStr( timestr, sizeof(timestr) ),
-                             tr_peerIoGetAddrStr( handshake->io ) );
-        va_start( args, fmt );
-        evbuffer_add_vprintf( buf, fmt, args );
-        va_end( args );
-        evbuffer_add_printf( buf, " (%s:%d)\n", basename(myfile), line );
-        fwrite( EVBUFFER_DATA(buf), 1, EVBUFFER_LENGTH(buf), fp );
-
-        tr_free( myfile );
-        evbuffer_free( buf );
-    }
-}
-
-#define dbgmsg(handshake, fmt...) myDebug(__FILE__, __LINE__, handshake, ##fmt )
+#define dbgmsg(handshake, fmt...) tr_deepLog( __FILE__, __LINE__, tr_peerIoGetAddrStr( handshake->io ), ##fmt )
 
 static const char* getStateName( short state )
 {
@@ -690,8 +665,8 @@ readHandshake( tr_handshake * handshake, struct evbuffer * inbuf )
 static int
 readPeerId( tr_handshake * handshake, struct evbuffer * inbuf )
 {
-    char * client;
     int peerIsGood;
+    char client[128];
 
     if( EVBUFFER_LENGTH(inbuf) < PEER_ID_LEN )
         return READ_MORE;
@@ -700,9 +675,8 @@ readPeerId( tr_handshake * handshake, struct evbuffer * inbuf )
     tr_peerIoReadBytes( handshake->io, inbuf, handshake->peer_id, PEER_ID_LEN );
     tr_peerIoSetPeersId( handshake->io, handshake->peer_id );
     handshake->havePeerID = TRUE;
-    client = tr_clientForId( handshake->peer_id );
+    tr_clientForId( client, sizeof( client ), handshake->peer_id );
     dbgmsg( handshake, "peer-id is [%s] ... isIncoming is %d", client, tr_peerIoIsIncoming( handshake->io ) );
-    tr_free( client );
 
     /* if we've somehow connected to ourselves, don't keep the connection */
     peerIsGood = memcmp( handshake->peer_id, tr_getPeerId(), PEER_ID_LEN ) ? 1 : 0;
@@ -810,11 +784,22 @@ readCryptoProvide( tr_handshake * handshake, struct evbuffer * inbuf )
     tr_sha1( req3, "req3", 4, handshake->mySecret, KEY_LEN, NULL );
     for( i=0; i<SHA_DIGEST_LENGTH; ++i )
         obfuscatedTorrentHash[i] = req2[i] ^ req3[i];
-    tor = tr_torrentFindFromObfuscatedHash( handshake->handle, obfuscatedTorrentHash );
-    if( tor != NULL ) {
+    if(( tor = tr_torrentFindFromObfuscatedHash( handshake->handle, obfuscatedTorrentHash )))
+    {
         dbgmsg( handshake, "found the torrent; it's [%s]", tor->info.name );
         tr_peerIoSetTorrentHash( handshake->io, tor->info.hash );
-    } else {
+        if( !tr_torrentAllowsPex( tor ) &&
+            tr_peerMgrPeerIsSeed( handshake->handle->peerMgr,
+                                  tor->info.hash,
+                                  tr_peerIoGetAddress( handshake->io, NULL )))
+        {
+            dbgmsg( handshake, "a peer has tried to reconnect to us!" );
+            tr_handshakeDone( handshake, FALSE );
+            return READ_DONE;
+        }
+    }
+    else
+    {
         dbgmsg( handshake, "can't find that torrent..." );
         tr_handshakeDone( handshake, FALSE );
         return READ_DONE;
@@ -1024,7 +1009,7 @@ gotError( struct bufferevent * evbuf UNUSED, short what, void * arg )
     else
     {
         dbgmsg( handshake, "libevent got an error what==%d, errno=%d (%s)",
-                (int)what, errno, strerror(errno) );
+                (int)what, errno, tr_strerror(errno) );
         tr_handshakeDone( handshake, FALSE );
     }
 }
