@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: inout.h 4776 2008-01-21 02:07:55Z charles $
+ * $Id: inout.h 5645 2008-04-18 23:17:40Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -32,31 +32,29 @@ struct tr_torrent;
  * @return 0 on success, TR_ERROR_ASSERT if the arguments are incorrect,
  * or TR_ERROR_IO_* otherwise.
  */
-int tr_ioRead  ( struct tr_torrent*, int index, int begin, int len, uint8_t * );
+tr_errno tr_ioRead( const struct tr_torrent  * tor,
+                    tr_piece_index_t           pieceIndex,
+                    uint32_t                   offset,
+                    uint32_t                   len,
+                    uint8_t                  * setme );
 
 /**
  * Writes the block specified by the piece index, offset, and length.
  * @return 0 on success, TR_ERROR_ASSERT if the arguments are incorrect,
  * or TR_ERROR_IO_* otherwise.
  */
-int tr_ioWrite ( struct tr_torrent *, int index, int begin, int len, const uint8_t * );
-
-/* hashes the specified piece and updates the completion accordingly. */
-int tr_ioHash ( tr_torrent*, int piece );
+tr_errno tr_ioWrite( const struct tr_torrent  * tor,
+                     tr_piece_index_t           pieceIndex,
+                     uint32_t                   offset,
+                     uint32_t                   len,
+                     const uint8_t            * writeme );
 
 /**
-***
-**/
+ * returns 0 if the piece matches its metainfo's SHA1 checksum,
+ * or TR_ERROR_IO_* if there was a problem reading the piece,
+ * or TR_ERROR if the checksum didn't match.
+ */
+tr_errno tr_ioTestPiece( const tr_torrent*, int piece );
 
-typedef void (*tr_recheck_done_cb)( tr_torrent * tor );
-
-void tr_ioRecheckAdd( tr_torrent          * tor,
-                      tr_recheck_done_cb    recheck_done_cb );
-
-void tr_ioRecheckRemove( tr_torrent * tor );
-
-/**
-***
-**/
 
 #endif
