@@ -7,14 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.c 5084 2008-02-20 11:36:41Z charles $
+ * $Id: makemeta.c 6187 2008-06-13 21:39:54Z charles $
  */
 
 #include <assert.h>
 #include <errno.h>
-#include <math.h>
 #include <stdio.h> /* FILE, snprintf, stderr */
-#include <stdlib.h>
+#include <stdlib.h> /* qsort */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -28,7 +27,6 @@
 #include "bencode.h"
 #include "makemeta.h"
 #include "platform.h" /* threads, locks */
-#include "shared.h" /* shared lock */
 #include "utils.h" /* buildpath */
 #include "version.h"
 
@@ -57,7 +55,7 @@ getFiles( const char        * dir,
     tr_buildPath( buf, sizeof(buf), dir, base, NULL );
     i = stat( buf, &sb );
     if( i ) {
-        tr_err("makemeta couldn't stat \"%s\"; skipping. (%s)", buf, strerror(errno));
+        tr_err( _( "Torrent Creator is skipping file \"%s\": %s" ), buf, tr_strerror(errno));
         return list;
     }
 
@@ -116,7 +114,7 @@ tr_metaInfoBuilderCreate( tr_handle * handle, const char * topFile )
     tr_metainfo_builder * ret = tr_new0( tr_metainfo_builder, 1 );
     ret->top = tr_strdup( topFile );
     ret->handle = handle; 
-    if (1) {
+    {
         struct stat sb;
         stat( topFile, &sb );
         ret->isSingleFile = !S_ISDIR( sb.st_mode );
@@ -124,7 +122,7 @@ tr_metaInfoBuilderCreate( tr_handle * handle, const char * topFile )
 
     /* build a list of files containing topFile and,
        if it's a directory, all of its children */
-    if (1) {
+    {
         char *dir, *base;
         char dirbuf[MAX_PATH_LENGTH];
         char basebuf[MAX_PATH_LENGTH];
@@ -169,7 +167,7 @@ tr_metaInfoBuilderCreate( tr_handle * handle, const char * topFile )
 void
 tr_metaInfoBuilderFree( tr_metainfo_builder * builder )
 {
-    if( builder != NULL )
+    if( builder )
     {
         uint32_t i;
         for( i=0; i<builder->fileCount; ++i )
@@ -205,6 +203,13 @@ getHashInfo ( tr_metainfo_builder * b )
     b->pieceIndex = 0;
     totalRemain = b->totalSize;
     fp = fopen( b->files[fileIndex].filename, "rb" );
+    if( !fp ) {
+        b->my_errno = errno;
+        snprintf( b->errfile, sizeof( b->errfile ), b->files[fileIndex].filename );
+        b->result = TR_MAKEMETA_IO_READ;
+        tr_free( ret );
+        return NULL;
+    }
     while ( totalRemain )
     {
         uint8_t *bufptr = buf;
@@ -228,6 +233,13 @@ getHashInfo ( tr_metainfo_builder * b )
                 fp = NULL;
                 if( ++fileIndex < b->fileCount ) {
                     fp = fopen( b->files[fileIndex].filename, "rb" );
+                    if( !fp ) {
+                        b->my_errno = errno;
+                        snprintf( b->errfile, sizeof( b->errfile ), b->files[fileIndex].filename );
+                        b->result = TR_MAKEMETA_IO_READ;
+                        tr_free( ret );
+                        return NULL;
+                    }
                 }
             }
         }
@@ -238,7 +250,7 @@ getHashInfo ( tr_metainfo_builder * b )
         walk += SHA_DIGEST_LENGTH;
 
         if( b->abortFlag ) {
-            b->failed = 1;
+            b->result = TR_MAKEMETA_CANCELLED;
             break;
         }
 
@@ -247,7 +259,6 @@ getHashInfo ( tr_metainfo_builder * b )
     }
     assert( b->abortFlag || (walk-ret == (int)(SHA_DIGEST_LENGTH*b->pieceCount)) );
     assert( b->abortFlag || !totalRemain );
-    assert( b->abortFlag || fp == NULL );
 
     if( fp != NULL )
         fclose( fp );
@@ -259,10 +270,9 @@ getHashInfo ( tr_metainfo_builder * b )
 static void
 getFileInfo( const char                      * topFile,
              const tr_metainfo_builder_file  * file,
-             benc_val_t                      * uninitialized_length,
-             benc_val_t                      * uninitialized_path )
+             tr_benc                         * uninitialized_length,
+             tr_benc                         * uninitialized_path )
 {
-    benc_val_t *sub;
     const char *pch, *prev;
     const size_t topLen = strlen(topFile) + 1; /* +1 for '/' */
     int n;
@@ -275,8 +285,7 @@ getFileInfo( const char                      * topFile,
     for( pch=file->filename+topLen; *pch; ++pch )
         if (*pch == TR_PATH_DELIMITER)
             ++n;
-    tr_bencInit( uninitialized_path, TYPE_LIST );
-    tr_bencListReserve( uninitialized_path, n );
+    tr_bencInitList( uninitialized_path, n );
     for( prev=pch=file->filename+topLen; ; ++pch )
     {
         char buf[MAX_PATH_LENGTH];
@@ -287,8 +296,7 @@ getFileInfo( const char                      * topFile,
         memcpy( buf, prev, pch-prev );
         buf[pch-prev] = '\0';
 
-        sub = tr_bencListAdd( uninitialized_path );
-        tr_bencInitStrDup( sub, buf );
+        tr_bencListAddStr( uninitialized_path, buf );
 
         prev = pch + 1;
         if (!*pch)
@@ -297,111 +305,77 @@ getFileInfo( const char                      * topFile,
 }
 
 static void
-makeFilesList( benc_val_t                 * list,
-               const tr_metainfo_builder  * builder )
-{
-    uint32_t i = 0;
-
-    tr_bencListReserve( list, builder->fileCount );
-
-    for( i=0; i<builder->fileCount; ++i )
-    {
-        benc_val_t * dict = tr_bencListAdd( list );
-        benc_val_t *length, *pathVal;
-
-        tr_bencInit( dict, TYPE_DICT );
-        tr_bencDictReserve( dict, 2 );
-        length = tr_bencDictAdd( dict, "length" );
-        pathVal = tr_bencDictAdd( dict, "path" );
-        getFileInfo( builder->top, &builder->files[i], length, pathVal );
-    }
-}
-
-static void
-makeInfoDict ( benc_val_t           * dict,
+makeInfoDict ( tr_benc              * dict,
                tr_metainfo_builder  * builder )
 {
     uint8_t * pch;
-    benc_val_t * val;
     char base[MAX_PATH_LENGTH];
 
     tr_bencDictReserve( dict, 5 );
     
     if ( builder->isSingleFile )
-    {
-        val = tr_bencDictAdd( dict, "length" );
-        tr_bencInitInt( val, builder->files[0].size );
-    }
-    else
-    {
-        val = tr_bencDictAdd( dict, "files" );
-        tr_bencInit( val, TYPE_LIST );
-        makeFilesList( val, builder );
+        tr_bencDictAddInt( dict, "length", builder->files[0].size );
+    else {
+        uint32_t i;
+        tr_benc * list = tr_bencDictAddList( dict, "files", builder->fileCount );
+        for( i=0; i<builder->fileCount; ++i ) {
+            tr_benc * dict = tr_bencListAddDict( list, 2 );
+            tr_benc * length = tr_bencDictAdd( dict, "length" );
+            tr_benc * pathVal = tr_bencDictAdd( dict, "path" );
+            getFileInfo( builder->top, &builder->files[i], length, pathVal );
+        }
     }
 
-    val = tr_bencDictAdd( dict, "name" );
     strlcpy( base, builder->top, sizeof( base ) );
-    tr_bencInitStrDup ( val, basename( base ) );
+    tr_bencDictAddStr( dict, "name", basename( base ) );
 
-    val = tr_bencDictAdd( dict, "piece length" );
-    tr_bencInitInt( val, builder->pieceSize );
+    tr_bencDictAddInt( dict, "piece length", builder->pieceSize );
 
-    pch = getHashInfo( builder );
-    val = tr_bencDictAdd( dict, "pieces" );
-    tr_bencInitStr( val, pch, SHA_DIGEST_LENGTH * builder->pieceCount, 0 );
+    if(( pch = getHashInfo( builder ))) {
+        tr_bencDictAddRaw( dict, "pieces", pch, SHA_DIGEST_LENGTH * builder->pieceCount );
+        tr_free( pch );
+    }
 
-    val = tr_bencDictAdd( dict, "private" );
-    tr_bencInitInt( val, builder->isPrivate ? 1 : 0 );
+    tr_bencDictAddInt( dict, "private", builder->isPrivate ? 1 : 0 );
 }
 
-static void tr_realMakeMetaInfo ( tr_metainfo_builder * builder )
+static void
+tr_realMakeMetaInfo ( tr_metainfo_builder * builder )
 {
     int n = 5;
-    benc_val_t top, *val;
+    tr_benc top;
 
-    tr_bencInit ( &top, TYPE_DICT );
     if ( builder->comment && *builder->comment ) ++n;
-    tr_bencDictReserve( &top, n );
+    tr_bencInitDict( &top, n );
 
-    val = tr_bencDictAdd( &top, "announce" );
-    tr_bencInitStrDup( val, builder->announce );
+    tr_bencDictAddStr( &top, "announce", builder->announce );
+
+    if( tr_httpParseURL( builder->announce, -1, NULL, NULL, NULL ) )
+        builder->result = TR_MAKEMETA_URL;
     
-    if( builder->comment && *builder->comment ) {
-        val = tr_bencDictAdd( &top, "comment" );
-        tr_bencInitStrDup( val, builder->comment );
+    if( !builder->result && !builder->abortFlag )
+    {
+        if( builder->comment && *builder->comment )
+            tr_bencDictAddStr( &top, "comment", builder->comment );
+        tr_bencDictAddStr( &top, "created by", TR_NAME "/" LONG_VERSION_STRING );
+        tr_bencDictAddInt( &top, "creation date", time(0) );
+        tr_bencDictAddStr( &top, "encoding", "UTF-8" );
+        makeInfoDict( tr_bencDictAddDict( &top, "info", 666 ), builder );
     }
 
-    val = tr_bencDictAdd( &top, "created by" );
-    tr_bencInitStrDup( val, TR_NAME "/" LONG_VERSION_STRING );
-
-    val = tr_bencDictAdd( &top, "creation date" );
-    tr_bencInitInt( val, time(0) );
-
-    val = tr_bencDictAdd( &top, "encoding" );
-    tr_bencInitStrDup( val, "UTF-8" );
-
-    val = tr_bencDictAdd( &top, "info" );
-    tr_bencInit( val, TYPE_DICT );
-    tr_bencDictReserve( val, 666 );
-    makeInfoDict( val, builder );
-
     /* save the file */
-    if ( !builder->abortFlag ) {
-        size_t nmemb;
-        char * pch = tr_bencSave( &top, &n );
-        FILE * fp = fopen( builder->outputFile, "wb+" );
-        nmemb = n;
-        if( fp == NULL )
-            builder->failed = 1;
-        else if( fwrite( pch, 1, nmemb, fp ) != nmemb )
-            builder->failed = 1;
-        tr_free( pch );
-        fclose( fp );
+    if ( !builder->result && !builder->abortFlag ) {
+        if( tr_bencSaveFile( builder->outputFile, &top ) ) {
+            builder->my_errno = errno;
+            strlcpy( builder->errfile, builder->outputFile, sizeof( builder->errfile ) );
+            builder->result = TR_MAKEMETA_IO_WRITE;
+        }
     }
 
     /* cleanup */
     tr_bencFree( & top );
-    builder->failed |= builder->abortFlag;
+    if( builder->abortFlag )
+        builder->result = TR_MAKEMETA_CANCELLED;
     builder->isDone = 1;
 }
 
@@ -463,6 +437,12 @@ tr_makeMetaInfo( tr_metainfo_builder  * builder,
 {
     tr_lock * lock;
 
+    /* free any variables from a previous run */
+    tr_free( builder->announce );
+    tr_free( builder->comment );
+    tr_free( builder->outputFile );
+
+    /* initialize the builder variables */
     builder->abortFlag = 0;
     builder->isDone = 0;
     builder->announce = tr_strdup( announce );
