@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.h 3574 2007-10-26 03:15:55Z charles $
+ * $Id: makemeta.h 5358 2008-03-24 17:18:08Z charles $
  */
 
 #ifndef TR_MAKEMETA_H
@@ -20,6 +20,18 @@ typedef struct tr_metainfo_builder_file
 }
 tr_metainfo_builder_file;
 
+typedef enum
+{
+    TR_MAKEMETA_OK,
+    TR_MAKEMETA_URL,
+    TR_MAKEMETA_CANCELLED,
+    TR_MAKEMETA_IO_READ,   /* see builder.errfile, builder.errno */
+    TR_MAKEMETA_IO_WRITE   /* see builder.errfile, builder.errno */
+}
+tr_metainfo_builder_err;
+
+    
+
 typedef struct tr_metainfo_builder
 {
     /**
@@ -57,7 +69,13 @@ typedef struct tr_metainfo_builder
     uint32_t pieceIndex;
     int abortFlag;
     int isDone;
-    int failed; /* only meaningful if isDone is set */
+    tr_metainfo_builder_err result;
+
+    /* file in use when result was set to _IO_READ or _IO_WRITE */
+    char errfile[2048];
+
+    /* errno encountered when result was set to _IO_READ or _IO_WRITE */
+    int my_errno;
 
     /**
     ***  This is an implementation detail.
