@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 4772 2008-01-21 01:51:31Z charles $
+ * $Id: peer-io.c 5738 2008-05-01 19:31:58Z charles $
  */
 
 #include <assert.h>
@@ -34,8 +34,8 @@
 
 #define IO_TIMEOUT_SECS 8
 
-/* arbitrary */
-#define TR_RDBUF (1024*8)
+/* the size of a typical request message */
+#define TR_RDBUF ((1024*16) + 13)
 
 /**
 ***
@@ -139,6 +139,10 @@ tr_peerIoNew( struct tr_handle     * handle,
               int                    socket )
 {
     tr_peerIo * c;
+
+    if( socket >= 0 )
+        tr_netSetTOS( socket, handle->peerSocketTOS );
+
     c = tr_new0( tr_peerIo, 1 );
     c->crypto = tr_cryptoNew( torrentHash, isIncoming );
     c->handle = handle;
@@ -292,6 +296,8 @@ tr_peerIoReconnect( tr_peerIo * io )
   
     if( io->socket >= 0 )
     {
+        tr_netSetTOS( io->socket, io->handle->peerSocketTOS );
+
         bufferevent_free( io->bufev );
 
         io->bufev = bufferevent_new( io->socket,
