@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr-private.h 4773 2008-01-21 01:54:54Z charles $
+ * $Id: peer-mgr-private.h 5179 2008-03-01 14:32:35Z charles $
  */
 
 #ifndef TR_PEER_MGR_PRIVATE_H
@@ -46,7 +46,7 @@ enum
 /**
  * Use SWIFT?
  */
-static const int SWIFT_ENABLED = 1;
+static const int SWIFT_ENABLED = 0;
 
 /**
  * For every byte the peer uploads to us,
@@ -63,8 +63,6 @@ typedef struct tr_peer
     unsigned int  clientIsInterested : 1;
     unsigned int  doPurge : 1;
 
-    tr_peer_status status;
-
     /* number of bad pieces they've contributed to */
     uint8_t strikes;
 
@@ -73,7 +71,6 @@ typedef struct tr_peer
     struct in_addr in_addr;
     struct tr_peerIo * io;
 
-    struct tr_bitfield * banned;
     struct tr_bitfield * blame;
     struct tr_bitfield * have;
     float progress;
