@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.c 5106 2008-02-25 01:37:53Z charles $
+ * $Id: peer-mgr.c 6175 2008-06-13 16:44:16Z charles $
  */
 
 #include <assert.h>
@@ -16,11 +16,10 @@
 #include <stdio.h> /* printf */
 #include <limits.h> /* INT_MAX */
 
-#include <libgen.h> /* basename */
-
 #include <event.h>
 
 #include "transmission.h"
+#include "blocklist.h"
 #include "clients.h"
 #include "completion.h"
 #include "crypto.h"
@@ -30,10 +29,8 @@
 #include "peer-mgr.h"
 #include "peer-mgr-private.h"
 #include "peer-msgs.h"
-#include "platform.h"
 #include "ptrarray.h"
 #include "ratecontrol.h"
-#include "shared.h"
 #include "torrent.h"
 #include "trcompat.h" /* strlcpy */
 #include "trevent.h"
@@ -99,12 +96,6 @@ enum
     /* number of unchoked peers per torrent */
     MAX_UNCHOKED_PEERS = 12,
 
-    /* corresponds to ut_pex's added.f flags */
-    ADDED_F_ENCRYPTION_FLAG = 1,
-
-    /* corresponds to ut_pex's added.f flags */
-    ADDED_F_SEED_FLAG = 2,
-
     /* number of bad pieces a peer is allowed to send before we ban them */
     MAX_BAD_PIECES_PER_PEER = 3,
 
@@ -163,48 +154,19 @@ struct tr_peerMgr
     tr_ptrArray * incomingHandshakes; /* tr_handshake */
 };
 
-/**
-***
-**/
-
-static void
-myDebug( const char * file, int line, const Torrent * t, const char * fmt, ... )
-{
-    FILE * fp = tr_getLog( );
-    if( fp != NULL )
-    {
-        va_list args;
-        char timestr[64];
-        struct evbuffer * buf = evbuffer_new( );
-        char * myfile = tr_strdup( file );
-
-        evbuffer_add_printf( buf, "[%s] ", tr_getLogTimeStr( timestr, sizeof(timestr) ) );
-        if( t != NULL )
-            evbuffer_add_printf( buf, "%s ", t->tor->info.name );
-        va_start( args, fmt );
-        evbuffer_add_vprintf( buf, fmt, args );
-        va_end( args );
-        evbuffer_add_printf( buf, " (%s:%d)\n", basename(myfile), line );
-        fwrite( EVBUFFER_DATA(buf), 1, EVBUFFER_LENGTH(buf), fp );
-
-        tr_free( myfile );
-        evbuffer_free( buf );
-    }
-}
-
-#define tordbg(t, fmt...) myDebug(__FILE__, __LINE__, t, ##fmt )
+#define tordbg(t, fmt...) tr_deepLog( __FILE__, __LINE__, t->tor->info.name, ##fmt )
 
 /**
 ***
 **/
 
 static void
-managerLock( struct tr_peerMgr * manager )
+managerLock( const struct tr_peerMgr * manager )
 {
     tr_globalLock( manager->handle );
 }
 static void
-managerUnlock( struct tr_peerMgr * manager )
+managerUnlock( const struct tr_peerMgr * manager )
 {
     tr_globalUnlock( manager->handle );
 }
@@ -547,6 +509,40 @@ getConnectedPeers( Torrent * t, int * setmeCount )
     return ret;
 }
 
+static int
+clientIsDownloadingFrom( const tr_peer * peer )
+{
+    return peer->clientIsInterested && !peer->clientIsChoked;
+}
+
+static int
+clientIsUploadingTo( const tr_peer * peer )
+{
+    return peer->peerIsInterested && !peer->peerIsChoked;
+}
+
+/***
+****
+***/
+
+int
+tr_peerMgrPeerIsSeed( const tr_peerMgr       * mgr,
+                      const uint8_t          * torrentHash,
+                      const struct in_addr   * addr )
+{
+    int isSeed = FALSE;
+    const Torrent * t = NULL;
+    const struct peer_atom * atom = NULL;
+
+    t = getExistingTorrent( (tr_peerMgr*)mgr, torrentHash );
+    if( t )
+        atom = getExistingAtom( t, addr );
+    if( atom )
+        isSeed = ( atom->flags & ADDED_F_SEED_FLAG ) != 0;
+
+    return isSeed;
+}
+
 /***
 ****  Refill
 ***/
@@ -554,7 +550,7 @@ getConnectedPeers( Torrent * t, int * setmeCount )
 struct tr_refill_piece
 {
     tr_priority_t priority;
-    int percentDone;
+    int missingBlockCount;
     uint16_t random;
     uint32_t piece;
     uint32_t peerCount;
@@ -565,14 +561,14 @@ compareRefillPiece (const void * aIn, const void * bIn)
 {
     const struct tr_refill_piece * a = aIn;
     const struct tr_refill_piece * b = bIn;
+
+    /* fewer missing pieces goes first */
+    if( a->missingBlockCount != b->missingBlockCount )
+        return a->missingBlockCount < b->missingBlockCount ? -1 : 1;
     
     /* if one piece has a higher priority, it goes first */
     if( a->priority != b->priority )
         return a->priority > b->priority ? -1 : 1;
-
-    /* try to fill partial pieces */
-    if( a->percentDone != b->percentDone )
-        return a->percentDone > b->percentDone ? -1 : 1;
     
     /* otherwise if one has fewer peers, it goes first */
     if (a->peerCount != b->peerCount)
@@ -584,7 +580,7 @@ compareRefillPiece (const void * aIn, const void * bIn)
 
 static int
 isPieceInteresting( const tr_torrent  * tor,
-                    int                 piece )
+                    tr_piece_index_t    piece )
 {
     if( tor->info.pieces[piece].dnd ) /* we don't want it */
         return 0;
@@ -601,7 +597,7 @@ getPreferredPieces( Torrent     * t,
 {
     const tr_torrent * tor = t->tor;
     const tr_info * inf = &tor->info;
-    int i;
+    tr_piece_index_t i;
     uint32_t poolSize = 0;
     uint32_t * pool = tr_new( uint32_t, inf->pieceCount );
     int peerCount;
@@ -631,7 +627,7 @@ getPreferredPieces( Torrent     * t,
             setme->priority = inf->pieces[piece].priority;
             setme->peerCount = 0;
             setme->random = tr_rand( UINT16_MAX );
-            setme->percentDone = (int)( 100.0 * tr_cpPercentBlocksInPiece( tor->completion, piece ) );
+            setme->missingBlockCount = tr_cpMissingBlocksInPiece( tor->completion, piece );
 
             for( k=0; k<peerCount; ++k ) {
                 const tr_peer * peer = peers[k];
@@ -655,7 +651,7 @@ getPreferredPieces( Torrent     * t,
 }
 
 static uint64_t*
-getPreferredBlocks( Torrent * t, uint64_t * setmeCount )
+getPreferredBlocks( Torrent * t, tr_block_index_t * setmeCount )
 {
     int s;
     uint32_t i;
@@ -693,11 +689,11 @@ getPreferredBlocks( Torrent * t, uint64_t * setmeCount )
     /* sort the blocks into our temp bins */
     for( i=blockCount=0; i<pieceCount; ++i )
     {
-        const uint32_t index = pieces[i];
+        const tr_piece_index_t index = pieces[i];
         const int priorityIndex = tor->info.pieces[index].priority + 1;
-        const int begin = tr_torPieceFirstBlock( tor, index );
-        const int end = begin + tr_torPieceCountBlocks( tor, (int)index );
-        int block;
+        const tr_block_index_t begin = tr_torPieceFirstBlock( tor, index );
+        const tr_block_index_t end = begin + tr_torPieceCountBlocks( tor, index );
+        tr_block_index_t block;
 
         for( block=begin; block<end; ++block )
         {
@@ -740,15 +736,44 @@ getPreferredBlocks( Torrent * t, uint64_t * setmeCount )
     return ret;
 }
 
+static tr_peer**
+getPeersUploadingToClient( Torrent * t, int * setmeCount )
+{
+    int i;
+    int peerCount = 0;
+    int retCount = 0;
+    tr_peer ** peers = (tr_peer **) tr_ptrArrayPeek( t->peers, &peerCount );
+    tr_peer ** ret = tr_new( tr_peer*, peerCount );
+
+    /* get a list of peers we're downloading from */
+    for( i=0; i<peerCount; ++i )
+        if( clientIsDownloadingFrom( peers[i] ) )
+            ret[retCount++] = peers[i];
+
+    /* pick a different starting point each time so all peers
+     * get a chance at the first blocks in the queue */
+    if( retCount ) {
+        tr_peer ** tmp = tr_new( tr_peer*, retCount );
+        i = tr_rand( retCount );
+        memcpy( tmp, ret, sizeof(tr_peer*) * retCount );
+        memcpy( ret, tmp+i, sizeof(tr_peer*) * (retCount-i) );
+        memcpy( ret+(retCount-i), tmp, sizeof(tr_peer*) * i );
+        tr_free( tmp );
+    }
+
+    *setmeCount = retCount;
+    return ret;
+}
+
 static int
 refillPulse( void * vtorrent )
 {
     Torrent * t = vtorrent;
     tr_torrent * tor = t->tor;
-    uint32_t i;
+    tr_block_index_t i;
     int peerCount;
     tr_peer ** peers;
-    uint64_t blockCount;
+    tr_block_index_t blockCount;
     uint64_t * blocks;
 
     if( !t->isRunning )
@@ -760,18 +785,21 @@ refillPulse( void * vtorrent )
     tordbg( t, "Refilling Request Buffers..." );
 
     blocks = getPreferredBlocks( t, &blockCount );
-    peers = getConnectedPeers( t, &peerCount );
+    peers = getPeersUploadingToClient( t, &peerCount );
 
     for( i=0; peerCount && i<blockCount; ++i )
     {
-        const uint64_t block = blocks[i];
-        const uint32_t index = tr_torBlockPiece( tor, block );
-        const uint32_t begin = (block * tor->blockSize) - (index * tor->info.pieceSize);
-        const uint32_t length = tr_torBlockCountBytes( tor, (int)block );
         int j;
-        assert( _tr_block( tor, index, begin ) == (int)block );
-        assert( begin < (uint32_t)tr_torPieceCountBytes( tor, (int)index ) );
-        assert( (begin + length) <= (uint32_t)tr_torPieceCountBytes( tor, (int)index ) );
+
+        const tr_block_index_t block = blocks[i];
+        const tr_piece_index_t index = tr_torBlockPiece( tor, block );
+        const uint32_t begin = (block * tor->blockSize) - (index * tor->info.pieceSize);
+        const uint32_t length = tr_torBlockCountBytes( tor, block );
+
+        assert( tr_torrentReqIsValid( tor, index, begin, length ) );
+        assert( _tr_block( tor, index, begin ) == block );
+        assert( begin < tr_torPieceCountBytes( tor, index ) );
+        assert( (begin + length) <= tr_torPieceCountBytes( tor, index ) );
 
         /* find a peer who can ask for this block */
         for( j=0; j<peerCount; )
@@ -901,12 +929,14 @@ msgsCallbackFunc( void * vpeer, void * vevent, void * vt )
             broadcastGotBlock( t, e->pieceIndex, e->offset, e->length );
             break;
 
-        case TR_PEERMSG_GOT_ASSERT_ERROR:
-            addStrike( t, peer );
-            peer->doPurge = 1;
-            break;
-
         case TR_PEERMSG_ERROR:
+            if( TR_ERROR_IS_IO( e->err ) ) {
+                t->tor->error = e->err;
+                strlcpy( t->tor->errorString, tr_errorString( e->err ), sizeof(t->tor->errorString) );
+                tr_torrentStop( t->tor );
+            } else if( e->err == TR_ERROR_ASSERT ) {
+                addStrike( t, peer );
+            }
             peer->doPurge = 1;
             break;
 
@@ -1015,7 +1045,14 @@ myHandshakeDoneCB( tr_handshake    * handshake,
             {
                 peer = getPeer( t, addr );
                 tr_free( peer->client );
-                peer->client = peer_id ? tr_clientForId( peer_id ) : NULL;
+
+                if( !peer_id )
+                    peer->client = NULL;
+                else {
+                    char client[128];
+                    tr_clientForId( client, sizeof( client ), peer_id );
+                    peer->client = tr_strdup( client );
+                }
                 peer->port = port;
                 peer->io = io;
                 peer->msgs = tr_peerMsgsNew( t->tor, peer, msgsCallbackFunc, t, &peer->msgsTag );
@@ -1036,7 +1073,13 @@ tr_peerMgrAddIncoming( tr_peerMgr      * manager,
 {
     managerLock( manager );
 
-    if( getExistingHandshake( manager->incomingHandshakes, addr ) )
+    if( tr_blocklistHasAddress( manager->handle, addr ) )
+    {
+        tr_dbg( "Banned IP address \"%s\" tried to connect to us",
+                inet_ntoa( *addr ) );
+        tr_netClose( socket );
+    }
+    else if( getExistingHandshake( manager->incomingHandshakes, addr ) )
     {
         tr_netClose( socket );
     }
@@ -1045,8 +1088,6 @@ tr_peerMgrAddIncoming( tr_peerMgr      * manager,
         tr_peerIo * io;
         tr_handshake * handshake;
 
-        tordbg( NULL, "Got an INCOMING connection with %s", tr_peerIoAddrStr( addr, port ) );
-
         io = tr_peerIoNewIncoming( manager->handle, addr, port, socket );
 
         handshake = tr_handshakeNew( io,
@@ -1070,35 +1111,30 @@ tr_peerMgrAddPex( tr_peerMgr     * manager,
     managerLock( manager );
 
     t = getExistingTorrent( manager, torrentHash );
-    ensureAtomExists( t, &pex->in_addr, pex->port, pex->flags, from );
+    if( !tr_blocklistHasAddress( t->manager->handle, &pex->in_addr ) )
+        ensureAtomExists( t, &pex->in_addr, pex->port, pex->flags, from );
 
     managerUnlock( manager );
 }
 
-void
-tr_peerMgrAddPeers( tr_peerMgr    * manager,
-                    const uint8_t * torrentHash,
-                    uint8_t         from,
-                    const uint8_t * peerCompact,
-                    int             peerCount )
+tr_pex *
+tr_peerMgrCompactToPex( const void  * compact,
+                        size_t        compactLen,
+                        const char  * added_f,
+                        size_t      * pexCount )
 {
-    int i;
-    const uint8_t * walk = peerCompact;
-    Torrent * t;
-
-    managerLock( manager );
-
-    t = getExistingTorrent( manager, torrentHash );
-    for( i=0; t!=NULL && i<peerCount; ++i )
-    {
-        struct in_addr addr;
-        uint16_t port;
-        memcpy( &addr, walk, 4 ); walk += 4;
-        memcpy( &port, walk, 2 ); walk += 2;
-        ensureAtomExists( t, &addr, port, 0, from );
+    size_t i;
+    size_t n = compactLen / 6;
+    const uint8_t * walk = compact;
+    tr_pex * pex = tr_new0( tr_pex, n );
+    for( i=0; i<n; ++i ) {
+        memcpy( &pex[i].in_addr, walk, 4 ); walk += 4;
+        memcpy( &pex[i].port, walk, 2 ); walk += 2;
+        if( added_f )
+            pex[i].flags = added_f[i];
     }
-
-    managerUnlock( manager );
+    *pexCount = n;
+    return pex;
 }
 
 /**
@@ -1171,7 +1207,7 @@ tr_peerMgrGetPeers( tr_peerMgr      * manager,
     tr_pex * pex;
     tr_pex * walk;
 
-    torrentLock( (Torrent*)t );
+    managerLock( manager );
 
     peers = (const tr_peer **) tr_ptrArrayPeek( t->peers, &peerCount );
     pex = walk = tr_new( tr_pex, peerCount );
@@ -1193,7 +1229,7 @@ tr_peerMgrGetPeers( tr_peerMgr      * manager,
     qsort( pex, peerCount, sizeof(tr_pex), tr_pexCompare );
     *setme_pex = pex;
 
-    torrentUnlock( (Torrent*)t );
+    managerUnlock( manager );
 
     return peerCount;
 }
@@ -1346,26 +1382,25 @@ tr_peerMgrTorrentAvailability( const tr_peerMgr * manager,
     managerUnlock( (tr_peerMgr*)manager );
 }
 
-/* Returns the pieces that we and/or a connected peer has */
+/* Returns the pieces that are available from peers */
 tr_bitfield*
 tr_peerMgrGetAvailable( const tr_peerMgr * manager,
                         const uint8_t    * torrentHash )
 {
     int i, size;
-    const Torrent * t;
-    const tr_peer ** peers;
+    Torrent * t;
+    tr_peer ** peers;
     tr_bitfield * pieces;
-
     managerLock( (tr_peerMgr*)manager );
 
     t = getExistingTorrent( (tr_peerMgr*)manager, torrentHash );
-    peers = (const tr_peer **) tr_ptrArrayPeek( t->peers, &size );
-    pieces = tr_bitfieldDup( tr_cpPieceBitfield( t->tor->completion ) );
+    pieces = tr_bitfieldNew( t->tor->info.pieceCount );
+    peers = getConnectedPeers( t, &size );
     for( i=0; i<size; ++i )
-        if( peers[i]->io != NULL )
-            tr_bitfieldOr( pieces, peers[i]->have );
+        tr_bitfieldOr( pieces, peers[i]->have );
 
     managerUnlock( (tr_peerMgr*)manager );
+    tr_free( peers );
     return pieces;
 }
 
@@ -1384,18 +1419,6 @@ tr_peerMgrHasConnections( const tr_peerMgr * manager,
     return ret;
 }
 
-static int
-clientIsDownloadingFrom( const tr_peer * peer )
-{
-    return peer->clientIsInterested && !peer->clientIsChoked;
-}
-
-static int
-clientIsUploadingTo( const tr_peer * peer )
-{
-    return peer->peerIsInterested && !peer->peerIsChoked;
-}
-
 void
 tr_peerMgrTorrentStats( const tr_peerMgr * manager,
                         const uint8_t    * torrentHash,
@@ -1434,11 +1457,11 @@ tr_peerMgrTorrentStats( const tr_peerMgr * manager,
 
         ++setmePeersFrom[atom->from];
 
-        if( clientIsUploadingTo( peer ) )
-            ++*setmePeersGettingFromUs;
-
         if( clientIsDownloadingFrom( peer ) )
             ++*setmePeersSendingToUs;
+
+        if( clientIsUploadingTo( peer ) )
+            ++*setmePeersGettingFromUs;
     }
 
     managerUnlock( (tr_peerMgr*)manager );
@@ -1463,21 +1486,39 @@ tr_peerMgrPeerStats( const tr_peerMgr  * manager,
 
     for( i=0; i<size; ++i )
     {
+        char * pch;
         const tr_peer * peer = peers[i];
         const struct peer_atom * atom = getExistingAtom( t, &peer->in_addr );
         tr_peer_stat * stat = ret + i;
 
         tr_netNtop( &peer->in_addr, stat->addr, sizeof(stat->addr) );
         strlcpy( stat->client, (peer->client ? peer->client : ""), sizeof(stat->client) );
-        stat->port             = peer->port;
-        stat->from             = atom->from;
-        stat->progress         = peer->progress;
-        stat->isEncrypted      = tr_peerIoIsEncrypted( peer->io ) ? 1 : 0;
-        stat->uploadToRate     = peer->rateToPeer;
-        stat->downloadFromRate = peer->rateToClient;
-        stat->isDownloading    = clientIsDownloadingFrom( peer );
-        stat->isUploading      = clientIsUploadingTo( peer );
-        stat->status           = peer->status;
+        stat->port               = peer->port;
+        stat->from               = atom->from;
+        stat->progress           = peer->progress;
+        stat->isEncrypted        = tr_peerIoIsEncrypted( peer->io ) ? 1 : 0;
+        stat->uploadToRate       = peer->rateToPeer;
+        stat->downloadFromRate   = peer->rateToClient;
+        stat->peerIsChoked       = peer->peerIsChoked;
+        stat->peerIsInterested   = peer->peerIsInterested;
+        stat->clientIsChoked     = peer->clientIsChoked;
+        stat->clientIsInterested = peer->clientIsInterested;
+        stat->isIncoming         = tr_peerIoIsIncoming( peer->io );
+        stat->isDownloadingFrom  = clientIsDownloadingFrom( peer );
+        stat->isUploadingTo      = clientIsUploadingTo( peer );
+
+        pch = stat->flagStr;
+        if( t->optimistic == peer ) *pch++ = 'O';
+        if( stat->isDownloadingFrom ) *pch++ = 'D';
+        else if( stat->clientIsInterested ) *pch++ = 'd';
+        if( stat->isUploadingTo ) *pch++ = 'U';
+        else if( stat->peerIsInterested ) *pch++ = 'u';
+        if( !stat->clientIsChoked && !stat->clientIsInterested ) *pch++ = 'K';
+        if( !stat->peerIsChoked && !stat->peerIsInterested ) *pch++ = '?';
+        if( stat->isEncrypted ) *pch++ = 'E';
+        if( stat->from == TR_PEER_FROM_PEX ) *pch++ = 'X';
+        if( stat->isIncoming ) *pch++ = 'I';
+        *pch = '\0';
     }
 
     *setmeCount = size;
@@ -1533,7 +1574,7 @@ getWeightedRate( const tr_peer * peer, int clientIsSeed )
 static void
 rechoke( Torrent * t )
 {
-    int i, n, peerCount, size, unchokedInterested;
+    int i, peerCount, size, unchokedInterested;
     tr_peer ** peers = getConnectedPeers( t, &peerCount );
     struct ChokeData * choke = tr_new0( struct ChokeData, peerCount );
     const int clientIsSeed = tr_torrentIsSeed( t->tor );
@@ -1575,28 +1616,34 @@ rechoke( Torrent * t )
         if( choke[i].isInterested )
             ++unchokedInterested;
     }
-    n = i;
-    while( i<size )
-        choke[i++].doUnchoke = 0;
 
     /* optimistic unchoke */
     if( i < size )
     {
+        int n;
         struct ChokeData * c;
         tr_ptrArray * randPool = tr_ptrArrayNew( );
+
         for( ; i<size; ++i )
         {
-            const tr_peer * peer = choke[i].peer;
-            int x=1, y;
-            if( isNew( peer ) ) x *= 3;
-            if( isSame( peer ) ) x *= 3;
-            for( y=0; y<x; ++y )
-                tr_ptrArrayAppend( randPool, choke );
+            if( choke[i].isInterested )
+            {
+                const tr_peer * peer = choke[i].peer;
+                int x=1, y;
+                if( isNew( peer ) ) x *= 3;
+                if( isSame( peer ) ) x *= 3;
+                for( y=0; y<x; ++y )
+                    tr_ptrArrayAppend( randPool, choke );
+            }
         }
-        i = tr_rand( tr_ptrArraySize( randPool ) );
-        c = ( struct ChokeData* )tr_ptrArrayNth( randPool, i);
-        c->doUnchoke = 1;
-        t->optimistic = c->peer;
+
+        if(( n = tr_ptrArraySize( randPool )))
+        {
+            c = tr_ptrArrayNth( randPool, tr_rand( n ));
+            c->doUnchoke = 1;
+            t->optimistic = c->peer;
+        }
+
         tr_ptrArrayFree( randPool, NULL );
     }
 
@@ -1686,11 +1733,21 @@ shouldPeerBeClosed( const Torrent * t, const tr_peer * peer, int peerCount )
         return TRUE;
     }
 
-    /* if we're both seeds and it's been long enough for a pex exchange, close it */
-    if( 1 ) {
-        const int clientIsSeed = tr_torrentIsSeed( tor );
-        const int peerIsSeed = atom->flags & ADDED_F_SEED_FLAG;
-        if( peerIsSeed && clientIsSeed && ( !tr_torrentAllowsPex(tor) || (now-atom->time>=30) ) ) {
+    /* if we're seeding and the peer has everything we have,
+     * and enough time has passed for a pex exchange, then disconnect */
+    if( tr_torrentIsSeed( tor ) ) {
+        int peerHasEverything;
+        if( atom->flags & ADDED_F_SEED_FLAG )
+            peerHasEverything = TRUE;
+        else if( peer->progress < tr_cpPercentDone( tor->completion ) )
+            peerHasEverything = FALSE;
+        else {
+            tr_bitfield * tmp = tr_bitfieldDup( tr_cpPieceBitfield( tor->completion ) );
+            tr_bitfieldDifference( tmp, peer->have );
+            peerHasEverything = tr_bitfieldCountTrueBits( tmp ) == 0;
+            tr_bitfieldFree( tmp );
+        }
+        if( peerHasEverything && ( !tr_torrentAllowsPex(tor) || (now-atom->time>=30) ) ) {
             tordbg( t, "purging peer %s because we're both seeds", tr_peerIoAddrStr(&atom->addr,atom->port) );
             return TRUE;
         }
@@ -1698,7 +1755,7 @@ shouldPeerBeClosed( const Torrent * t, const tr_peer * peer, int peerCount )
 
     /* disconnect if it's been too long since piece data has been transferred.
      * this is on a sliding scale based on number of available peers... */
-    if( 1 ) {
+    {
         const int relaxStrictnessIfFewerThanN = (int)((getMaxPeerCount(tor) * 0.9) + 0.5);
         /* if we have >= relaxIfFewerThan, strictness is 100%.
          * if we have zero connections, strictness is 0% */
@@ -1814,6 +1871,10 @@ getPeerCandidates( Torrent * t, int * setmeSize )
             }
         }
 
+        /* Don't connect to peers in our blocklist */
+        if( tr_blocklistHasAddress( t->manager->handle, &atom->addr ) )
+            continue;
+
         ret[retCount++] = atom;
     }
 
