@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: platform.h 3254 2007-10-01 15:17:15Z charles $
+ * $Id: platform.h 5645 2008-04-18 23:17:40Z charles $
  *
- * Copyright (c) 2005 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,33 +24,43 @@
 #ifndef TR_PLATFORM_H
 #define TR_PLATFORM_H
 
+#if defined(WIN32)
+#define TR_PATH_DELIMITER '\\'
+#define TR_PATH_DELIMITER_STR "\\"
+#else
+#define TR_PATH_DELIMITER '/'
+#define TR_PATH_DELIMITER_STR "/"
+#endif
+
 typedef struct tr_lock   tr_lock;
-typedef struct tr_cond   tr_cond;
 typedef struct tr_thread tr_thread;
 
-const char * tr_getHomeDirectory( void );
-const char * tr_getCacheDirectory( void );
-const char * tr_getTorrentsDirectory( void );
+struct tr_handle;
+
+void tr_setConfigDir  ( struct tr_handle * handle,
+                        const char       * configDir );
+
+const char * tr_getResumeDir  ( const struct tr_handle * );
+
+const char * tr_getTorrentDir ( const struct tr_handle * );
 
 tr_thread*   tr_threadNew  ( void (*func)(void *), void * arg, const char * name );
-void         tr_threadJoin ( tr_thread * );
 int          tr_amInThread ( const tr_thread * );
 
 tr_lock *    tr_lockNew        ( void );
 void         tr_lockFree       ( tr_lock * );
-int          tr_lockTryLock    ( tr_lock * );
 void         tr_lockLock       ( tr_lock * );
 void         tr_lockUnlock     ( tr_lock * );
 int          tr_lockHave       ( const tr_lock * );
 
-tr_cond *    tr_condNew       ( void );
-void         tr_condFree      ( tr_cond * );
-void         tr_condSignal    ( tr_cond * );
-void         tr_condBroadcast ( tr_cond * );
-void         tr_condWait      ( tr_cond *, tr_lock * );
 
-struct in_addr; /* forward declaration to calm gcc down */
-int
-tr_getDefaultRoute( struct in_addr * addr );
+enum
+{
+  TR_LOCKFILE_SUCCESS,
+  TR_LOCKFILE_EOPEN,
+  TR_LOCKFILE_ELOCK
+};
+
+int          tr_lockfile       ( const char * filename );
 
 #endif
