@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: torrent.c 5010 2008-02-13 01:21:23Z charles $
+ * $Id: torrent.c 6181 2008-06-13 21:08:16Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,32 +22,32 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-#include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <sys/types.h> /* stat */
+#include <sys/stat.h> /* stat */
+#include <unistd.h> /* stat */
 
-#include <sys/types.h>
+#include <assert.h>
+#include <limits.h> /* INT_MAX */
+#include <string.h> /* memcmp */
+#include <stdlib.h> /* qsort */
 
 #include "transmission.h"
 #include "bencode.h"
 #include "completion.h"
 #include "crypto.h" /* for tr_sha1 */
-#include "fastresume.h"
+#include "resume.h"
 #include "fdlimit.h" /* tr_fdFileClose */
-#include "handshake.h"
-#include "inout.h"
 #include "metainfo.h"
-#include "net.h" /* tr_netNtop */
 #include "peer-mgr.h"
-#include "platform.h"
 #include "ratecontrol.h"
-#include "shared.h"
 #include "torrent.h"
 #include "tracker.h"
 #include "trcompat.h" /* for strlcpy */
 #include "trevent.h"
 #include "utils.h"
+#include "verify.h"
+
+#define MAX_BLOCK_SIZE (1024*16)
 
 /***
 ****
@@ -154,22 +154,35 @@ onTrackerResponse( void * tracker UNUSED, void * vevent, void * user_data )
 
     switch( event->messageType )
     {
-        case TR_TRACKER_PEERS:
-            tr_peerMgrAddPeers( tor->handle->peerMgr,
-                                tor->info.hash,
-                                TR_PEER_FROM_TRACKER,
-                                event->peerCompact,
-                                event->peerCount );
+        case TR_TRACKER_PEERS: {
+            size_t i, n;
+            tr_pex * pex = tr_peerMgrCompactToPex( event->compact,
+                                                   event->compactLen,
+                                                   NULL, &n );
+            if( event->allAreSeeds )
+                tr_tordbg( tor, "Got %d seeds from tracker", (int)n );
+            else
+                tr_torinf( tor, _( "Got %d peers from tracker" ), (int)n );
+
+            for( i=0; i<n; ++i ) {
+                if( event->allAreSeeds )
+                    pex[i].flags |= ADDED_F_SEED_FLAG;
+                tr_peerMgrAddPex( tor->handle->peerMgr, tor->info.hash,
+                                  TR_PEER_FROM_TRACKER, pex+i );
+            }
+
+            tr_free( pex );
             break;
+        }
 
         case TR_TRACKER_WARNING:
-            tr_err( "Tracker: Warning - %s", event->text );
+            tr_torerr( tor, _( "Tracker warning: \"%s\"" ), event->text );
             tor->error = TR_ERROR_TC_WARNING;
             strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
             break;
 
         case TR_TRACKER_ERROR:
-            tr_err( "Tracker: Error - %s", event->text );
+            tr_torerr( tor, _( "Tracker error: \"%s\"" ), event->text );
             tor->error = TR_ERROR_TC_ERROR;
             strlcpy( tor->errorString, event->text, sizeof(tor->errorString) );
             break;
@@ -197,13 +210,13 @@ getBytePiece( const tr_info * info, uint64_t byteOffset )
 }
 
 static void
-initFilePieces ( tr_info * info, int fileIndex )
+initFilePieces ( tr_info * info, tr_file_index_t fileIndex )
 {
     tr_file * file = &info->files[fileIndex];
     uint64_t firstByte, lastByte;
 
     assert( info != NULL );
-    assert( 0<=fileIndex && fileIndex<info->fileCount );
+    assert( fileIndex < info->fileCount );
 
     file = &info->files[fileIndex];
     firstByte = file->offset;
@@ -213,17 +226,17 @@ initFilePieces ( tr_info * info, int fileIndex )
 }
 
 static int
-pieceHasFile( int piece, const tr_file * file )
+pieceHasFile( tr_piece_index_t piece, const tr_file * file )
 {
     return ( file->firstPiece <= piece ) && ( piece <= file->lastPiece );
 }
 
 static tr_priority_t
 calculatePiecePriority ( const tr_torrent * tor,
-                         int                piece,
+                         tr_piece_index_t   piece,
                          int                fileHint )
 {
-    int i;
+    tr_file_index_t i;
     int priority = TR_PRI_LOW;
 
     /* find the first file that has data in this piece */
@@ -262,21 +275,90 @@ calculatePiecePriority ( const tr_torrent * tor,
 static void
 tr_torrentInitFilePieces( tr_torrent * tor )
 {
-    int i;
+    tr_file_index_t ff;
+    tr_piece_index_t pp;
     uint64_t offset = 0;
 
     assert( tor != NULL );
 
-    for( i=0; i<tor->info.fileCount; ++i ) {
-      tor->info.files[i].offset = offset;
-      offset += tor->info.files[i].length;
-      initFilePieces( &tor->info, i );
+    for( ff=0; ff<tor->info.fileCount; ++ff ) {
+      tor->info.files[ff].offset = offset;
+      offset += tor->info.files[ff].length;
+      initFilePieces( &tor->info, ff );
     }
 
-    for( i=0; i<tor->info.pieceCount; ++i )
-        tor->info.pieces[i].priority = calculatePiecePriority( tor, i, -1 );
+    for( pp=0; pp<tor->info.pieceCount; ++pp )
+        tor->info.pieces[pp].priority = calculatePiecePriority( tor, pp, -1 );
 }
 
+int
+tr_torrentPromoteTracker( tr_torrent * tor, int pos )
+{
+    int i;
+    int tier;
+
+    assert( tor != NULL );
+    assert( ( 0 <= pos ) && ( pos < tor->info.trackerCount ) );
+
+    /* the tier of the tracker we're promoting */
+    tier = tor->info.trackers[pos].tier;
+
+    /* find the index of that tier's first tracker */
+    for( i=0; i<tor->info.trackerCount; ++i )
+        if( tor->info.trackers[i].tier == tier )
+            break;
+
+    assert( i < tor->info.trackerCount );
+
+    /* promote the tracker at `pos' to the front of the tier */
+    if( i != pos ) {
+        const tr_tracker_info tmp = tor->info.trackers[i];
+        tor->info.trackers[i] = tor->info.trackers[pos];
+        tor->info.trackers[pos] = tmp;
+    }
+
+    /* return the new position of the tracker that started out at [pos] */
+    return i;
+}
+
+struct RandomTracker
+{
+    tr_tracker_info tracker;
+    int random_value;
+};
+
+/* the tiers will be sorted from lowest to highest,
+ * and trackers are randomized within the tiers */
+static int
+compareRandomTracker( const void * va, const void * vb )
+{
+    const struct RandomTracker * a = va;
+    const struct RandomTracker * b = vb;
+
+    if( a->tracker.tier != b->tracker.tier )
+        return a->tracker.tier - b->tracker.tier;
+
+   return a->random_value - b->random_value;
+}
+    
+static void
+randomizeTiers( tr_info * info )
+{
+    int i;
+    const int n = info->trackerCount;
+    struct RandomTracker * r = tr_new0( struct RandomTracker, n );
+    for( i=0; i<n; ++i ) {
+        r[i].tracker = info->trackers[i];
+        r[i].random_value = tr_rand( INT_MAX );
+    }
+    qsort( r, n, sizeof( struct RandomTracker ), compareRandomTracker );
+    for( i=0; i<n; ++i )
+        info->trackers[i] = r[i].tracker;
+    tr_free( r );
+}
+
+static void torrentStart( tr_torrent * tor, int reloadProgress );
+
 static void
 torrentRealInit( tr_handle     * h,
                  tr_torrent    * tor,
@@ -291,13 +373,15 @@ torrentRealInit( tr_handle     * h,
 
     tor->handle   = h;
 
+    randomizeTiers( info );
+
     /**
      * Decide on a block size.  constraints:
      * (1) most clients decline requests over 16 KiB
      * (2) pieceSize must be a multiple of block size
      */
     tor->blockSize = info->pieceSize;
-    while( tor->blockSize > (1024*16) )
+    while( tor->blockSize > MAX_BLOCK_SIZE )
         tor->blockSize /= 2;
 
     tor->lastPieceSize = info->totalSize % info->pieceSize;
@@ -358,7 +442,7 @@ torrentRealInit( tr_handle     * h,
 
     tor->checkedPieces = tr_bitfieldNew( tor->info.pieceCount );
     tr_torrentUncheck( tor );
-    loaded = tr_fastResumeLoad( tor, ~0, ctor );
+    loaded = tr_torrentLoadResume( tor, ~0, ctor );
     
     doStart = tor->isRunning;
     tor->isRunning = 0;
@@ -384,19 +468,18 @@ torrentRealInit( tr_handle     * h,
 
     /* maybe save our own copy of the metainfo */
     if( tr_ctorGetSave( ctor ) ) {
-        const benc_val_t * val;
+        const tr_benc * val;
         if( !tr_ctorGetMetainfo( ctor, &val ) ) {
-            int len;
-            uint8_t * text = (uint8_t*) tr_bencSave( val, &len );
-            tr_metainfoSave( tor->info.hashString,
-                             tor->handle->tag,
-                             text, len );
-            tr_free( text );
+            const char * filename = tor->info.torrent;
+            tr_bencSaveFile( filename, val );
+            tr_sessionSetTorrentFile( tor->handle, tor->info.hashString, filename );
         }
     }
 
+    tr_metainfoMigrate( h, &tor->info );
+
     if( doStart )
-        tr_torrentStart( tor );
+        torrentStart( tor, FALSE );
 }
 
 static int
@@ -420,7 +503,7 @@ tr_torrentParse( const tr_handle  * handle,
     int err = 0;
     int doFree;
     tr_info tmp;
-    const benc_val_t * metainfo;
+    const tr_benc * metainfo;
 
     if( setmeInfo == NULL )
         setmeInfo = &tmp;
@@ -429,7 +512,7 @@ tr_torrentParse( const tr_handle  * handle,
     if( !err && tr_ctorGetMetainfo( ctor, &metainfo ) )
         return TR_EINVALID;
 
-    err = tr_metainfoParse( setmeInfo, metainfo, handle->tag );
+    err = tr_metainfoParse( handle, setmeInfo, metainfo );
     doFree = !err && ( setmeInfo == &tmp );
 
     if( !err && hashExists( handle, setmeInfo->hash ) )
@@ -462,16 +545,6 @@ tr_torrentNew( tr_handle      * handle,
     return tor;
 }
 
-/***
-****
-***/
-
-static void
-saveFastResumeNow( tr_torrent * tor )
-{
-    tr_fastResumeSave( tor );
-}
-
 /**
 ***
 **/
@@ -479,9 +552,12 @@ saveFastResumeNow( tr_torrent * tor )
 void
 tr_torrentSetFolder( tr_torrent * tor, const char * path )
 {
-    tr_free( tor->destination );
-    tor->destination = tr_strdup( path );
-    saveFastResumeNow( tor );
+    if( !path || !tor->destination || strcmp( path, tor->destination ) )
+    {
+        tr_free( tor->destination );
+        tor->destination = tr_strdup( path );
+        tr_torrentSaveResume( tor );
+    }
 }
 
 const char*
@@ -551,7 +627,7 @@ tr_torrentGetRates( const tr_torrent * tor,
 const tr_info *
 tr_torrentInfo( const tr_torrent * tor )
 {
-    return &tor->info;
+    return tor ? &tor->info : NULL;
 }
 
 const tr_stat *
@@ -559,9 +635,25 @@ tr_torrentStatCached( tr_torrent * tor )
 {
     const time_t now = time( NULL );
 
-    return now == tor->lastStatTime
-        ? &tor->stats[tor->statCur]
-        : tr_torrentStat( tor );
+    return tor && ( now == tor->lastStatTime ) ? &tor->stats
+                                               : tr_torrentStat( tor );
+}
+
+tr_torrent_status
+tr_torrentGetStatus( tr_torrent * tor )
+{
+    tr_torrentRecheckCompleteness( tor );
+
+    if( tor->verifyState == TR_VERIFY_NOW )
+        return TR_STATUS_CHECK;
+    if( tor->verifyState == TR_VERIFY_WAIT )
+        return TR_STATUS_CHECK_WAIT;
+    if( !tor->isRunning )
+        return TR_STATUS_STOPPED;
+    if( tor->cpStatus == TR_CP_INCOMPLETE )
+        return TR_STATUS_DOWNLOAD;
+
+    return TR_STATUS_SEED;
 }
 
 const tr_stat *
@@ -570,14 +662,15 @@ tr_torrentStat( tr_torrent * tor )
     tr_stat * s;
     struct tr_tracker * tc;
 
+    if( !tor )
+        return NULL;
+
     tr_torrentLock( tor );
 
     tor->lastStatTime = time( NULL );
-    tr_torrentRecheckCompleteness( tor );
-
-    tor->statCur = !tor->statCur;
-    s = &tor->stats[tor->statCur];
 
+    s = &tor->stats;
+    s->status = tr_torrentGetStatus( tor );
     s->error  = tor->error;
     memcpy( s->errorString, tor->errorString,
             sizeof( s->errorString ) );
@@ -585,6 +678,8 @@ tr_torrentStat( tr_torrent * tor )
     tc = tor->tracker;
     s->tracker = tr_trackerGetAddress( tor->tracker );
 
+    tr_trackerStat( tor->tracker, &s->tracker_stat );
+
     tr_peerMgrTorrentStats( tor->handle->peerMgr,
                             tor->info.hash,
                             &s->peersKnown,
@@ -593,23 +688,13 @@ tr_torrentStat( tr_torrent * tor )
                             &s->peersGettingFromUs,
                              s->peersFrom );
 
+    s->manualAnnounceTime = tr_trackerGetManualAnnounceTime( tor->tracker );
+
     s->percentComplete = tr_cpPercentComplete ( tor->completion );
 
     s->percentDone = tr_cpPercentDone( tor->completion );
     s->leftUntilDone = tr_cpLeftUntilDone( tor->completion );
-
-    if( tor->recheckState == TR_RECHECK_NOW )
-        s->status = TR_STATUS_CHECK;
-    else if( tor->recheckState == TR_RECHECK_WAIT )
-        s->status = TR_STATUS_CHECK_WAIT;
-    else if( !tor->isRunning )
-        s->status = TR_STATUS_STOPPED;
-    else if( tor->cpStatus == TR_CP_INCOMPLETE )
-        s->status = TR_STATUS_DOWNLOAD;
-    else if( tor->cpStatus == TR_CP_DONE )
-        s->status = TR_STATUS_DONE;
-    else
-        s->status = TR_STATUS_SEED;
+    s->sizeWhenDone = tr_cpSizeWhenDone( tor->completion );
 
     s->recheckProgress =
         1.0 - (tr_torrentCountUncheckedPieces( tor ) / (double) tor->info.pieceCount);
@@ -626,10 +711,6 @@ tr_torrentStat( tr_torrent * tor )
     s->startDate = tor->startDate;
     s->activityDate = tor->activityDate;
 
-    s->eta = s->rateDownload < 0.1
-        ? -1.0f
-        : (s->leftUntilDone / s->rateDownload / 1024.0);
-
     s->corruptEver     = tor->corruptCur    + tor->corruptPrev;
     s->downloadedEver  = tor->downloadedCur + tor->downloadedPrev;
     s->uploadedEver    = tor->uploadedCur   + tor->uploadedPrev;
@@ -638,28 +719,24 @@ tr_torrentStat( tr_torrent * tor )
 
 
     {
-        int i;
-        tr_bitfield * availablePieces = tr_peerMgrGetAvailable( tor->handle->peerMgr,
-                                                                tor->info.hash );
-        s->desiredSize = 0;
+        tr_piece_index_t i;
+        tr_bitfield * peerPieces = tr_peerMgrGetAvailable( tor->handle->peerMgr,
+                                                           tor->info.hash );
         s->desiredAvailable = 0;
-
-        for( i=0; i<tor->info.pieceCount; ++i ) {
-            if( !tor->info.pieces[i].dnd ) {
-                const uint64_t byteCount = tr_torPieceCountBytes( tor, i );
-                s->desiredSize += byteCount;
-                if( tr_bitfieldHas( availablePieces, i ) )
-                    s->desiredAvailable += byteCount;
-            }
-        }
-
-        /* "availablePieces" can miss our unverified blocks... */
-        if( s->desiredAvailable < s->haveValid + s->haveUnchecked )
-            s->desiredAvailable = s->haveValid + s->haveUnchecked;
-
-        tr_bitfieldFree( availablePieces );
+        for( i=0; i<tor->info.pieceCount; ++i )
+            if( !tor->info.pieces[i].dnd && tr_bitfieldHas( peerPieces, i ) )
+                s->desiredAvailable += tr_cpMissingBlocksInPiece( tor->completion, i );
+        s->desiredAvailable *= tor->blockSize;
+        tr_bitfieldFree( peerPieces );
     }
 
+    if( s->leftUntilDone > s->desiredAvailable )
+        s->eta = TR_ETA_NOT_AVAIL;
+    else if( s->rateDownload < 0.1 )
+        s->eta = TR_ETA_UNKNOWN;
+    else
+        s->eta = s->leftUntilDone / s->rateDownload / 1024.0;
+
     s->ratio = tr_getRatio( s->uploadedEver,
                             s->downloadedEver ? s->downloadedEver : s->haveValid );
     
@@ -673,24 +750,24 @@ tr_torrentStat( tr_torrent * tor )
 ***/
 
 static uint64_t
-fileBytesCompleted ( const tr_torrent * tor, int fileIndex )
+fileBytesCompleted ( const tr_torrent * tor, tr_file_index_t fileIndex )
 {
     const tr_file * file     =  &tor->info.files[fileIndex];
-    const uint64_t firstBlock       =  file->offset / tor->blockSize;
+    const tr_block_index_t firstBlock       =  file->offset / tor->blockSize;
     const uint64_t firstBlockOffset =  file->offset % tor->blockSize;
     const uint64_t lastOffset       =  file->length ? (file->length-1) : 0;
-    const uint64_t lastBlock        = (file->offset + lastOffset) / tor->blockSize;
+    const tr_block_index_t lastBlock        = (file->offset + lastOffset) / tor->blockSize;
     const uint64_t lastBlockOffset  = (file->offset + lastOffset) % tor->blockSize;
     uint64_t haveBytes = 0;
 
     assert( tor != NULL );
-    assert( 0<=fileIndex && fileIndex<tor->info.fileCount );
+    assert( fileIndex < tor->info.fileCount );
     assert( file->offset + file->length <= tor->info.totalSize );
-    assert( ( (int)firstBlock < tor->blockCount ) || (!file->length && file->offset==tor->info.totalSize) );
-    assert( ( (int)lastBlock < tor->blockCount ) || (!file->length && file->offset==tor->info.totalSize) );
+    assert( ( firstBlock < tor->blockCount ) || (!file->length && file->offset==tor->info.totalSize) );
+    assert( ( lastBlock < tor->blockCount ) || (!file->length && file->offset==tor->info.totalSize) );
     assert( firstBlock <= lastBlock );
-    assert( (int)tr_torBlockPiece( tor, firstBlock ) == file->firstPiece );
-    assert( (int)tr_torBlockPiece( tor, lastBlock ) == file->lastPiece );
+    assert( tr_torBlockPiece( tor, firstBlock ) == file->firstPiece );
+    assert( tr_torBlockPiece( tor, lastBlock ) == file->lastPiece );
 
     if( firstBlock == lastBlock )
     {
@@ -699,7 +776,7 @@ fileBytesCompleted ( const tr_torrent * tor, int fileIndex )
     }
     else
     {
-        uint64_t i;
+        tr_block_index_t i;
 
         if( tr_cpBlockIsComplete( tor->completion, firstBlock ) )
             haveBytes += tor->blockSize - firstBlockOffset;
@@ -716,31 +793,27 @@ fileBytesCompleted ( const tr_torrent * tor, int fileIndex )
 }
 
 tr_file_stat *
-tr_torrentFiles( const tr_torrent * tor, int * fileCount )
+tr_torrentFiles( const tr_torrent * tor, tr_file_index_t * fileCount )
 {
-    int i;
-    const int n = tor->info.fileCount;
+    tr_file_index_t i;
+    const tr_file_index_t n = tor->info.fileCount;
     tr_file_stat * files = tr_new0( tr_file_stat, n );
     tr_file_stat * walk = files;
 
-    for( i=0; i<n; ++i, ++walk )
-    {
-        const tr_file * file = tor->info.files + i;
-
-        walk->bytesCompleted = fileBytesCompleted( tor, i );
-
-        walk->progress = file->length
-            ? walk->bytesCompleted / (float)file->length
-            : 1.0;
+    for( i=0; i<n; ++i, ++walk ) {
+        const uint64_t b = fileBytesCompleted( tor, i );
+        walk->bytesCompleted = b;
+        walk->progress = tr_getRatio( b, tor->info.files[i].length );
     }
 
-    *fileCount = n;
+    if( fileCount )
+        *fileCount = n;
 
     return files;
 }
 
 void
-tr_torrentFilesFree( tr_file_stat * files, int fileCount UNUSED )
+tr_torrentFilesFree( tr_file_stat * files, tr_file_index_t fileCount UNUSED )
 {
     tr_free( files );
 }
@@ -769,24 +842,16 @@ tr_torrentPeersFree( tr_peer_stat * peers, int peerCount UNUSED )
 
 void tr_torrentAvailability( const tr_torrent * tor, int8_t * tab, int size )
 {
-    return tr_peerMgrTorrentAvailability( tor->handle->peerMgr,
-                                          tor->info.hash,
-                                          tab, size );
+    tr_peerMgrTorrentAvailability( tor->handle->peerMgr,
+                                   tor->info.hash,
+                                   tab, size );
 }
 
-void tr_torrentAmountFinished( const tr_torrent * tor, float * tab, int size )
+void
+tr_torrentAmountFinished( const tr_torrent * tor, float * tab, int size )
 {
-    int i;
-    float interval;
     tr_torrentLock( tor );
-
-    interval = (float)tor->info.pieceCount / (float)size;
-    for( i = 0; i < size; i++ )
-    {
-        int piece = i * interval;
-        tab[i] = tr_cpPercentBlocksInPiece( tor->completion, piece );
-    }
-
+    tr_cpGetAmountDone( tor->completion, tab, size );
     tr_torrentUnlock( tor );
 }
 
@@ -807,12 +872,11 @@ tr_torrentResetTransferStats( tr_torrent * tor )
 
 
 void
-tr_torrentSetHasPiece( tr_torrent * tor, int pieceIndex, int has )
+tr_torrentSetHasPiece( tr_torrent * tor, tr_piece_index_t pieceIndex, int has )
 {
     tr_torrentLock( tor );
 
     assert( tor != NULL );
-    assert( pieceIndex >= 0 );
     assert( pieceIndex < tor->info.pieceCount );
 
     if( has )
@@ -823,9 +887,12 @@ tr_torrentSetHasPiece( tr_torrent * tor, int pieceIndex, int has )
     tr_torrentUnlock( tor );
 }
 
-void tr_torrentRemoveSaved( tr_torrent * tor )
+void
+tr_torrentRemoveSaved( tr_torrent * tor )
 {
-    tr_metainfoRemoveSaved( tor->info.hashString, tor->handle->tag );
+    tr_metainfoRemoveSaved( tor->handle, &tor->info );
+
+    tr_torrentRemoveResume( tor );
 }
 
 /***
@@ -872,8 +939,7 @@ freeTorrent( tr_torrent * tor )
     assert( h->torrentCount >= 1 );
     h->torrentCount--;
 
-    tr_inf( "closing torrent %s... %d torrents left",
-            tor->info.name, h->torrentCount );
+    tr_torinf( tor, _( "Closing torrent; %d left" ), h->torrentCount );
 
     tr_metainfoFree( inf );
     tr_free( tor );
@@ -881,13 +947,35 @@ freeTorrent( tr_torrent * tor )
     tr_globalUnlock( h );
 }
 
-enum
+/**
+***  Start/Stop Callback
+**/
+
+static void
+fireActiveChange( tr_torrent * tor, int isRunning )
 {
-    AFTER_RECHECK_NONE,
-    AFTER_RECHECK_START,
-    AFTER_RECHECK_STOP,
-    AFTER_RECHECK_CLOSE
-};
+    assert( tor != NULL );
+
+    if( tor->active_func != NULL )
+        (tor->active_func)( tor, isRunning, tor->active_func_user_data );
+}
+
+void
+tr_torrentSetActiveCallback( tr_torrent             * tor,
+                             tr_torrent_active_func   func,
+                             void                   * user_data )
+{
+    assert( tor != NULL );
+    tor->active_func = func;
+    tor->active_func_user_data = user_data;
+}
+
+void
+tr_torrentClearActiveCallback( tr_torrent * torrent )
+{
+    tr_torrentSetActiveCallback( torrent, NULL, NULL );
+}
+
 
 static void
 checkAndStartImpl( void * vtor )
@@ -897,10 +985,11 @@ checkAndStartImpl( void * vtor )
     tr_globalLock( tor->handle );
 
     tor->isRunning  = 1;
+    fireActiveChange( tor, tor->isRunning );
     *tor->errorString = '\0';
     tr_torrentResetTransferStats( tor );
     tor->cpStatus = tr_cpGetStatus( tor->completion );
-    saveFastResumeNow( tor );
+    tr_torrentSaveResume( tor );
     tor->startDate = tr_date( );
     tr_trackerStart( tor->tracker );
     tr_peerMgrStartTorrent( tor->handle->peerMgr, tor->info.hash );
@@ -914,21 +1003,28 @@ checkAndStartCB( tr_torrent * tor )
     tr_runInEventThread( tor->handle, checkAndStartImpl, tor );
 }
 
-void
-tr_torrentStart( tr_torrent * tor )
+static void
+torrentStart( tr_torrent * tor, int reloadProgress )
 {
     tr_globalLock( tor->handle );
 
     if( !tor->isRunning )
     {
-        tr_fastResumeLoad( tor, TR_FR_PROGRESS, NULL );
+        if( reloadProgress )
+            tr_torrentLoadResume( tor, TR_FR_PROGRESS, NULL );
         tor->isRunning = 1;
-        tr_ioRecheckAdd( tor, checkAndStartCB );
+        tr_verifyAdd( tor, checkAndStartCB );
     }
 
     tr_globalUnlock( tor->handle );
 }
 
+void
+tr_torrentStart( tr_torrent * tor )
+{
+    torrentStart( tor, TRUE );
+}
+
 static void
 torrentRecheckDoneImpl( void * vtor )
 {
@@ -940,12 +1036,14 @@ torrentRecheckDoneCB( tr_torrent * tor )
     tr_runInEventThread( tor->handle, torrentRecheckDoneImpl, tor );
 }
 void
-tr_torrentRecheck( tr_torrent * tor )
+tr_torrentVerify( tr_torrent * tor )
 {
+    tr_verifyRemove( tor );
+
     tr_globalLock( tor->handle );
 
     tr_torrentUncheck( tor );
-    tr_ioRecheckAdd( tor, torrentRecheckDoneCB );
+    tr_verifyAdd( tor, torrentRecheckDoneCB );
 
     tr_globalUnlock( tor->handle );
 }
@@ -954,12 +1052,13 @@ tr_torrentRecheck( tr_torrent * tor )
 static void
 stopTorrent( void * vtor )
 {
-    int i;
+    tr_file_index_t i;
 
     tr_torrent * tor = vtor;
-    tr_ioRecheckRemove( tor );
+    tr_verifyRemove( tor );
     tr_peerMgrStopTorrent( tor->handle->peerMgr, tor->info.hash );
     tr_trackerStop( tor->tracker );
+    fireActiveChange( tor, 0 );
 
     for( i=0; i<tor->info.fileCount; ++i )
     {
@@ -975,8 +1074,9 @@ tr_torrentStop( tr_torrent * tor )
 {
     tr_globalLock( tor->handle );
 
-    saveFastResumeNow( tor );
     tor->isRunning = 0;
+    if( !tor->isDeleting )
+        tr_torrentSaveResume( tor );
     tr_runInEventThread( tor->handle, stopTorrent, tor );
 
     tr_globalUnlock( tor->handle );
@@ -986,9 +1086,11 @@ static void
 closeTorrent( void * vtor )
 {
     tr_torrent * tor = vtor;
-    saveFastResumeNow( tor );
+    tr_torrentSaveResume( tor );
     tor->isRunning = 0;
     stopTorrent( tor );
+    if( tor->isDeleting )
+        tr_torrentRemoveSaved( tor );
     freeTorrent( tor );
 }
 
@@ -1007,10 +1109,33 @@ tr_torrentClose( tr_torrent * tor )
     }
 }
 
+void
+tr_torrentDelete( tr_torrent * tor )
+{
+    tor->isDeleting = 1;
+    tr_torrentClose( tor );
+}
+
+
 /**
 ***  Completeness
 **/
 
+static const char *
+getCompletionString( int type )
+{
+    switch( type )
+    {
+        /* Translators: this is a minor point that's safe to skip over, but FYI:
+           "Complete" and "Done" are specific, different terms in Transmission:
+           "Complete" means we've downloaded every file in the torrent.
+           "Done" means we're done downloading the files we wanted, but NOT all that exist */
+        case TR_CP_DONE:     return _( "Done" );
+        case TR_CP_COMPLETE: return _( "Complete" );
+        default:             return _( "Incomplete" );
+    }
+}
+
 static void
 fireStatusChange( tr_torrent * tor, cp_status_t status )
 {
@@ -1045,15 +1170,27 @@ tr_torrentRecheckCompleteness( tr_torrent * tor )
     tr_torrentLock( tor );
 
     cpStatus = tr_cpGetStatus( tor->completion );
-    if( cpStatus != tor->cpStatus ) {
+
+    if( cpStatus != tor->cpStatus )
+    {
+        const int recentChange = tor->downloadedCur != 0;
+
+        if( recentChange )
+        {
+            tr_torinf( tor, _( "State changed from \"%1$s\" to \"%2$s\"" ),
+                            getCompletionString( tor->cpStatus ),
+                            getCompletionString( cpStatus ) );
+        }
+
         tor->cpStatus = cpStatus;
         fireStatusChange( tor, cpStatus );
-        if( (cpStatus == TR_CP_COMPLETE) /* ...and if we're complete */
-            && tor->downloadedCur ) {        /* and it just happened */
-            tr_trackerCompleted( tor->tracker ); /* tell the tracker */
-        }
-        saveFastResumeNow( tor );
+
+        if( recentChange && ( cpStatus == TR_CP_COMPLETE ) )
+            tr_trackerCompleted( tor->tracker );
+
+        tr_torrentSaveResume( tor );
     }
+
     tr_torrentUnlock( tor );
 }
 
@@ -1068,15 +1205,15 @@ tr_torrentIsSeed( const tr_torrent * tor )
 **/
 
 void
-tr_torrentInitFilePriority( tr_torrent   * tor,
-                            int            fileIndex,
-                            tr_priority_t  priority )
+tr_torrentInitFilePriority( tr_torrent      * tor,
+                            tr_file_index_t   fileIndex,
+                            tr_priority_t     priority )
 {
-    int i;
+    tr_piece_index_t i;
     tr_file * file;
 
     assert( tor != NULL );
-    assert( 0<=fileIndex && fileIndex<tor->info.fileCount );
+    assert( fileIndex < tor->info.fileCount );
     assert( priority==TR_PRI_LOW || priority==TR_PRI_NORMAL || priority==TR_PRI_HIGH );
 
     file = &tor->info.files[fileIndex];
@@ -1086,29 +1223,29 @@ tr_torrentInitFilePriority( tr_torrent   * tor,
 }
 
 void
-tr_torrentSetFilePriorities( tr_torrent     * tor,
-                             int            * files,
-                             int              fileCount,
-                             tr_priority_t    priority )
+tr_torrentSetFilePriorities( tr_torrent       * tor,
+                             tr_file_index_t  * files,
+                             tr_file_index_t    fileCount,
+                             tr_priority_t      priority )
 {
-    int i;
+    tr_file_index_t i;
     tr_torrentLock( tor );
 
     for( i=0; i<fileCount; ++i )
         tr_torrentInitFilePriority( tor, files[i], priority );
 
-    saveFastResumeNow( tor );
+    tr_torrentSaveResume( tor );
     tr_torrentUnlock( tor );
 }
 
 tr_priority_t
-tr_torrentGetFilePriority( const tr_torrent *  tor, int file )
+tr_torrentGetFilePriority( const tr_torrent *  tor, tr_file_index_t file )
 {
     tr_priority_t ret;
 
     tr_torrentLock( tor );
     assert( tor != NULL );
-    assert( 0<=file && file<tor->info.fileCount );
+    assert( file < tor->info.fileCount );
     ret = tor->info.files[file].priority;
     tr_torrentUnlock( tor );
 
@@ -1118,7 +1255,7 @@ tr_torrentGetFilePriority( const tr_torrent *  tor, int file )
 tr_priority_t*
 tr_torrentGetFilePriorities( const tr_torrent * tor )
 {
-    int i;
+    tr_file_index_t i;
     tr_priority_t * p;
 
     tr_torrentLock( tor );
@@ -1136,12 +1273,12 @@ tr_torrentGetFilePriorities( const tr_torrent * tor )
 
 int
 tr_torrentGetFileDL( const tr_torrent * tor,
-                     int                file )
+                     tr_file_index_t    file )
 {
     int doDownload;
     tr_torrentLock( tor );
 
-    assert( 0<=file && file<tor->info.fileCount );
+    assert( file < tor->info.fileCount );
     doDownload = !tor->info.files[file].dnd;
 
     tr_torrentUnlock( tor );
@@ -1149,15 +1286,15 @@ tr_torrentGetFileDL( const tr_torrent * tor,
 }
 
 static void
-setFileDND( tr_torrent  * tor,
-            int           fileIndex,
-            int           doDownload )
+setFileDND( tr_torrent      * tor,
+            tr_file_index_t   fileIndex,
+            int               doDownload )
 {
     tr_file * file;
     const int dnd = !doDownload;
-    int firstPiece, firstPieceDND;
-    int lastPiece, lastPieceDND;
-    int i;
+    tr_piece_index_t firstPiece, firstPieceDND;
+    tr_piece_index_t lastPiece, lastPieceDND;
+    tr_file_index_t i;
 
     file = &tor->info.files[fileIndex];
     file->dnd = dnd;
@@ -1167,10 +1304,14 @@ setFileDND( tr_torrent  * tor,
     /* can't set the first piece to DND unless
        every file using that piece is DND */
     firstPieceDND = dnd;
-    for( i=fileIndex-1; firstPieceDND && i>=0; --i ) {
-        if( tor->info.files[i].lastPiece != firstPiece )
-            break;
-        firstPieceDND = tor->info.files[i].dnd;
+    if( fileIndex > 0 ) {
+        for( i=fileIndex-1; firstPieceDND; --i ) {
+            if( tor->info.files[i].lastPiece != firstPiece )
+                break;
+            firstPieceDND = tor->info.files[i].dnd;
+            if( !i )
+                break;
+        }
     }
 
     /* can't set the last piece to DND unless
@@ -1188,20 +1329,21 @@ setFileDND( tr_torrent  * tor,
     }
     else
     {
+        tr_piece_index_t pp;
         tor->info.pieces[firstPiece].dnd = firstPieceDND;
         tor->info.pieces[lastPiece].dnd = lastPieceDND;
-        for( i=firstPiece+1; i<lastPiece; ++i )
-            tor->info.pieces[i].dnd = dnd;
+        for( pp=firstPiece+1; pp<lastPiece; ++pp )
+            tor->info.pieces[pp].dnd = dnd;
     }
 }
 
 void
-tr_torrentInitFileDLs ( tr_torrent   * tor,
-                        int          * files,
-                        int            fileCount,
-                        int            doDownload )
+tr_torrentInitFileDLs ( tr_torrent       * tor,
+                        tr_file_index_t  * files,
+                        tr_file_index_t    fileCount,
+                        int                doDownload )
 {
-    int i;
+    tr_file_index_t i;
     tr_torrentLock( tor );
 
     for( i=0; i<fileCount; ++i )
@@ -1212,14 +1354,14 @@ tr_torrentInitFileDLs ( tr_torrent   * tor,
 }
 
 void
-tr_torrentSetFileDLs ( tr_torrent  * tor,
-                       int         * files,
-                       int           fileCount,
-                       int           doDownload )
+tr_torrentSetFileDLs ( tr_torrent      * tor,
+                       tr_file_index_t * files,
+                       tr_file_index_t   fileCount,
+                       int               doDownload )
 {
     tr_torrentLock( tor );
     tr_torrentInitFileDLs( tor, files, fileCount, doDownload );
-    saveFastResumeNow( tor );
+    tr_torrentSaveResume( tor );
     tr_torrentUnlock( tor );
 }
 
@@ -1244,20 +1386,50 @@ tr_torrentGetMaxConnectedPeers( const tr_torrent  * tor )
 ****
 ***/
 
-int _tr_block( const tr_torrent * tor, int index, int begin )
+tr_block_index_t
+_tr_block( const tr_torrent  * tor,
+           tr_piece_index_t    index,
+           uint32_t            offset )
 {
     const tr_info * inf = &tor->info;
-    return index * ( inf->pieceSize / tor->blockSize ) +
-        begin / tor->blockSize;
+    tr_block_index_t ret;
+    ret = index;
+    ret *= ( inf->pieceSize / tor->blockSize );
+    ret += offset / tor->blockSize;
+    return ret;
 }
 
+int
+tr_torrentReqIsValid( const tr_torrent * tor,
+                      tr_piece_index_t   index,
+                      uint32_t           offset,
+                      uint32_t           length )
+{
+    int err = 0;
+
+    if( index >= tor->info.pieceCount )
+        err = 1;
+    else if ( offset >= tr_torPieceCountBytes( tor, index ) )
+        err = 2;
+    else if( length > MAX_BLOCK_SIZE )
+        err = 3;
+    else if( tr_pieceOffset( tor, index, offset, length ) > tor->info.totalSize )
+        err = 4;
+
+    return !err;
+}
+
+
 uint64_t
-tr_pieceOffset( const tr_torrent * tor, int index, int begin, int length )
+tr_pieceOffset( const tr_torrent * tor,
+                tr_piece_index_t   index,
+                uint32_t           offset,
+                uint32_t           length )
 {
     uint64_t ret;
     ret = tor->info.pieceSize;
     ret *= index;
-    ret += begin;
+    ret += offset;
     ret += length;
     return ret;
 }
@@ -1267,13 +1439,13 @@ tr_pieceOffset( const tr_torrent * tor, int index, int begin, int length )
 ***/
 
 int
-tr_torrentIsPieceChecked( const tr_torrent * tor, int piece )
+tr_torrentIsPieceChecked( const tr_torrent * tor, tr_piece_index_t piece )
 {
     return tr_bitfieldHas( tor->checkedPieces, piece );
 }
 
 void
-tr_torrentSetPieceChecked( tr_torrent * tor, int piece, int isChecked )
+tr_torrentSetPieceChecked( tr_torrent * tor, tr_piece_index_t piece, int isChecked )
 {
     if( isChecked )
         tr_bitfieldAdd( tor->checkedPieces, piece );
@@ -1282,11 +1454,11 @@ tr_torrentSetPieceChecked( tr_torrent * tor, int piece, int isChecked )
 }
 
 void
-tr_torrentSetFileChecked( tr_torrent * tor, int fileIndex, int isChecked )
+tr_torrentSetFileChecked( tr_torrent * tor, tr_file_index_t fileIndex, int isChecked )
 {
     const tr_file * file = &tor->info.files[fileIndex];
-    const size_t begin = file->firstPiece;
-    const size_t end = file->lastPiece + 1;
+    const tr_piece_index_t begin = file->firstPiece;
+    const tr_piece_index_t end = file->lastPiece + 1;
 
     if( isChecked )
         tr_bitfieldAddRange ( tor->checkedPieces, begin, end );
@@ -1295,12 +1467,12 @@ tr_torrentSetFileChecked( tr_torrent * tor, int fileIndex, int isChecked )
 }
 
 int
-tr_torrentIsFileChecked( const tr_torrent * tor, int fileIndex )
+tr_torrentIsFileChecked( const tr_torrent * tor, tr_file_index_t fileIndex )
 {
     const tr_file * file = &tor->info.files[fileIndex];
-    const size_t begin = file->firstPiece;
-    const size_t end = file->lastPiece + 1;
-    size_t i;
+    const tr_piece_index_t begin = file->firstPiece;
+    const tr_piece_index_t end = file->lastPiece + 1;
+    tr_piece_index_t i;
     int isChecked = TRUE;
 
     for( i=begin; isChecked && i<end; ++i )
@@ -1321,3 +1493,28 @@ tr_torrentCountUncheckedPieces( const tr_torrent * tor )
 {
     return tor->info.pieceCount - tr_bitfieldCountTrueBits( tor->checkedPieces );
 }
+
+time_t*
+tr_torrentGetMTimes( const tr_torrent * tor, int * setme_n )
+{
+    int i;
+    const int n = tor->info.fileCount;
+    time_t * m = tr_new0( time_t, n );
+
+    for( i=0; i<n; ++i ) {
+        char fname[MAX_PATH_LENGTH];
+        struct stat sb;
+        tr_buildPath( fname, sizeof(fname),
+                      tor->destination, tor->info.files[i].name, NULL );
+        if ( !stat( fname, &sb ) ) {
+#ifdef SYS_DARWIN
+            m[i] = sb.st_mtimespec.tv_sec;
+#else
+            m[i] = sb.st_mtime;
+#endif
+        }
+    }
+
+    *setme_n = n;
+    return m;
+}
