@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: transmission.h 4854 2008-01-28 17:53:32Z charles $
+ * $Id: transmission.h 5673 2008-04-24 01:42:53Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -31,14 +31,14 @@ extern "C" {
 
 #include "version.h"
 
-#include <inttypes.h>
+#include <inttypes.h> /* uintN_t */
 #ifndef PRId64
 # define PRId64 "lld"
 #endif
 #ifndef PRIu64
 # define PRIu64 "llu"
 #endif
-#include <time.h>
+#include <time.h> /* time_t */
 
 #define SHA_DIGEST_LENGTH 20
 #ifdef __BEOS__
@@ -48,19 +48,9 @@ extern "C" {
 # define MAX_PATH_LENGTH  1024
 #endif
 
-#ifndef INET_ADDRSTRLEN
-#define INET_ADDRSTRLEN 16
-#endif
-
-#if defined(WIN32)
-#define TR_PATH_DELIMITER '\\'
-#define TR_PATH_DELIMITER_STR "\\"
-#else
-#define TR_PATH_DELIMITER '/'
-#define TR_PATH_DELIMITER_STR "/"
-#endif
-
-#define TR_DEFAULT_PORT   51413
+typedef uint32_t tr_file_index_t;
+typedef uint32_t tr_piece_index_t;
+typedef uint64_t tr_block_index_t;
 
 enum
 {
@@ -71,49 +61,32 @@ enum
     TR_PEER_FROM__MAX
 };
 
-/***********************************************************************
- * Error codes
- **********************************************************************/
-/* General errors */
-#define TR_OK                       0x00000000
-#define TR_ERROR                    0x81000000
-#define TR_ERROR_ASSERT             0x82000000
-/* I/O errors */
-#define TR_ERROR_IO_MASK            0x000000FF
-#define TR_ERROR_IO_PERMISSIONS     0x80000002
-#define TR_ERROR_IO_SPACE           0x80000004
-#define TR_ERROR_IO_FILE_TOO_BIG    0x80000008
-#define TR_ERROR_IO_OPEN_FILES      0x80000010
-#define TR_ERROR_IO_DUP_DOWNLOAD    0x80000020
-#define TR_ERROR_IO_OTHER           0x80000040
-/* Misc */
-#define TR_ERROR_TC_MASK            0x00000F00
-#define TR_ERROR_TC_ERROR           0x80000100
-#define TR_ERROR_TC_WARNING         0x80000200
-
-#define TR_ERROR_ISSET( num, code ) ( (code) == ( (code) & (num) ) )
-
 /***********************************************************************
  * tr_init
  ***********************************************************************
- * Initializes a libtransmission instance and returns an opaque handle
+ * Initializes and returns an opaque libtransmission handle
  * to be passed to functions below. The tag argument is a short string
  * unique to the program invoking tr_init(), it is currently used as
  * part of saved torrent files' names to prevent one frontend from
  * deleting a torrent used by another. The following tags are used:
- *   beos cli daemon gtk macosx
+ *   beos cli daemon gtk macosx wx
  **********************************************************************/
 
 typedef struct tr_handle tr_handle;
 
-tr_handle * tr_init( const char * tag );
+const char* tr_getDefaultConfigDir( void );
 
-/* shut down a libtransmission instance created by tr_init(). */
-void tr_close( tr_handle * );
+#define TR_DEFAULT_CONFIG_DIR                  tr_getDefaultConfigDir()
+#define TR_DEFAULT_PEX_ENABLED                 1
+#define TR_DEFAULT_PORT_FORWARDING_ENABLED     0
+#define TR_DEFAULT_PORT                        51413
+#define TR_DEFAULT_GLOBAL_PEER_LIMIT           200
+#define TR_DEFAULT_PEER_SOCKET_TOS             8
 
-tr_handle * tr_initFull( const char * tag,
+tr_handle * tr_initFull( const char * configDir,
+                         const char * tag,
                          int          isPexEnabled,
-                         int          isNatEnabled,
+                         int          isPortForwardingEnabled,
                          int          publicPort,
                          int          encryptionMode,
                          int          isUploadLimitEnabled,
@@ -122,7 +95,20 @@ tr_handle * tr_initFull( const char * tag,
                          int          downloadLimit,
                          int          globalPeerLimit,
                          int          messageLevel,
-                         int          isMessageQueueingEnabled );
+                         int          isMessageQueueingEnabled,
+                         int          isBlocklistEnabled,
+                         int          peerSocketTOS );
+
+/**
+ * Like tr_initFull() but with default values supplied.
+ */ 
+tr_handle * tr_init( const char * configDir,
+                     const char * tag );
+
+/**
+ * Shut down a libtransmission instance created by tr_init*()
+ */
+void tr_close( tr_handle * );
 
 
 /**
@@ -140,13 +126,15 @@ typedef struct tr_session_stats
 }
 tr_session_stats;
 
+/* stats from the current session. */
+void tr_getSessionStats( const tr_handle   * handle,
+                         tr_session_stats  * setme );
+
 /* stats from the current and past sessions. */
 void tr_getCumulativeSessionStats( const tr_handle   * handle,
                                    tr_session_stats  * setme );
 
-/* stats from the current session. */
-void tr_getSessionStats( const tr_handle   * handle,
-                         tr_session_stats  * setme );
+void tr_clearSessionStats( tr_handle * handle );
 
 
 /**
@@ -165,78 +153,101 @@ tr_encryption_mode tr_getEncryptionMode( tr_handle * handle );
 
 void tr_setEncryptionMode( tr_handle * handle, tr_encryption_mode mode );
 
-/***********************************************************************
- * tr_setMessageLevel
- ***********************************************************************
- * Set the level of messages to be output or queued
- **********************************************************************/
-#define TR_MSG_ERR 1
-#define TR_MSG_INF 2
-#define TR_MSG_DBG 3
-void tr_setMessageLevel( int );
-int tr_getMessageLevel( void );
-
-/***********************************************************************
- * tr_setMessageQueuing
- ***********************************************************************
- * Enable or disable message queuing
- **********************************************************************/
-typedef struct tr_msg_list tr_msg_list;
-void tr_setMessageQueuing( int );
-
-/***********************************************************************
- * tr_getQueuedMessages
- ***********************************************************************
- * Return a list of queued messages
- **********************************************************************/
-tr_msg_list * tr_getQueuedMessages( void );
-void tr_freeMessageList( tr_msg_list * list );
-
 /***********************************************************************
  * tr_getPrefsDirectory
  ***********************************************************************
  * Returns the full path to a directory which can be used to store
  * preferences. The string belongs to libtransmission, do not free it.
  **********************************************************************/
-const char * tr_getPrefsDirectory( void );
+const char * tr_getConfigDir( const tr_handle * );
+
+
+
 
 /***********************************************************************
- * tr_setBindPort
- ***********************************************************************
- * Sets the port to listen for incoming peer connections.
- * This can be safely called even with active torrents.
- **********************************************************************/
+** Message Logging
+*/
+
+enum
+{
+    TR_MSG_ERR = 1,
+    TR_MSG_INF = 2,
+    TR_MSG_DBG = 3
+};
+void tr_setMessageLevel( int );
+int tr_getMessageLevel( void );
+
+typedef struct tr_msg_list
+{
+    /* TR_MSG_ERR, TR_MSG_INF, or TR_MSG_DBG */
+    uint8_t level;
+
+    /* Time the message was generated */
+    time_t when;
+
+    /* The torrent associated with this message,
+     * or a module name such as "Port Forwarding" for non-torrent messages,
+     * or NULL. */
+    char * name;
+
+    /* The message */
+    char * message;
+
+    /* The source file where this message originated */
+    const char * file;
+
+    /* The line number in the source file where this message originated */
+    int line;
+
+    /* linked list of messages */
+    struct tr_msg_list * next;
+}
+tr_msg_list;
+
+void tr_setMessageQueuing( int enable );
+
+tr_msg_list * tr_getQueuedMessages( void );
+void tr_freeMessageList( tr_msg_list * freeme );
+
+/***********************************************************************
+** Incoming Peer Connections Port
+*/
+
 void tr_setBindPort( tr_handle *, int );
 
+void tr_natTraversalEnable( tr_handle *, int enable );
+
 int tr_getPublicPort( const tr_handle * );
 
-/***********************************************************************
- * tr_natTraversalEnable
- * tr_natTraversalDisable
- ***********************************************************************
- * Enable or disable NAT traversal using NAT-PMP or UPnP IGD.
- **********************************************************************/
-void tr_natTraversalEnable( tr_handle *, int enable );
+typedef enum
+{
+    TR_NAT_TRAVERSAL_ERROR,
+    TR_NAT_TRAVERSAL_UNMAPPED,
+    TR_NAT_TRAVERSAL_UNMAPPING,
+    TR_NAT_TRAVERSAL_MAPPING,
+    TR_NAT_TRAVERSAL_MAPPED
+}
+tr_nat_traversal_status;
 
-/***********************************************************************
- * tr_handleStatus
- ***********************************************************************
- * Returns some status info for the given handle.
- **********************************************************************/
-typedef struct tr_handle_status tr_handle_status;
-tr_handle_status * tr_handleStatus( tr_handle * );
+typedef struct tr_handle_status
+{
+    tr_nat_traversal_status natTraversalStatus;
+    int publicPort;
+}
+tr_handle_status;
+
+const tr_handle_status * tr_handleStatus( tr_handle * );
 
 
 /***********************************************************************
- * tr_torrentCount
- ***********************************************************************
- * Returns the count of open torrents
- **********************************************************************/
-int tr_torrentCount( const tr_handle * h );
-
+***
+***  TORRENTS
+**/
 
 typedef struct tr_torrent tr_torrent;
 
+int tr_torrentCount( const tr_handle * h );
+
 /***********************************************************************
 *** Speed Limits
 **/
@@ -294,6 +305,39 @@ void tr_setGlobalPeerLimit( tr_handle * handle,
 uint16_t tr_getGlobalPeerLimit( const tr_handle * handle );
 
 
+/**
+ * Specify a range of IPs for Transmission to block.
+ *
+ * filename must be an uncompressed ascii file,
+ * using the same format as the bluetack level1 file.
+ *
+ * libtransmission does not keep a handle to `filename'
+ * after this call returns, so the caller is free to
+ * keep or delete `filename' as it wishes.
+ * libtransmission makes its own copy of the file
+ * massaged into a format easier to search.
+ *
+ * The caller only needs to invoke this when the blocklist
+ * has changed.
+ *
+ * Passing NULL for a filename will clear the blocklist.
+ */
+int tr_blocklistSetContent( tr_handle  * handle,
+                            const char * filename );
+
+int tr_blocklistGetRuleCount( tr_handle * handle );
+
+int tr_blocklistExists( const tr_handle * handle );
+
+int tr_blocklistIsEnabled( const tr_handle * handle );
+
+void tr_blocklistSetEnabled( tr_handle * handle,
+                             int         isEnabled );
+
+struct in_addr;
+
+int tr_blocklistHasAddress( tr_handle             * handle,
+                            const struct in_addr  * addr);
 
 
 /***********************************************************************
@@ -312,8 +356,8 @@ typedef int8_t tr_priority_t;
 /* set a batch of files to a particular priority.
  * priority must be one of TR_PRI_NORMAL, _HIGH, or _LOW */
 void tr_torrentSetFilePriorities( tr_torrent        * tor,
-                                  int               * files,
-                                  int                 fileCount,
+                                  tr_file_index_t   * files,
+                                  tr_file_index_t     fileCount,
                                   tr_priority_t       priority );
 
 /* returns a malloc()ed array of tor->info.fileCount items,
@@ -323,16 +367,16 @@ tr_priority_t* tr_torrentGetFilePriorities( const tr_torrent * );
 
 /* single-file form of tr_torrentGetFilePriorities.
  * returns one of TR_PRI_NORMAL, _HIGH, or _LOW. */
-tr_priority_t tr_torrentGetFilePriority( const tr_torrent *, int file );
+tr_priority_t tr_torrentGetFilePriority( const tr_torrent *, tr_file_index_t file );
 
 /* returns true if the file's `download' flag is set */
-int tr_torrentGetFileDL( const tr_torrent *, int file );
+int tr_torrentGetFileDL( const tr_torrent *, tr_file_index_t file );
 
 /* set a batch of files to be downloaded or not. */
-void tr_torrentSetFileDLs ( tr_torrent   * tor,
-                            int          * files,
-                            int            fileCount,
-                            int            do_download );
+void tr_torrentSetFileDLs ( tr_torrent      * tor,
+                            tr_file_index_t * files,
+                            tr_file_index_t   fileCount,
+                            int               do_download );
 
 /***********************************************************************
  * tr_torrentRates
@@ -346,10 +390,9 @@ void tr_torrentRates( tr_handle *, float *, float * );
 /**
  *  Torrent Instantiation
  *
- *  Instantiating a tr_torrent has gotten a lot more complaticated as we
- *  add more options to them.  At the worst point there were four functions
- *  to check metainfo, and four (or five) functions to create tr_torrents.
- *  They all did mostly the same thing, but each was just *slightly* different.
+ *  Instantiating a tr_torrent had gotten more complicated as features were
+ *  added.  At one point there were four functions to check metainfo and five
+ *  to create tr_torrent.
  *
  *  To remedy this, a Torrent Constructor (struct tr_ctor) has been introduced:
  *  + Simplifies the API down to two (non-deprecated) functions.
@@ -381,14 +424,17 @@ typedef enum
 tr_ctorMode;
 
 typedef struct tr_ctor tr_ctor;
-struct benc_val_s;
+struct tr_benc;
 
 tr_ctor* tr_ctorNew                    ( const tr_handle  * handle);
 
 void     tr_ctorFree                   ( tr_ctor        * ctor );
 
 void     tr_ctorSetSave                ( tr_ctor        * ctor,
-                                         int              saveMetadataInOurTorrentsDir );
+                                         int     saveMetadataInOurTorrentsDir );
+
+void     tr_ctorSetDeleteSource       ( tr_ctor        * ctor,
+                                         uint8_t          doDelete );
 
 int      tr_ctorSetMetainfo            ( tr_ctor        * ctor,
                                          const uint8_t  * metainfo,
@@ -416,7 +462,7 @@ int      tr_ctorGetMaxConnectedPeers   ( const tr_ctor  * ctor,
                                          tr_ctorMode      mode,
                                          uint16_t       * setmeCount );
 
-int      tr_ctorGetIsPaused            ( const tr_ctor  * ctor,
+int      tr_ctorGetPaused              ( const tr_ctor  * ctor,
                                          tr_ctorMode      mode,
                                          uint8_t        * setmeIsPaused );
 
@@ -425,27 +471,34 @@ int      tr_ctorGetDestination         ( const tr_ctor  * ctor,
                                          const char    ** setmeDestination );
 
 int      tr_ctorGetMetainfo            ( const tr_ctor  * ctor,
-                                         const struct benc_val_s ** setme );
+                                         const struct tr_benc ** setme );
 
 int      tr_ctorGetSave                ( const tr_ctor  * ctor );
+ 
+int      tr_ctorGetDeleteSource        ( const tr_ctor  * ctor,
+                                         uint8_t        * setmeDoDelete );
+
+/* returns NULL if tr_ctorSetMetainfoFromFile() wasn't used */
+const char* tr_ctorGetSourceFile       ( const tr_ctor  * ctor );
+
 
 typedef struct tr_info tr_info;
 
 /**
  * Parses the specified metainfo.
  * Returns TR_OK if it parsed and can be added to Transmission.
- * Returns TR_INVALID if it couldn't be parsed.
+ * Returns TR_EINVALID if it couldn't be parsed.
  * Returns TR_EDUPLICATE if it parsed but can't be added. 
  *     "destination" must be set to test for TR_EDUPLICATE.
  *
- * "setme_info" can be NULL if you don't need the information.
- * If the metainfo can be parsed and setme_info is non-NULL,
- * it will be filled with the metainfo's info.  You'll need to
- * call tr_metainfoFree( setme_info ) when done with it.
+ * If setme_info is non-NULL and parsing is successful
+ * (that is, if TR_EINVALID is not returned), then the parsed
+ * metainfo is stored in setme_info and should be freed by the
+ * caller via tr_metainfoFree().
  */
 int tr_torrentParse( const tr_handle  * handle,
                      const tr_ctor    * ctor,
-                     tr_info          * setme_info );
+                     tr_info          * setme_info_or_NULL );
 
 /**
  * Instantiate a single torrent.
@@ -460,7 +513,7 @@ tr_torrent * tr_torrentNew( tr_handle      * handle,
 
 
 /**
- *  Load all the torrents in tr_getTorrentsDirectory().
+ *  Load all the torrents in tr_getTorrentDir().
  *  This can be used at startup to kickstart all the torrents
  *  from the previous session.
  */
@@ -472,8 +525,8 @@ tr_torrent ** tr_loadTorrents ( tr_handle  * h,
 
 /**
  * Set whether or not torrents are allowed to do peer exchanges.
- * By default, PEX is enabled.
  * PEX is always disabled in private torrents regardless of this.
+ * In public torrents, PEX is enabled by default.
  */
 void tr_setPexEnabled( tr_handle *, int isEnabled );
 
@@ -481,34 +534,17 @@ int tr_isPexEnabled( const tr_handle * );
 
 const tr_info * tr_torrentInfo( const tr_torrent * );
 
-void   tr_torrentSetFolder( tr_torrent *, const char * );
+void tr_torrentSetFolder( tr_torrent *, const char * );
 
 const char * tr_torrentGetFolder( const tr_torrent * );
 
-/***********************************************************************
- * tr_torrentStart
- ***********************************************************************
- * Starts downloading. The download is launched in a seperate thread,
- * therefore tr_torrentStart returns immediately.
- **********************************************************************/
-void   tr_torrentStart( tr_torrent * );
+void tr_torrentStart( tr_torrent * );
 
-/***********************************************************************
- * tr_torrentStop
- ***********************************************************************
- * Stops downloading and notices the tracker that we are leaving. The
- * thread keeps running while doing so.
- * The thread will eventually be joined, either:
- * - by tr_torrentStat when the tracker has been successfully noticed,
- * - by tr_torrentStat if the tracker could not be noticed within 60s,
- * - by tr_torrentClose if you choose to remove the torrent without
- *   waiting any further.
- **********************************************************************/
 void tr_torrentStop( tr_torrent * );
 
 
 /**
-***  Register to be notified whenever a torrent's state changes.
+***
 **/
 
 typedef enum
@@ -523,12 +559,45 @@ typedef void (tr_torrent_status_func)(tr_torrent   * torrent,
                                       cp_status_t    status,
                                       void         * user_data );
 
+/**
+ * Register to be notified whenever a torrent's state changes.
+ *
+ * func is invoked FROM LIBTRANSMISSION'S THREAD!
+ * This means func must be fast (to avoid blocking peers),
+ * shouldn't call libtransmission functions (to avoid deadlock),
+ * and shouldn't modify client-level memory without using a mutex!
+ */
 void tr_torrentSetStatusCallback( tr_torrent             * torrent,
                                   tr_torrent_status_func   func,
                                   void                   * user_data );
 
 void tr_torrentClearStatusCallback( tr_torrent * torrent );
 
+
+
+/**
+***
+**/
+
+typedef void (tr_torrent_active_func)(tr_torrent   * torrent,
+                                      int            isRunning,
+                                      void         * user_data );
+
+/**
+ * Register to be notified whenever a torrent starts or stops.
+ *
+ * func is invoked FROM LIBTRANSMISSION'S THREAD!
+ * This means func must be fast (to avoid blocking peers),
+ * shouldn't call libtransmission functions (to avoid deadlock),
+ * and shouldn't modify client-level memory without using a mutex!
+ */
+void tr_torrentSetActiveCallback( tr_torrent             * torrent,
+                                  tr_torrent_active_func   func,
+                                  void                   * user_data );
+
+void tr_torrentClearActiveCallback( tr_torrent * torrent );
+
+
 /**
  * MANUAL ANNOUNCE
  *
@@ -568,8 +637,8 @@ tr_peer_stat * tr_torrentPeers( const tr_torrent *, int * peerCount );
 void tr_torrentPeersFree( tr_peer_stat *, int peerCount );
 
 typedef struct tr_file_stat tr_file_stat;
-tr_file_stat * tr_torrentFiles( const tr_torrent *, int * fileCount );
-void tr_torrentFilesFree( tr_file_stat *, int fileCount );
+tr_file_stat * tr_torrentFiles( const tr_torrent *, tr_file_index_t * fileCount );
+void tr_torrentFilesFree( tr_file_stat *, tr_file_index_t fileCount );
 
 
 /***********************************************************************
@@ -588,11 +657,11 @@ void tr_torrentAmountFinished( const tr_torrent * tor, float * tab, int size );
  * tr_torrentRemoveSaved
  ***********************************************************************
  * delete's Transmission's copy of the torrent's metadata from
- * tr_getTorrentsDirectory().
+ * tr_getTorrentDir().
  **********************************************************************/
 void tr_torrentRemoveSaved( tr_torrent * );
 
-void tr_torrentRecheck( tr_torrent * );
+void tr_torrentVerify( tr_torrent * );
 
 /**
  * Frees memory allocated by tr_torrentNew().
@@ -600,19 +669,26 @@ void tr_torrentRecheck( tr_torrent * );
  */
 void tr_torrentClose( tr_torrent * );
 
+/**
+ * Like tr_torrentClose() but also deletes
+ * the fastresume file and our copy of the
+ * torrent file
+ */
+void tr_torrentDelete( tr_torrent * );
+
 /***********************************************************************
  * tr_info
  **********************************************************************/
 
 typedef struct tr_file
 {
-    uint64_t length;                /* Length of the file, in bytes */
-    char     name[MAX_PATH_LENGTH]; /* Path to the file */
-    int8_t   priority;              /* TR_PRI_HIGH, _NORMAL, or _LOW */
-    int8_t   dnd;                   /* nonzero if the file shouldn't be downloaded */
-    int      firstPiece;            /* We need pieces [firstPiece... */
-    int      lastPiece;             /* ...lastPiece] to dl this file */
-    uint64_t offset;                /* file begins at the torrent's nth byte */
+    uint64_t          length;      /* Length of the file, in bytes */
+    char            * name;        /* Path to the file */
+    int8_t            priority;    /* TR_PRI_HIGH, _NORMAL, or _LOW */
+    int8_t            dnd;         /* nonzero if the file shouldn't be downloaded */
+    tr_piece_index_t  firstPiece;  /* We need pieces [firstPiece... */
+    tr_piece_index_t  lastPiece;   /* ...lastPiece] to dl this file */
+    uint64_t          offset;      /* file begins at the torrent's nth byte */
 }
 tr_file;
 
@@ -626,8 +702,7 @@ tr_piece;
     
 typedef struct tr_tracker_info
 {
-    char * address;
-    int    port;
+    int    tier;
     char * announce;
     char * scrape;
 }
@@ -636,39 +711,34 @@ tr_tracker_info;
 struct tr_info
 {
     /* Path to torrent */
-    char                 torrent[MAX_PATH_LENGTH];
+    char               * torrent;
 
     /* General info */
     uint8_t              hash[SHA_DIGEST_LENGTH];
     char                 hashString[2*SHA_DIGEST_LENGTH+1];
-    char                 name[MAX_PATH_LENGTH];
+    char               * name;
 
     /* Flags */
     unsigned int isPrivate : 1;
     unsigned int isMultifile : 1;
 
-    /* Tracker info */
-    struct
-    {
-        tr_tracker_info  * list;
-        int                 count;
-    }                  * trackerList;
-    int                  trackerTiers;
-    char               * primaryAddress;
+    /* these trackers are sorted by tier */
+    tr_tracker_info    * trackers;
+    int                  trackerCount;
 
     /* Torrent info */
-    char                 comment[MAX_PATH_LENGTH];
-    char                 creator[MAX_PATH_LENGTH];
-    int                  dateCreated;
+    char               * comment;
+    char               * creator;
+    time_t               dateCreated;
 
     /* Pieces info */
-    int                  pieceSize;
-    int                  pieceCount;
+    uint32_t             pieceSize;
+    tr_piece_index_t     pieceCount;
     uint64_t             totalSize;
     tr_piece           * pieces;
 
     /* Files info */
-    int                  fileCount;
+    tr_file_index_t      fileCount;
     tr_file            * files;
 };
 
@@ -677,85 +747,174 @@ typedef enum
     TR_STATUS_CHECK_WAIT   = (1<<0), /* Waiting in queue to check files */
     TR_STATUS_CHECK        = (1<<1), /* Checking files */
     TR_STATUS_DOWNLOAD     = (1<<2), /* Downloading */
-    TR_STATUS_DONE         = (1<<3), /* not at 100% so can't tell the tracker
-                                        we're a seeder, but due to DND files
-                                        there's nothing we want right now */
-    TR_STATUS_SEED         = (1<<4), /* Seeding */
-    TR_STATUS_STOPPED      = (1<<5)  /* Torrent is stopped */
+    TR_STATUS_SEED         = (1<<3), /* Seeding */
+    TR_STATUS_STOPPED      = (1<<4)  /* Torrent is stopped */
 }
 tr_torrent_status;
 
 #define TR_STATUS_IS_ACTIVE(s) ((s) != TR_STATUS_STOPPED)
 
-/***********************************************************************
- * tr_stat
- **********************************************************************/
+/**
+ * Transmission error codes
+ * errors are always negative, and 0 refers to no error.
+ */
+typedef enum tr_errno
+{
+    TR_OK = 0,
+
+    /* general errors */
+    TR_ERROR = -100,
+    TR_ERROR_ASSERT,
+
+    /* io errors */
+    TR_ERROR_IO_PARENT = -200,
+    TR_ERROR_IO_PERMISSIONS,
+    TR_ERROR_IO_SPACE,
+    TR_ERROR_IO_FILE_TOO_BIG,
+    TR_ERROR_IO_OPEN_FILES,
+    TR_ERROR_IO_DUP_DOWNLOAD,
+    TR_ERROR_IO_CHECKSUM,
+    TR_ERROR_IO_OTHER,
+
+    /* tracker errors */
+    TR_ERROR_TC_ERROR = -300,
+    TR_ERROR_TC_WARNING,
+
+    /* peer errors */
+    TR_ERROR_PEER_MESSAGE = -400
+}
+tr_errno;
+
+#define TR_ERROR_IS_IO(e) (TR_ERROR_IO_PARENT<=(e) && (e)<=TR_ERROR_IO_OTHER)
+#define TR_ERROR_IS_TC(e) (TR_ERROR_TC_ERROR<=(e) && (e)<=TR_ERROR_TC_WARNING)
+
+struct tr_tracker_stat
+{
+    /* This is the unmodified string returned by the tracker in response
+     * to the torrent's most recent scrape request.  If no request was
+     * sent or there was no response, this string is empty. */
+    char scrapeResponse[256];
+
+    /* The unmodified string returned by the tracker in response
+     * to the torrent's most recent scrape request.  If no request was
+     * sent or there was no response, this string is empty. */
+    char announceResponse[256];
+
+    /* Time the most recent scrape request was sent,
+     * or zero if one hasn't been sent yet. */
+    time_t lastScrapeTime;
+
+    /* Time when the next scrape request will be sent.
+     * This value is always a valid time. */
+    time_t nextScrapeTime;
+
+    /* Time the most recent announce request was sent,
+     * or zero if one hasn't been sent yet. */
+    time_t lastAnnounceTime;
+
+    /* Time when the next reannounce request will be sent,
+     * or zero if the torrent is stopped. */
+    time_t nextAnnounceTime;
+
+    /* When the tracker will allow a human-driven "manual" announce to be sent,
+     * derived from the "min interval" field given by the tracker.
+     * This value is 0 when the torrent is stopped.
+     * This value is ~(time_t)0 if the tracker returned a serious error.
+     * Otherwise, the value is a valid time.
+     * @see tr_manualUpdate( tr_torrent * );
+     * @see tr_torrentCanManualUpdate( const tr_torrent * ); */
+    time_t nextManualAnnounceTime;
+};
+
+tr_torrent_status tr_torrentGetStatus( tr_torrent * );
+
 struct tr_stat
 {
-    tr_torrent_status   status;
-
-    int                 error;
-    char                errorString[128];
+    tr_torrent_status status;
 
+    struct tr_tracker_stat tracker_stat;
     const tr_tracker_info * tracker;
 
-    float               recheckProgress;
-    float               percentComplete;
-    float               percentDone;
-    float               rateDownload;
-    float               rateUpload;
-    int                 eta;
-    int                 peersKnown;
-    int                 peersConnected;
-    int                 peersFrom[TR_PEER_FROM__MAX];
-    int                 peersSendingToUs;
-    int                 peersGettingFromUs;
-    int                 seeders;
-    int                 leechers;
-    int                 completedFromTracker;
+    tr_errno error;
+    char errorString[128];
+
+    /* [0..1] */
+    float recheckProgress;
+
+    /* [0..1] */
+    float percentComplete;
+
+    /* [0..1] */
+    float percentDone;
+
+    /* KiB/s */
+    float rateDownload;
+
+    /* KiB/s */
+    float rateUpload;
+
+ #define TR_ETA_NOT_AVAIL -1
+ #define TR_ETA_UNKNOWN -2
+    /* seconds */
+    int eta;
+
+    int peersKnown;
+    int peersConnected;
+    int peersFrom[TR_PEER_FROM__MAX];
+    int peersSendingToUs;
+    int peersGettingFromUs;
+    int seeders;
+    int leechers;
+    int completedFromTracker;
+
+    /* if the torrent is running, this is the time at which
+     * the client can manually ask the torrent's tracker
+     * for more peers.  otherwise, the value is zero. */
+    time_t manualAnnounceTime;
+
+    /* Byte count of all the piece data we'll have downloaded when we're done.
+     * whether or not we have it yet. [0...tr_info.totalSize] */
+    uint64_t sizeWhenDone;
 
     /* Byte count of how much data is left to be downloaded until
-     * we're done -- that is, until we've got all the pieces we wanted. */
-    uint64_t            leftUntilDone;
+     * we're done -- that is, until we've got all the pieces we wanted.
+     * [0...tr_info.sizeWhenDone] */
+    uint64_t leftUntilDone;
+
+    /* Byte count of all the piece data we want and don't have yet,
+     * but that a connected peer does have. [0...leftUntilDone] */
+    uint64_t desiredAvailable;
 
     /* Byte count of all the corrupt data you've ever downloaded for
      * this torrent.  If you're on a poisoned torrent, this number can
      * grow very large. */
-    uint64_t            corruptEver;
+    uint64_t corruptEver;
 
     /* Byte count of all data you've ever uploaded for this torrent. */
-    uint64_t            uploadedEver;
+    uint64_t uploadedEver;
 
     /* Byte count of all the non-corrupt data you've ever downloaded
      * for this torrent.  If you deleted the files and downloaded a second time,
      * this will be 2*totalSize.. */
-    uint64_t            downloadedEver;
+    uint64_t downloadedEver;
 
     /* Byte count of all the checksum-verified data we have for this torrent. */
-    uint64_t            haveValid;
+    uint64_t haveValid;
 
     /* Byte count of all the partial piece data we have for this torrent.
      * As pieces become complete, this value may decrease as portions of it are
      * moved to `corrupt' or `haveValid'. */
-    uint64_t            haveUnchecked;
+    uint64_t haveUnchecked;
 
-    /* Byte count of all the non-DND piece data that either we already have,
-     * or that a peer we're connected to has. [0...desiredSize] */
-    uint64_t            desiredAvailable;
-
-    /* Byte count of all the piece data we want, whether we currently
-     * have it nor not. [0...tr_info.totalSize] */
-    uint64_t            desiredSize;
-
-    float               swarmspeed;
+    float swarmspeed;
 
 #define TR_RATIO_NA  -1
 #define TR_RATIO_INF -2
     /* TR_RATIO_INF, TR_RATIO_NA, or a regular ratio */
-    float               ratio;
+    float ratio;
     
-    uint64_t            startDate;
-    uint64_t            activityDate;
+    uint64_t startDate;
+    uint64_t activityDate;
 };
 
 struct tr_file_stat
@@ -764,31 +923,22 @@ struct tr_file_stat
     float progress;
 };
 
-typedef enum
-{
-    TR_PEER_STATUS_HANDSHAKE             = (1<<0), /* we're handshaking with peer */
-
-    TR_PEER_STATUS_PEER_IS_SENDING       = (1<<1), /* peer is sending data to us */
-    TR_PEER_STATUS_PEER_IS_INTERESTED    = (1<<2), /* we have data the peer wants */
-    TR_PEER_STATUS_PEER_IS_CHOKED        = (1<<3), /* we refuse to send data to the peer */
-
-    TR_PEER_STATUS_CLIENT_IS_SENDING     = (1<<4), /* we're sending data to the peer */
-    TR_PEER_STATUS_CLIENT_SENT_REQUEST   = (1<<5), /* we've sent the peer a request */
-    TR_PEER_STATUS_CLIENT_IS_INTERESTED  = (1<<6), /* peer has data that we want */
-    TR_PEER_STATUS_CLIENT_IS_CHOKED      = (1<<7), /* peer refuses to send data to us */
-}
-tr_peer_status;
-
 struct tr_peer_stat
 {
-    char addr[INET_ADDRSTRLEN];
+    char addr[16];
     char client[80];
     
-    unsigned int isEncrypted   : 1;
-    unsigned int isDownloading : 1;
-    unsigned int isUploading   : 1;
+    unsigned int isEncrypted : 1;
+    unsigned int isDownloadingFrom : 1;
+    unsigned int isUploadingTo : 1;
 
-    tr_peer_status status;
+    unsigned int peerIsChoked : 1;
+    unsigned int peerIsInterested : 1;
+    unsigned int clientIsChoked : 1;
+    unsigned int clientIsInterested : 1;
+    unsigned int isIncoming : 1;
+
+    char flagStr[32];
 
     uint8_t  from;
     uint16_t port;
@@ -798,34 +948,9 @@ struct tr_peer_stat
     float uploadToRate;
 };
 
-struct tr_msg_list
-{
-    uint8_t              level;
-    time_t               when;
-    char               * message;
-    const char         * file;
-    int                  line;
-    struct tr_msg_list * next;
-};
-
-typedef enum
-{
-    TR_NAT_TRAVERSAL_MAPPING,
-    TR_NAT_TRAVERSAL_MAPPED,
-    TR_NAT_TRAVERSAL_UNMAPPING,
-    TR_NAT_TRAVERSAL_UNMAPPED,
-    TR_NAT_TRAVERSAL_ERROR,
-}
-tr_nat_traversal_status;
-
-struct tr_handle_status
-{
-    tr_nat_traversal_status natTraversalStatus;
-    int publicPort;
-};
 
 #ifdef __TRANSMISSION__
-#  include "internal.h"
+#  include "session.h"
 #endif
 
 #ifdef __cplusplus
