@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: trevent.h 4404 2008-01-01 17:20:20Z charles $
+ * $Id: trevent.h 5754 2008-05-06 15:52:57Z charles $
  */
 
 #ifndef TR_EVENT_H
@@ -22,18 +22,6 @@ void tr_eventInit( struct tr_handle * tr_handle );
 
 void tr_eventClose( struct tr_handle * tr_handle );
 
-/**
-**/
-
-struct event;
-enum evhttp_cmd_type;
-struct evhttp_request;
-struct evhttp_connection;
-struct bufferevent;
-
-/**
-***
-**/
 
 typedef struct tr_timer  tr_timer;
 
