@@ -7,25 +7,23 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: upnp.c 4404 2008-01-01 17:20:20Z charles $
+ * $Id: upnp.c 5700 2008-04-25 19:46:36Z charles $
  */
 
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h> /* snprintf */
-#include <string.h> /* strerror */
 
-#include <miniupnp/miniwget.h>
 #include <miniupnp/miniupnpc.h>
 #include <miniupnp/upnpcommands.h>
 
 #include "transmission.h"
-#include "internal.h"
-#include "shared.h"
+#include "port-forwarding.h"
+#include "session.h"
 #include "utils.h"
 #include "upnp.h"
 
-#define KEY "Port Mapping (UPNP): "
+static const char * getKey( void ) { return _( "Port Forwarding (UPnP)" ); }
 
 typedef enum
 {
@@ -89,18 +87,18 @@ tr_upnpPulse( tr_upnp * handle, int port, int isEnabled )
         errno = 0;
         devlist = upnpDiscover( 2000, NULL, NULL );
         if( devlist == NULL ) {
-            tr_err( KEY "upnpDiscover returned NULL (errno %d - %s)", errno, strerror(errno) );
+            tr_ndbg( getKey(), "upnpDiscover failed (errno %d - %s)", errno, tr_strerror(errno) );
         }
         errno = 0;
         if( UPNP_GetValidIGD( devlist, &handle->urls, &handle->data, handle->lanaddr, sizeof(handle->lanaddr))) {
-            tr_inf( KEY "Found Internet Gateway Device '%s'", handle->urls.controlURL );
-            tr_inf( KEY "Local LAN IP Address is '%s'", handle->lanaddr );
+            tr_ninf( getKey(), _( "Found Internet Gateway Device \"%s\"" ), handle->urls.controlURL );
+            tr_ninf( getKey(), _( "Local Address is \"%s\"" ), handle->lanaddr );
             handle->state = TR_UPNP_IDLE;
             handle->hasDiscovered = 1;
         } else {
             handle->state = TR_UPNP_ERR;
-            tr_err( KEY "UPNP_GetValidIGD failed.  (errno %d - %s)", errno, strerror(errno) );
-            tr_err( KEY "If your router supports UPnP, please make sure UPnP is enabled!" );
+            tr_ndbg( getKey(), "UPNP_GetValidIGD failed (errno %d - %s)", errno, tr_strerror(errno) );
+            tr_ndbg( getKey(), "If your router supports UPnP, please make sure UPnP is enabled!" );
         }
         freeUPNPDevlist( devlist );
     }
@@ -118,7 +116,7 @@ tr_upnpPulse( tr_upnp * handle, int port, int isEnabled )
         UPNP_DeletePortMapping( handle->urls.controlURL,
                                 handle->data.servicetype,
                                 portStr, "TCP" );
-        tr_dbg( KEY "Stopping port forwarding of '%s', service '%s'",
+        tr_ninf( getKey(), _( "Stopping port forwarding through \"%s\", service \"%s\"" ),
                 handle->urls.controlURL, handle->data.servicetype );
         handle->isMapped = 0;
         handle->state = TR_UPNP_IDLE;
@@ -147,15 +145,15 @@ tr_upnpPulse( tr_upnp * handle, int port, int isEnabled )
                                        "Transmission", "TCP" );
             handle->isMapped = !err;
         }
-        tr_inf( KEY "Port forwarding via '%s', service '%s'.  (local address: %s:%d)",
-                handle->urls.controlURL, handle->data.servicetype, handle->lanaddr, port );
+        tr_ninf( getKey(), _( "Port forwarding through \"%s\", service \"%s\".  (local address: %s:%d)" ),
+                 handle->urls.controlURL, handle->data.servicetype, handle->lanaddr, port );
         if( handle->isMapped ) {
-            tr_inf( KEY "Port forwarding successful!" );
+            tr_ninf( getKey(), _( "Port forwarding successful!" ) );
             handle->port = port;
             handle->state = TR_UPNP_IDLE;
         } else {
-            tr_err( KEY "Port forwarding failed with err %d (%d - %s)", err, errno, strerror(errno) );
-            tr_err( KEY "If your router supports UPnP, please make sure UPnP is enabled!" );
+            tr_ndbg( getKey(), "Port forwarding failed with error %d (errno %d - %s)", err, errno, tr_strerror(errno) );
+            tr_ndbg( getKey(), "If your router supports UPnP, please make sure UPnP is enabled!" );
             handle->port = -1;
             handle->state = TR_UPNP_ERR;
         }
