@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: utils.c 3473 2007-10-20 15:17:36Z charles $
+ * $Id: utils.c 5890 2008-05-21 20:56:10Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,23 +23,25 @@
  *****************************************************************************/
 
 #include <assert.h>
-#include <ctype.h>
+#include <ctype.h> /* isalpha */
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <string.h> /* strerror */
 
+#include <libgen.h> /* basename */
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h> /* usleep, stat */
 
+#include "event.h"
+
 #ifdef WIN32
     #include <windows.h> /* for Sleep */
 #elif defined(__BEOS__)
     #include <kernel/OS.h>
-    extern int vasprintf( char **, const char *, va_list );
 #endif
 
 #include "transmission.h"
@@ -47,8 +49,6 @@
 #include "utils.h"
 #include "platform.h"
 
-#define SPRINTF_BUFSIZE         100
-
 static tr_lock      * messageLock = NULL;
 static int            messageLevel = 0;
 static int            messageQueuing = FALSE;
@@ -70,7 +70,7 @@ tr_getLog( void )
     if( !initialized )
     {
         const char * str = getenv( "TR_DEBUG_FD" );
-        int fd;
+        int fd = 0;
         if( str && *str )
             fd = atoi( str );
         switch( fd ) {
@@ -84,26 +84,6 @@ tr_getLog( void )
     return file;
 }
 
-char*
-tr_getLogTimeStr( char * buf, int buflen )
-{
-    char tmp[64];
-    time_t now;
-    struct tm now_tm;
-    struct timeval tv;
-    int milliseconds;
-
-    now = time( NULL );
-    gettimeofday( &tv, NULL );
-
-    localtime_r( &now, &now_tm );
-    strftime( tmp, sizeof(tmp), "%H:%M:%S", &now_tm );
-    milliseconds = (int)(tv.tv_usec / 1000);
-    snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
-
-    return buf;
-}
-
 void
 tr_setMessageLevel( int level )
 {
@@ -113,7 +93,8 @@ tr_setMessageLevel( int level )
     tr_lockUnlock( messageLock );
 }
 
-int tr_getMessageLevel( void )
+int
+tr_getMessageLevel( void )
 {
     int ret;
 
@@ -125,7 +106,8 @@ int tr_getMessageLevel( void )
     return ret;
 }
 
-void tr_setMessageQueuing( int enabled )
+void
+tr_setMessageQueuing( int enabled )
 {
     tr_msgInit();
     tr_lockLock( messageLock );
@@ -133,7 +115,8 @@ void tr_setMessageQueuing( int enabled )
     tr_lockUnlock( messageLock );
 }
 
-tr_msg_list * tr_getQueuedMessages( void )
+tr_msg_list *
+tr_getQueuedMessages( void )
 {
     tr_msg_list * ret;
 
@@ -147,7 +130,8 @@ tr_msg_list * tr_getQueuedMessages( void )
     return ret;
 }
 
-void tr_freeMessageList( tr_msg_list * list )
+void
+tr_freeMessageList( tr_msg_list * list )
 {
     tr_msg_list * next;
 
@@ -155,71 +139,128 @@ void tr_freeMessageList( tr_msg_list * list )
     {
         next = list->next;
         free( list->message );
+        free( list->name );
         free( list );
         list = next;
     }
 }
 
-void tr_msg( int level, const char * msg, ... )
-{
-    va_list       args1, args2;
-    tr_msg_list * newmsg;
-    int           len1, len2;
-    FILE        * fp;
+/**
+***
+**/
 
-    assert( NULL != messageLock );
-    tr_lockLock( messageLock );
+char*
+tr_getLogTimeStr( char * buf, int buflen )
+{
+    char tmp[64];
+    time_t now;
+    struct tm now_tm;
+    struct timeval tv;
+    int milliseconds;
+
+    now = time( NULL );
+    gettimeofday( &tv, NULL );
+
+#ifdef WIN32
+    now_tm = *localtime( &now );
+#else
+    localtime_r( &now, &now_tm );
+#endif
+    strftime( tmp, sizeof(tmp), "%H:%M:%S", &now_tm );
+    milliseconds = (int)(tv.tv_usec / 1000);
+    snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
+
+    return buf;
+}
+
+void
+tr_deepLog( const char * file, int line, const char * name, const char * fmt, ... )
+{
+    FILE * fp = tr_getLog( );
+    if( fp != NULL )
+    {
+        va_list args;
+        char timestr[64];
+        struct evbuffer * buf = evbuffer_new( );
+        char * myfile = tr_strdup( file );
+
+        evbuffer_add_printf( buf, "[%s] ", tr_getLogTimeStr( timestr, sizeof(timestr) ) );
+        if( name )
+            evbuffer_add_printf( buf, "%s ", name );
+        va_start( args, fmt );
+        evbuffer_add_vprintf( buf, fmt, args );
+        va_end( args );
+        evbuffer_add_printf( buf, " (%s:%d)\n", basename(myfile), line );
+        fwrite( EVBUFFER_DATA(buf), 1, EVBUFFER_LENGTH(buf), fp );
+
+        tr_free( myfile );
+        evbuffer_free( buf );
+    }
+}
+
+/***
+****
+***/
+
+void
+tr_msg( const char * file, int line, int level,
+        const char * name,
+        const char * fmt, ... )
+{
+    FILE * fp;
+
+    if( messageLock )
+        tr_lockLock( messageLock );
 
     fp = tr_getLog( );
 
     if( !messageLevel )
     {
-        char * env;
-        env          = getenv( "TR_DEBUG" );
+        char * env = getenv( "TR_DEBUG" );
         messageLevel = ( env ? atoi( env ) : 0 ) + 1;
         messageLevel = MAX( 1, messageLevel );
     }
 
     if( messageLevel >= level )
     {
-        va_start( args1, msg );
-        if( messageQueuing )
+        va_list ap;
+        struct evbuffer * buf = evbuffer_new( );
+
+        /* build the text message */
+        va_start( ap, fmt );
+        evbuffer_add_vprintf( buf, fmt, ap );
+        va_end( ap );
+
+        if( EVBUFFER_LENGTH( buf ) )
         {
-            newmsg = calloc( 1, sizeof( *newmsg ) );
-            if( NULL != newmsg )
+            if( messageQueuing )
             {
+                tr_msg_list * newmsg;
+                newmsg = tr_new0( tr_msg_list, 1 );
                 newmsg->level = level;
                 newmsg->when = time( NULL );
-                len1 = len2 = 0;
-                va_start( args2, msg );
-                tr_vsprintf( &newmsg->message, &len1, &len2, msg,
-                             args1, args2 );
-                va_end( args2 );
-                if( fp != NULL )
-                    fprintf( fp, "%s\n", newmsg->message );
-                if( NULL == newmsg->message )
-                {
-                    free( newmsg );
-                }
-                else
-                {
-                    *messageQueueTail = newmsg;
-                    messageQueueTail = &newmsg->next;
-                }
+                newmsg->message = tr_strdup( (char*)EVBUFFER_DATA( buf ) );
+                newmsg->file = file;
+                newmsg->line = line;
+                newmsg->name = tr_strdup( name );
+
+                *messageQueueTail = newmsg;
+                messageQueueTail = &newmsg->next;
             }
+            else
+            {
+                if( fp == NULL )
+                    fp = stderr;
+                fprintf( fp, "%s\n", (char*)EVBUFFER_DATA(buf) );
+                fflush( fp );
+            }
+
+            evbuffer_free( buf );
         }
-        else
-        {
-            if( fp == NULL )
-                fp = stderr;
-            vfprintf( fp, msg, args1 );
-            fputc( '\n', fp );
-            fflush( fp );
-        }
-        va_end( args1 );
     }
 
-    tr_lockUnlock( messageLock );
+    if( messageLock )
+        tr_lockUnlock( messageLock );
 }
 
 int tr_rand( int sup )
@@ -295,40 +336,6 @@ tr_set_compare( const void * va, size_t aCount,
 ****
 ***/
 
-void * tr_memmem ( const void *vbig, size_t big_len,
-                   const void *vlittle, size_t little_len )
-{
-    const char *big = vbig;
-    const char *little = vlittle;
-    size_t ii, jj;
-
-    if( 0 == big_len || 0 == little_len )
-    {
-        return NULL;
-    }
-
-    for( ii = 0; ii + little_len <= big_len; ii++ )
-    {
-        for( jj = 0; jj < little_len; jj++ )
-        {
-            if( big[ii + jj] != little[jj] )
-            {
-                break;
-            }
-        }
-        if( jj == little_len )
-        {
-            return (char*)big + ii;
-        }
-    }
-
-    return NULL;
-}
-
-/**
-***
-**/
-
 int
 tr_compareUint16( uint16_t a, uint16_t b )
 {
@@ -350,7 +357,7 @@ tr_compareUint32( uint32_t a, uint32_t b )
 **/
 
 struct timeval
-timevalMsec( uint64_t milliseconds )
+tr_timevalMsec( uint64_t milliseconds )
 {
     struct timeval ret;
     const uint64_t microseconds = milliseconds * 1000;
@@ -365,18 +372,19 @@ tr_loadFile( const char * path, size_t * size )
     uint8_t    * buf;
     struct stat  sb;
     FILE       * file;
+    const char * err_fmt = _( "Couldn't read \"%1$s\": %2$s" );
 
     /* try to stat the file */
     errno = 0;
     if( stat( path, &sb ) )
     {
-        tr_err( "Couldn't get information for file \"%s\" %s", path, strerror(errno) );
+        tr_dbg( err_fmt, path, tr_strerror(errno) );
         return NULL;
     }
 
     if( ( sb.st_mode & S_IFMT ) != S_IFREG )
     {
-        tr_err( "Not a regular file (%s)", path );
+        tr_err( err_fmt, path, _( "Not a regular file" ) );
         return NULL;
     }
 
@@ -384,20 +392,20 @@ tr_loadFile( const char * path, size_t * size )
     file = fopen( path, "rb" );
     if( !file )
     {
-        tr_err( "Couldn't open file \"%s\" %s", path, strerror(errno) );
+        tr_err( err_fmt, path, tr_strerror(errno) );
         return NULL;
     }
     buf = malloc( sb.st_size );
     if( NULL == buf )
     {
-        tr_err( "Couldn't allocate memory (%"PRIu64" bytes)",
-                ( uint64_t )sb.st_size );
+        tr_err( err_fmt, path, _( "Memory allocation failed" ) );
         fclose( file );
+        return NULL;
     }
     fseek( file, 0, SEEK_SET );
     if( fread( buf, sb.st_size, 1, file ) != 1 )
     {
-        tr_err( "Error reading \"%s\" %s", path, strerror(errno) );
+        tr_err( err_fmt, path, tr_strerror(errno) );
         free( buf );
         fclose( file );
         return NULL;
@@ -417,6 +425,8 @@ tr_mkdir( const char * path, int permissions
                                                     )
 {
 #ifdef WIN32
+    if( path && isalpha(path[0]) && path[1]==':' && !path[2] )
+        return 0;
     return mkdir( path );
 #else
     return mkdir( path, permissions );
@@ -424,202 +434,88 @@ tr_mkdir( const char * path, int permissions
 }
 
 int
-tr_mkdirp( char * path, int permissions )
+tr_mkdirp( const char * path_in, int permissions )
 {
-    char      * p, * pp;
+    char * path = tr_strdup( path_in );
+    char * p, * pp;
     struct stat sb;
     int done;
 
+    /* walk past the root */
     p = path;
-    while( '/' == *p )
-      p++;
+    while( *p == TR_PATH_DELIMITER )
+        ++p;
+
     pp = p;
     done = 0;
-    while( ( p = strchr( pp, '/' ) ) || ( p = strchr( pp, '\0' ) ) )
+    while( ( p = strchr( pp, TR_PATH_DELIMITER ) ) || ( p = strchr( pp, '\0' ) ) )
     {
-        if( '\0' == *p)
-        {
+        if( !*p )
             done = 1;
-        }
         else
-        {
             *p = '\0';
-        }
+
         if( stat( path, &sb ) )
         {
             /* Folder doesn't exist yet */
-            if( tr_mkdir( path, permissions ) )
-            {
-                tr_err( "Could not create directory %s (%s)", path,
-                        strerror( errno ) );
-                *p = '/';
-                return 1;
+            if( tr_mkdir( path, permissions ) ) {
+                const int err = errno;
+                tr_err( _( "Couldn't create \"%1$s\": %2$s" ), path, tr_strerror( err ) );
+                tr_free( path );
+                errno = err;
+                return -1;
             }
         }
         else if( ( sb.st_mode & S_IFMT ) != S_IFDIR )
         {
             /* Node exists but isn't a folder */
-            tr_err( "Remove %s, it's in the way.", path );
-            *p = '/';
-            return 1;
+            char buf[MAX_PATH_LENGTH];
+            snprintf( buf, sizeof( buf ), _( "File \"%s\" is in the way" ), path );
+            tr_err( _( "Couldn't create \"%1$s\": %2$s" ), path_in, buf );
+            tr_free( path );
+            errno = ENOTDIR;
+            return -1;
         }
+
         if( done )
-        {
             break;
-        }
-        *p = '/';
+
+        *p = TR_PATH_DELIMITER;
         p++;
         pp = p;
     }
 
-    return 0;
-}
-
-int
-tr_strncasecmp( const char * s1, const char * s2, size_t n )
-{
-    if ( !n )
-        return 0;
-
-    while( n-- != 0 && tolower( *s1 ) == tolower( *s2 ) ) {
-        if( !n || !*s1 || !*s2 )
-	    break;
-        ++s1;
-        ++s2;
-    }
-
-    return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
-}
-
-int tr_sprintf( char ** buf, int * used, int * max, const char * format, ... )
-{
-    va_list ap1, ap2;
-    int     ret;
-
-    va_start( ap1, format );
-    va_start( ap2, format );
-    ret = tr_vsprintf( buf, used, max, format, ap1, ap2 );
-    va_end( ap2 );
-    va_end( ap1 );
-
-    return ret;
-}
-
-int tr_vsprintf( char ** buf, int * used, int * max, const char * fmt,
-                 va_list ap1, va_list ap2 )
-{
-    int     want;
-
-    want = vsnprintf( NULL, 0, fmt, ap1 );
-
-    if( tr_concat( buf, used, max, NULL, want ) )
-    {
-        return 1;
-    }
-    assert( *used + want + 1 <= *max );
-
-    *used += vsnprintf( *buf + *used, *max - *used, fmt, ap2 );
-
-    return 0;
-}
-
-#ifndef HAVE_ASPRINTF
-
-int
-asprintf( char ** buf, const char * format, ... )
-{
-    va_list ap;
-    int     ret;
-
-    va_start( ap, format );
-    ret = vasprintf( buf, format, ap );
-    va_end( ap );
-
-    return ret;
-}
-
-int
-vasprintf( char ** buf, const char * format, va_list ap )
-{
-    va_list ap2;
-    int     used, max;
-
-    va_copy( ap2, ap );
-
-    *buf = NULL;
-    used = 0;
-    max  = 0;
-
-    if( tr_vsprintf( buf, &used, &max, format, ap, ap2 ) )
-    {
-        free( *buf );
-        return -1;
-    }
-
-    return used;
-}
-
-#endif /* HAVE_ASPRINTF */
-
-int tr_concat( char ** buf, int * used, int * max, const char * data, int len )
-{
-    int     newmax;
-    char  * newbuf;
-
-    newmax = *max;
-    while( *used + len + 1 > newmax )
-    {
-        newmax += SPRINTF_BUFSIZE;
-    }
-    if( newmax > *max )
-    {
-        newbuf = realloc( *buf, newmax );
-        if( NULL == newbuf )
-        {
-            return 1;
-        }
-        *buf = newbuf;
-        *max = newmax;
-    }
-
-    if( NULL != data )
-    {
-        memcpy( *buf + *used, data, len );
-        *used += len;
-    }
-
+    tr_free( path );
     return 0;
 }
 
 void
 tr_buildPath ( char *buf, size_t buflen, const char *first_element, ... )
 {
-    va_list vl;
-    char* walk = buf;
+    struct evbuffer * evbuf = evbuffer_new( );
     const char * element = first_element;
-
-    if( first_element == NULL )
-        return;
-
+    va_list vl;
     va_start( vl, first_element );
-    for( ;; ) {
-        const size_t n = strlen( element );
-        memcpy( walk, element, n );
-        walk += n;
+    while( element ) {
+        if( EVBUFFER_LENGTH(evbuf) )
+            evbuffer_add_printf( evbuf, "%c", TR_PATH_DELIMITER );
+        evbuffer_add_printf( evbuf, "%s", element );
         element = (const char*) va_arg( vl, const char* );
-        if( element == NULL )
-            break;
-        *walk++ = TR_PATH_DELIMITER;
     }
-    *walk = '\0';
-    assert( walk-buf <= (int)buflen );
+    if( EVBUFFER_LENGTH(evbuf) )
+        strlcpy( buf, (char*)EVBUFFER_DATA(evbuf), buflen );
+    else
+        *buf = '\0';
+    evbuffer_free( evbuf );
 }
 
 int
-tr_ioErrorFromErrno( void )
+tr_ioErrorFromErrno( int err )
 {
-    switch( errno )
+    switch( err )
     {
+        case 0:
+            return TR_OK;
         case EACCES:
         case EROFS:
             return TR_ERROR_IO_PERMISSIONS;
@@ -630,38 +526,52 @@ tr_ioErrorFromErrno( void )
         case EFBIG:
             return TR_ERROR_IO_FILE_TOO_BIG;
         default:
-            tr_dbg( "generic i/o errno from errno: %s", strerror( errno ) );
+            tr_dbg( "generic i/o errno from errno: %s", tr_strerror( errno ) );
             return TR_ERROR_IO_OTHER;
     }
 }
 
-char *
+const char *
 tr_errorString( int code )
 {
     switch( code )
     {
         case TR_OK:
-            return "No error";
+            return _( "No error" );
+
         case TR_ERROR:
-            return "Generic error";
+            return _( "Unspecified error" );
         case TR_ERROR_ASSERT:
-            return "Assert error";
+            return _( "Assert error" );
+
         case TR_ERROR_IO_PARENT:
-            return "Download folder does not exist";
+            return _( "Destination folder doesn't exist" );
         case TR_ERROR_IO_PERMISSIONS:
-            return "Insufficient permissions";
+            return tr_strerror( EACCES );
         case TR_ERROR_IO_SPACE:
-            return "Insufficient free space";
-        case TR_ERROR_IO_DUP_DOWNLOAD:
-            return "Already active transfer with same name and download folder";
+            return tr_strerror( ENOSPC );
         case TR_ERROR_IO_FILE_TOO_BIG:
-            return "File too large";
+            return tr_strerror( EFBIG );
         case TR_ERROR_IO_OPEN_FILES:
-            return "Too many open files";
+            return tr_strerror( EMFILE );
+        case TR_ERROR_IO_DUP_DOWNLOAD:
+            return _( "A torrent with this name and destination folder already exists." );
+        case TR_ERROR_IO_CHECKSUM:
+            return _( "Checksum failed" );
         case TR_ERROR_IO_OTHER:
-            return "Generic I/O error";
+            return _( "Unspecified I/O error" );
+
+        case TR_ERROR_TC_ERROR:
+            return _( "Tracker error" );
+        case TR_ERROR_TC_WARNING:
+            return _( "Tracker warning" );
+
+        case TR_ERROR_PEER_MESSAGE:
+            return _( "Peer sent a bad message" );
+
+        default:
+            return _( "Unknown error" );
     }
-    return "Unknown error";
 }
 
 /****
@@ -679,7 +589,11 @@ tr_strndup( const char * in, int len )
 {
     char * out = NULL;
 
-    if( in != NULL )
+    if( len < 0 )
+    {
+        out = tr_strdup( in );
+    }
+    else if( in != NULL )
     {
         out = tr_malloc( len+1 );
         memcpy( out, in, len );
@@ -715,6 +629,15 @@ tr_free( void * p )
         free( p );
 }
 
+const char*
+tr_strerror( int i )
+{
+    const char * ret = strerror( i );
+    if( ret == NULL )
+        ret = "Unknown Error";
+    return ret;
+}
+
 /****
 *****
 ****/
@@ -765,7 +688,7 @@ tr_bitfieldClear( tr_bitfield * bitfield )
 int
 tr_bitfieldIsEmpty( const tr_bitfield * bitfield )
 {
-    unsigned int i;
+    size_t i;
 
     for( i=0; i<bitfield->len; ++i )
         if( bitfield->bits[i] )
@@ -778,62 +701,102 @@ int
 tr_bitfieldHas( const tr_bitfield * bitfield, size_t nth )
 {
     static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    return bitfield!=NULL && (bitfield->bits[nth>>3u] & ands[nth&7u] );
+    const size_t i = nth >> 3u;
+    return ( bitfield != NULL )
+        && ( bitfield->bits != NULL )
+        && ( i < bitfield->len )
+        && ( ( bitfield->bits[i] & ands[nth&7u] ) != 0 );
 }
 
-void
+int
 tr_bitfieldAdd( tr_bitfield  * bitfield, size_t nth )
 {
     static const uint8_t ands[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-    bitfield->bits[nth>>3u] |= ands[nth&7u];
+    const size_t i = nth >> 3u;
+
+    assert( bitfield != NULL );
+    assert( bitfield->bits != NULL );
+
+    if( i >= bitfield->len )
+        return -1;
+
+    bitfield->bits[i] |= ands[nth&7u];
+    assert( tr_bitfieldHas( bitfield, nth ) );
+    return 0;
 }
 
-void
+int
 tr_bitfieldAddRange( tr_bitfield  * bitfield,
                      size_t         begin,
                      size_t         end )
 {
-    /* TODO: there are faster ways to do this */
-    unsigned int i;
+    int err = 0;
+    size_t i;
     for( i=begin; i<end; ++i )
-        tr_bitfieldAdd( bitfield, i );
+        if(( err = tr_bitfieldAdd( bitfield, i )))
+            break;
+    return err;
 }
 
-void
+int
 tr_bitfieldRem( tr_bitfield   * bitfield,
                 size_t          nth )
 {
     static const uint8_t rems[8] = { 127, 191, 223, 239, 247, 251, 253, 254 };
+    const size_t i = nth >> 3u;
 
-    if( bitfield != NULL )
-        bitfield->bits[nth>>3u] &= rems[nth&7u];
+    assert( bitfield != NULL );
+    assert( bitfield->bits != NULL );
+
+    if( i >= bitfield->len )
+        return -1;
+
+    bitfield->bits[i] &= rems[nth&7u];
+    assert( !tr_bitfieldHas( bitfield, nth ) );
+    return 0;
 }
 
-void
+int
 tr_bitfieldRemRange ( tr_bitfield  * b,
                       size_t         begin,
                       size_t         end )
 {
-    /* TODO: there are faster ways to do this */
-    unsigned int i;
+    int err = 0;
+    size_t i;
     for( i=begin; i<end; ++i )
-        tr_bitfieldRem( b, i );
+        if(( err = tr_bitfieldRem( b, i )))
+            break;
+    return err;
 }
 
 tr_bitfield*
-tr_bitfieldAnd( tr_bitfield * a, const tr_bitfield * b )
+tr_bitfieldOr( tr_bitfield * a, const tr_bitfield * b )
 {
     uint8_t *ait;
     const uint8_t *aend, *bit;
 
     assert( a->len == b->len );
 
-    for( ait=a->bits, bit=b->bits, aend=ait+a->len; ait!=aend; ++ait, ++bit )
-        *ait &= *bit;
+    for( ait=a->bits, bit=b->bits, aend=ait+a->len; ait!=aend; )
+        *ait++ |= *bit++;
 
     return a;
 }
 
+/* set 'a' to all the flags that were in 'a' but not 'b' */
+void
+tr_bitfieldDifference( tr_bitfield * a, const tr_bitfield * b )
+{
+    uint8_t *ait;
+    const uint8_t *aend, *bit;
+
+    assert( a->len == b->len );
+
+    for( ait=a->bits, bit=b->bits, aend=ait+a->len; ait!=aend; )
+        *ait++ &= ~(*bit++);
+}
+
+
 size_t
 tr_bitfieldCountTrueBits( const tr_bitfield* b )
 {
@@ -890,3 +853,161 @@ tr_wait( uint64_t delay_milliseconds )
     usleep( 1000 * delay_milliseconds );
 #endif
 }
+
+/***
+****
+***/
+
+
+#ifndef HAVE_STRLCPY
+
+/*
+ * Copy src to string dst of size siz.  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz == 0).
+ * Returns strlen(src); if retval >= siz, truncation occurred.
+ */
+size_t
+strlcpy(char *dst, const char *src, size_t siz)
+{
+	char *d = dst;
+	const char *s = src;
+	size_t n = siz;
+
+	assert( s != NULL );
+	assert( d != NULL );
+
+	/* Copy as many bytes as will fit */
+	if (n != 0) {
+		while (--n != 0) {
+			if ((*d++ = *s++) == '\0')
+				break;
+		}
+	}
+
+	/* Not enough room in dst, add NUL and traverse rest of src */
+	if (n == 0) {
+		if (siz != 0)
+			*d = '\0';		/* NUL-terminate dst */
+		while (*s++)
+			;
+	}
+
+	return(s - src - 1);	/* count does not include NUL */
+}
+
+#endif /* HAVE_STRLCPY */
+
+/***
+****
+***/
+
+double
+tr_getRatio( double numerator, double denominator )
+{
+    double ratio;
+
+    if( denominator )
+        ratio = numerator / denominator;
+    else if( numerator )
+        ratio = TR_RATIO_INF;
+    else
+        ratio = TR_RATIO_NA;
+
+    return ratio;
+}
+
+void
+tr_sha1_to_hex( char * out, const uint8_t * sha1 )
+{
+    static const char hex[] = "0123456789abcdef";
+    int i;
+    for (i = 0; i < 20; i++) {
+        unsigned int val = *sha1++;
+        *out++ = hex[val >> 4];
+        *out++ = hex[val & 0xf];
+    }
+    *out = '\0';
+}
+
+/***
+****
+***/
+
+int
+tr_httpIsValidURL( const char * url )
+{
+    const char * c;
+    static const char * rfc2396_valid_chars =
+        "abcdefghijklmnopqrstuvwxyz" /* lowalpha */
+        "ABCDEFGHIJKLMNOPQRSTUVWXYZ" /* upalpha */
+        "0123456789"                 /* digit */
+        "-_.!~*'()"                  /* mark */
+        ";/?:@&=+$,"                 /* reserved */
+        "<>#%<\""                    /* delims */
+        "{}|\\^[]`";                 /* unwise */
+
+    for( c=url; c && *c; ++c )
+        if( !strchr( rfc2396_valid_chars, *c ) )
+            return FALSE;
+
+    return !tr_httpParseURL( url, -1, NULL, NULL, NULL );
+}
+
+int
+tr_httpParseURL( const char * url_in, int len,
+                 char ** setme_host,
+                 int * setme_port,
+                 char ** setme_path )
+{
+    int err;
+    int port = 0;
+    int n;
+    char * tmp;
+    char * pch;
+    const char * protocol = NULL;
+    const char * host = NULL;
+    const char * path = NULL;
+
+    tmp = tr_strndup( url_in, len );
+    if(( pch = strstr( tmp, "://" )))
+    {
+       *pch = '\0';
+       protocol = tmp;
+       pch += 3;
+/*fprintf( stderr, "protocol is [%s]... what's left is [%s]\n", protocol, pch );*/
+       if(( n = strcspn( pch, ":/" )))
+       {
+           const int havePort = pch[n] == ':';
+           host = pch;
+           pch += n;
+           *pch++ = '\0';
+/*fprintf( stderr, "host is [%s]... what's left is [%s]\n", host, pch );*/
+           if( havePort )
+           {
+               char * end;
+               port = strtol( pch, &end, 10 );
+               pch = end;
+/*fprintf( stderr, "port is [%d]... what's left is [%s]\n", port, pch );*/
+           }
+           path = pch;
+/*fprintf( stderr, "path is [%s]\n", path );*/
+       }
+    }
+
+    err = !host || !path || !protocol || ( strcmp(protocol,"http") && strcmp(protocol,"https") );
+
+    if( !err && !port ) {
+        if( !strcmp(protocol,"http") ) port = 80;
+        if( !strcmp(protocol,"https") ) port = 443;
+    }
+
+    if( !err ) {
+        if( setme_host) { ((char*)host)[-3]=':'; *setme_host = tr_strdup( protocol ); }
+        if( setme_path) { ((char*)path)[-1]='/'; *setme_path = tr_strdup( path-1 ); }
+        if( setme_port) *setme_port = port;
+    }
+
+
+    tr_free( tmp );
+    return err;
+}
