@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: utils.h 3473 2007-10-20 15:17:36Z charles $
+ * $Id: utils.h 5759 2008-05-06 19:06:46Z charles $
  *
- * Copyright (c) 2005-2007 Transmission authors and contributors
+ * Copyright (c) 2005-2008 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,63 +32,70 @@
 
 void tr_msgInit( void );
 
-#define tr_err( a... ) tr_msg( TR_MSG_ERR, ## a )
-#define tr_inf( a... ) tr_msg( TR_MSG_INF, ## a )
-#define tr_dbg( a... ) tr_msg( TR_MSG_DBG, ## a )
-void tr_msg  ( int level, const char * msg, ... );
+#if !defined(_)
+#if defined(SYS_DARWIN)
+  #define _(a) (a)
+#elif HAVE_LIBINTL
+  #include <libintl.h>
+  #define _(a) gettext (a)
+#else
+  #define _(a) (a)
+#endif
+#endif
+
+#define tr_nerr( n, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_ERR, n, ## a )
+#define tr_ninf( n, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_INF, n, ## a )
+#define tr_ndbg( n, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_DBG, n, ## a )
+
+#define tr_torerr( tor, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_ERR, tor->info.name, ## a )
+#define tr_torinf( tor, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_INF, tor->info.name, ## a )
+#define tr_tordbg( tor, a... ) tr_msg( __FILE__, __LINE__, TR_MSG_DBG, tor->info.name, ## a )
+
+#define tr_err( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_ERR, NULL, ## a )
+#define tr_inf( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_INF, NULL, ## a )
+#define tr_dbg( a... ) tr_msg( __FILE__, __LINE__, TR_MSG_DBG, NULL, ## a )
+void tr_msg  ( const char * file, int line, int level, const char * torrent, const char * fmt, ... );
 FILE* tr_getLog( void );
 
+void tr_deepLog( const char * file, int line, const char * name, const char * fmt, ... ); 
+
 char* tr_getLogTimeStr( char * buf, int buflen );
 
-int  tr_rand ( int );
-
-void * tr_memmem( const void *, size_t, const void *, size_t );
-
-/***********************************************************************
- * tr_mkdirp
- ***********************************************************************
- * Create a directory and any needed parent directories.
- * Note that the string passed in must be writable!
- **********************************************************************/
-int tr_mkdirp( char * path, int permissions );
+/** Returns a random number in the range of [0...n) */
+int tr_rand ( int n );
 
+/**
+ * a portability wrapper around mkdir().
+ * On WIN32, the `permissions' argument is unused.
+ *
+ * @return zero on success, or -1 if an error occurred
+ * (in which case errno is set appropriately).
+ */
 int tr_mkdir( const char * path, int permissions );
 
+/**
+ * Like mkdir, but makes parent directories as needed.
+ *
+ * @return zero on success, or -1 if an error occurred
+ * (in which case errno is set appropriately).
+ */
+int tr_mkdirp( const char * path, int permissions );
+
+
 uint8_t* tr_loadFile( const char * filename, size_t * size );
 
-/***********************************************************************
- * tr_strcasecmp
- ***********************************************************************
- * A case-insensitive strncmp()
- **********************************************************************/
-#define tr_strcasecmp( ff, ss ) ( tr_strncasecmp( (ff), (ss), ULONG_MAX ) )
-int tr_strncasecmp( const char * first, const char * second, size_t len );
-
-/***********************************************************************
- * tr_sprintf
- ***********************************************************************
- * Appends to the end of a buffer using printf formatting,
- * growing the buffer if needed
- **********************************************************************/
-int tr_sprintf( char ** buf, int * used, int * max,
-                const char * format, ... );
-/* gee, it sure would be nice if BeOS had va_copy() */
-int tr_vsprintf( char **, int *, int *, const char *, va_list, va_list );
-/* this concatenates some binary data onto the end of a buffer */
-int tr_concat( char ** buf, int * used, int * max,
-               const char * data, int len );
 
 /* creates a filename from a series of elements using the
    correct separator for filenames. */
 void tr_buildPath( char* buf, size_t buflen,
                    const char * first_element, ... );
 
-struct timeval timevalMsec( uint64_t milliseconds );
+struct timeval tr_timevalMsec( uint64_t milliseconds );
 
 
-int    tr_ioErrorFromErrno( void );
+int tr_ioErrorFromErrno( int err );
 
-char * tr_errorString( int code );
+const char * tr_errorString( int code );
 
 /* return the current date in milliseconds */
 uint64_t tr_date( void );
@@ -128,6 +135,8 @@ void  tr_free    ( void* );
 char* tr_strdup( const char * str );
 char* tr_strndup( const char * str, int len );
 
+const char* tr_strerror( int );
+
 /***
 ****
 ***/
@@ -146,6 +155,18 @@ void tr_set_compare( const void * a, size_t aCount,
 int tr_compareUint16( uint16_t a, uint16_t b );
 int tr_compareUint32( uint32_t a, uint32_t b );
 
+void tr_sha1_to_hex( char * out, const uint8_t * sha1 );
+
+
+int tr_httpIsValidURL( const char * url );
+
+int tr_httpParseURL( const char * url,
+                     int          url_len,
+                     char     ** setme_host,
+                     int       * setme_port,
+                     char     ** setme_path );
+
+
 /***
 ****
 ***/
@@ -164,15 +185,18 @@ tr_bitfield* tr_bitfieldDup( const tr_bitfield* );
 void tr_bitfieldFree( tr_bitfield*);
 
 void tr_bitfieldClear( tr_bitfield* );
-void tr_bitfieldAdd( tr_bitfield*, size_t bit );
-void tr_bitfieldRem( tr_bitfield*, size_t bit );
-void tr_bitfieldAddRange( tr_bitfield *, size_t begin, size_t end );
-void tr_bitfieldRemRange ( tr_bitfield*, size_t begin, size_t end );
+int  tr_bitfieldAdd( tr_bitfield*, size_t bit );
+int  tr_bitfieldRem( tr_bitfield*, size_t bit );
+int  tr_bitfieldAddRange( tr_bitfield *, size_t begin, size_t end );
+int  tr_bitfieldRemRange ( tr_bitfield*, size_t begin, size_t end );
+void tr_bitfieldDifference( tr_bitfield *, const tr_bitfield * );
 
 int    tr_bitfieldHas( const tr_bitfield*, size_t bit );
 int    tr_bitfieldIsEmpty( const tr_bitfield* );
 size_t tr_bitfieldCountTrueBits( const tr_bitfield* );
 
-tr_bitfield* tr_bitfieldAnd( tr_bitfield*, const tr_bitfield* );
+tr_bitfield* tr_bitfieldOr( tr_bitfield*, const tr_bitfield* );
+
+double tr_getRatio( double numerator, double denominator );
 
 #endif
