@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Badger.h 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: Badger.h 5688 2008-04-24 21:37:46Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -31,11 +31,12 @@
     
     NSImage * fDockIcon, * fBadge;
     
-    NSDictionary * fAttributes;
-    int fCompleted, fCompletedBadged;
-    BOOL fSpeedBadge;
+    int fCompleted;
     
-    BOOL fQuittingTiger;
+    //tiger-only fields
+    NSDictionary * fAttributes;
+    int fCompletedBadged;
+    BOOL fSpeedBadge, fQuittingTiger;
 }
 
 - (id) initWithLib: (tr_handle *) lib;
