@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Badger.m 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: Badger.m 5693 2008-04-25 04:38:08Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -48,8 +48,6 @@
         fLib = lib;
         
         fCompleted = 0;
-        fCompletedBadged = 0;
-        fSpeedBadge = NO;
         
         if ([NSApp isOnLeopardOrBetter])
         {
@@ -58,7 +56,11 @@
             [view release];
         }
         else
+        {
             fQuittingTiger = NO;
+            fSpeedBadge = NO;
+            fCompletedBadged = 0;
+        }
         
         //change that just impacts the dock badge
         [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBadge) name: @"DockBadgeChange" object: nil];
@@ -71,7 +73,8 @@
 {
     [[NSNotificationCenter defaultCenter] removeObserver: self];
     
-    [NSApp setApplicationIconImage: nil]; //needed on 10.4
+    if (![NSApp isOnLeopardOrBetter])
+        [NSApp setApplicationIconImage: nil]; //needed on 10.4
     
     [fDockIcon release];
     [fAttributes release];
@@ -83,7 +86,16 @@
 {
     if ([NSApp isOnLeopardOrBetter])
     {
-        [[NSApp dockTile] display];
+        float downloadRate = 0.0, uploadRate = 0.0;
+        BOOL badgeDownload = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeDownloadRate"],
+            badgeUpload = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeUploadRate"];
+        if (badgeDownload || badgeUpload)
+            tr_torrentRates(fLib, badgeDownload ? &downloadRate : NULL, badgeUpload ? &uploadRate : NULL);
+        
+        //only update if the badged values change
+        if ([(BadgeView *)[[NSApp dockTile] contentView] setRatesWithDownload: downloadRate upload: uploadRate])
+            [[NSApp dockTile] display];
+        
         return;
     }
     else if (fQuittingTiger)
@@ -157,7 +169,7 @@
             dockIcon = [fDockIcon copy];
             
             NSRect badgeRect;
-            badgeRect.size = [[NSImage imageNamed: @"UploadBadge"] size];
+            badgeRect.size = [[NSImage imageNamed: @"UploadBadge.png"] size];
             badgeRect.origin = NSZeroPoint;
             
             //ignore shadow of badge when placing string
@@ -170,7 +182,7 @@
             if (uploadRateString)
             {
                 //place badge and text
-                [[NSImage imageNamed: @"UploadBadge"] compositeToPoint: badgeRect.origin operation: NSCompositeSourceOver];
+                [[NSImage imageNamed: @"UploadBadge.png"] compositeToPoint: badgeRect.origin operation: NSCompositeSourceOver];
                 [self badgeString: uploadRateString forRect: stringRect];
             }
             
@@ -185,7 +197,7 @@
                 }
                 
                 //place badge and text
-                [[NSImage imageNamed: @"DownloadBadge"] compositeToPoint: badgeRect.origin operation: NSCompositeSourceOver];
+                [[NSImage imageNamed: @"DownloadBadge.png"] compositeToPoint: badgeRect.origin operation: NSCompositeSourceOver];
                 [self badgeString: downloadRateString forRect: stringRect];
             }
             
@@ -231,7 +243,7 @@
     {
         [self clearCompleted];
         [(BadgeView *)[[NSApp dockTile] contentView] setQuitting];
-        [self updateBadge];
+        [[NSApp dockTile] display];
     }
     else
     {
@@ -246,13 +258,10 @@
         rect.size = [quitIcon size];
         
         NSRect badgeRect = NSMakeRect(0.0, (rect.size.height - BADGE_HEIGHT) * 0.5, rect.size.width, BADGE_HEIGHT);
-        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: badgeRect radius: 15.0];
         
         [quitIcon lockFocus];
-            
-        [[NSColor colorWithCalibratedWhite: 0.0 alpha: 0.75] set];
-        [bp fill];
         
+        [[NSImage imageNamed: @"QuitBadge.png"] compositeToPoint: badgeRect.origin operation: NSCompositeSourceOver];
         [self badgeString: NSLocalizedString(@"Quitting", "Dock Badger -> quit message") forRect: badgeRect];
         
         [quitIcon unlockFocus];
