@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: IPCController.m 4514 2008-01-06 07:02:57Z charles $
+ * $Id: IPCController.m 5715 2008-04-28 18:09:17Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -33,6 +33,7 @@
 #include "bencode.h"
 #include "ipcparse.h"
 #include "transmission.h"
+#include "utils.h"
 
 #import "IPCController.h"
 #import "Torrent.h"
@@ -145,43 +146,45 @@ PrefsController         * fPrefsController;
     if( nil  == _sock    ||
         nil  == _listen  ||
         NULL == _funcs   ||
-        nil  == _clients ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_ADDMANYFILES, msg_addold    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_ADDONEFILE,   msg_addnew    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_AUTOMAP,      msg_setbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_AUTOSTART,    msg_setbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_CRYPTO,       msg_setstr    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_DIR,          msg_setstr    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_DOWNLIMIT,    msg_setint    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETAUTOMAP,   msg_getbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETAUTOSTART, msg_getbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETCRYPTO,    msg_getstr    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETDIR,       msg_getstr    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETDOWNLIMIT, msg_getint    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETINFO,      msg_info      ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETINFOALL,   msg_infoall   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETPEX,       msg_getbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETPORT,      msg_getint    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETSTAT,      msg_info      ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETSTATALL,   msg_infoall   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_GETUPLIMIT,   msg_getint    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_LOOKUP,       msg_lookup    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_NOOP,         msg_empty     ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_PEX,          msg_setbool   ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_PORT,         msg_setint    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_QUIT,         msg_empty     ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_REMOVE,       msg_action    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_REMOVEALL,    msg_actionall ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_START,        msg_action    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_STARTALL,     msg_actionall ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_STOP,         msg_action    ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_STOPALL,      msg_actionall ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_SUP,          msg_sup       ) ||
-        0 > ipc_addmsg( _funcs, IPC_MSG_UPLIMIT,      msg_setint    ) )
+        nil  == _clients )
     {
         [self release];
         return nil;
     }
+
+    ipc_addmsg( _funcs, IPC_MSG_ADDMANYFILES, msg_addold    );
+    ipc_addmsg( _funcs, IPC_MSG_ADDONEFILE,   msg_addnew    );
+    ipc_addmsg( _funcs, IPC_MSG_AUTOMAP,      msg_setbool   );
+    ipc_addmsg( _funcs, IPC_MSG_AUTOSTART,    msg_setbool   );
+    ipc_addmsg( _funcs, IPC_MSG_CRYPTO,       msg_setstr    );
+    ipc_addmsg( _funcs, IPC_MSG_DIR,          msg_setstr    );
+    ipc_addmsg( _funcs, IPC_MSG_DOWNLIMIT,    msg_setint    );
+    ipc_addmsg( _funcs, IPC_MSG_GETAUTOMAP,   msg_getbool   );
+    ipc_addmsg( _funcs, IPC_MSG_GETAUTOSTART, msg_getbool   );
+    ipc_addmsg( _funcs, IPC_MSG_GETCRYPTO,    msg_getstr    );
+    ipc_addmsg( _funcs, IPC_MSG_GETDIR,       msg_getstr    );
+    ipc_addmsg( _funcs, IPC_MSG_GETDOWNLIMIT, msg_getint    );
+    ipc_addmsg( _funcs, IPC_MSG_GETINFO,      msg_info      );
+    ipc_addmsg( _funcs, IPC_MSG_GETINFOALL,   msg_infoall   );
+    ipc_addmsg( _funcs, IPC_MSG_GETPEX,       msg_getbool   );
+    ipc_addmsg( _funcs, IPC_MSG_GETPORT,      msg_getint    );
+    ipc_addmsg( _funcs, IPC_MSG_GETSTAT,      msg_info      );
+    ipc_addmsg( _funcs, IPC_MSG_GETSTATALL,   msg_infoall   );
+    ipc_addmsg( _funcs, IPC_MSG_GETUPLIMIT,   msg_getint    );
+    ipc_addmsg( _funcs, IPC_MSG_LOOKUP,       msg_lookup    );
+    ipc_addmsg( _funcs, IPC_MSG_NOOP,         msg_empty     );
+    ipc_addmsg( _funcs, IPC_MSG_PEX,          msg_setbool   );
+    ipc_addmsg( _funcs, IPC_MSG_PORT,         msg_setint    );
+    ipc_addmsg( _funcs, IPC_MSG_QUIT,         msg_empty     );
+    ipc_addmsg( _funcs, IPC_MSG_REMOVE,       msg_action    );
+    ipc_addmsg( _funcs, IPC_MSG_REMOVEALL,    msg_actionall );
+    ipc_addmsg( _funcs, IPC_MSG_START,        msg_action    );
+    ipc_addmsg( _funcs, IPC_MSG_STARTALL,     msg_actionall );
+    ipc_addmsg( _funcs, IPC_MSG_STOP,         msg_action    );
+    ipc_addmsg( _funcs, IPC_MSG_STOPALL,      msg_actionall );
+    ipc_addmsg( _funcs, IPC_MSG_SUP,          msg_sup       );
+    ipc_addmsg( _funcs, IPC_MSG_UPLIMIT,      msg_setint    );
+    ipc_addmsg( _funcs, IPC_MSG_VERIFY,       msg_action    );
     ipc_setdefmsg( _funcs, msg_default );
 
     [[NSNotificationCenter defaultCenter]
@@ -353,7 +356,7 @@ PrefsController         * fPrefsController;
     if( IPC_MIN_MSG_LEN > [_buf length] )
         return;
 
-    res = ipc_parse( _ipc, [_buf mutableBytes], [_buf length], self );
+    res = ipc_handleMessages( _ipc, [_buf mutableBytes], [_buf length], self );
 
     if( 0 > res )
     {
@@ -483,9 +486,9 @@ PrefsController         * fPrefsController;
     while( nil != ( tor = [enumerator nextObject] ) )
     {
         if( IPC_MSG_INFO == respid )
-            res = ipc_addinfo( pkinf, [tor torrentID], [tor torrentInfo], types );
+            res = ipc_addinfo( pkinf, [tor torrentID], [tor torrentStruct], types );
         else
-            res = ipc_addstat( pkinf, [tor torrentID], [tor torrentStat], types );
+            res = ipc_addstat( pkinf, [tor torrentID], [tor torrentStruct], types );
         if( 0 > res )
         {
             tr_bencFree( &packet );
@@ -493,7 +496,7 @@ PrefsController         * fPrefsController;
         }
     }
 
-    buf = ipc_mkval( &packet, &size );
+    buf = ipc_serialize( &packet, &size );
     tr_bencFree( &packet );
     if( NULL == buf )
         goto fail;
@@ -512,8 +515,7 @@ void getaddr( struct sockaddr_un * sun )
 {
     bzero( sun, sizeof *sun );
     sun->sun_family = AF_LOCAL;
-    strlcpy( sun->sun_path, tr_getPrefsDirectory(), sizeof sun->sun_path );
-    strlcat( sun->sun_path, "/socket", sizeof sun->sun_path );
+    tr_buildPath( sun->sun_path, sizeof( sun->sun_path ), tr_getDefaultConfigDir(), "socket", NULL );
 }
 
 NSArray * bencarray( benc_val_t * val, int type )
@@ -551,11 +553,6 @@ void msg_lookup( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg )
 {
     IPCClient      * client = arg;
     NSArray        * hashes, * tors;
-    benc_val_t       packet, * pkinf;
-    NSEnumerator   * enumerator;
-    Torrent        * tor;
-    uint8_t        * buf;
-    size_t           size;
 
     hashes = bencarray( val, TYPE_STR );
     if( NULL == hashes )
@@ -646,6 +643,9 @@ void msg_action( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg )
         case IPC_MSG_STOP:
             res = [delegate ipcStopTorrents: tors];
             break;
+        case IPC_MSG_VERIFY:
+            res = [delegate ipcVerifyTorrents: tors];
+            break;
         default:
             assert( 0 );
             return;
@@ -692,7 +692,6 @@ void msg_addold( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg )
 {
     IPCClient * client = arg;
     NSArray   * paths;
-    BOOL        res;
 
     paths = bencarray( val, TYPE_STR );
     if( nil == paths )
@@ -1053,7 +1052,7 @@ void msg_sup( enum ipc_msg msgid, benc_val_t * val, int64_t tag, void * arg )
                         name->val.s.s, name->val.s.i, 1 );
     }
 
-    buf = ipc_mkval( &packet, &size );
+    buf = ipc_serialize( &packet, &size );
     tr_bencFree( &packet );
     if( NULL == buf )
         goto fail;
