@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: MessageWindowController.m 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: MessageWindowController.m 5628 2008-04-15 19:57:56Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -23,6 +23,8 @@
  *****************************************************************************/
 
 #import "MessageWindowController.h"
+#import "NSStringAdditions.h"
+#import "NSApplicationAdditions.h"
 #import <transmission.h>
 
 #define LEVEL_ERROR 0
@@ -36,7 +38,6 @@
 
 - (void) resizeColumn;
 - (NSString *) stringForMessage: (NSDictionary *) message;
-- (NSString *) fileForMessage: (NSDictionary *) message;
 
 @end
 
@@ -64,9 +65,12 @@
     [window setFrameUsingName: @"MessageWindowFrame"];
     
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(resizeColumn)
-                    name: @"NSTableViewColumnDidResizeNotification" object: fMessageTable];
+        name: @"NSTableViewColumnDidResizeNotification" object: fMessageTable];
     
-    //initially sort peer table by IP
+    if ([NSApp isOnLeopardOrBetter])
+        [window setContentBorderThickness: [[fMessageTable enclosingScrollView] frame].origin.y forEdge: NSMinYEdge];
+    
+    //initially sort peer table by date
     if ([[fMessageTable sortDescriptors] count] == 0)
         [fMessageTable setSortDescriptors: [NSArray arrayWithObject: [[fMessageTable tableColumnWithIdentifier: @"Date"]
                                             sortDescriptorPrototype]]];
@@ -75,10 +79,41 @@
     fInfoImage = [NSImage imageNamed: @"YellowDot.png"];
     fDebugImage = [NSImage imageNamed: @"PurpleDot.png"];
     
-    //set images to popup button items
+    [[self window] setTitle: NSLocalizedString(@"Message Log", "Message window -> title")];
+    
+    //set images and text for popup button items
     [[fLevelButton itemAtIndex: LEVEL_ERROR] setImage: fErrorImage];
+    [[fLevelButton itemAtIndex: LEVEL_ERROR] setTitle: NSLocalizedString(@"Error", "Message window -> level string")];
     [[fLevelButton itemAtIndex: LEVEL_INFO] setImage: fInfoImage];
+    [[fLevelButton itemAtIndex: LEVEL_INFO] setTitle: NSLocalizedString(@"Info", "Message window -> level string")];
     [[fLevelButton itemAtIndex: LEVEL_DEBUG] setImage: fDebugImage];
+    [[fLevelButton itemAtIndex: LEVEL_DEBUG] setTitle: NSLocalizedString(@"Debug", "Message window -> level string")];
+    
+    //set table column text
+    [[[fMessageTable tableColumnWithIdentifier: @"Date"] headerCell] setTitle: NSLocalizedString(@"Date",
+        "Message window -> table column")];
+    [[[fMessageTable tableColumnWithIdentifier: @"Name"] headerCell] setTitle: NSLocalizedString(@"Process",
+        "Message window -> table column")];
+    [[[fMessageTable tableColumnWithIdentifier: @"Message"] headerCell] setTitle: NSLocalizedString(@"Message",
+        "Message window -> table column")];
+    
+    //set and size buttons
+    [fSaveButton setTitle: [NSLocalizedString(@"Save", "Message window -> save button") stringByAppendingEllipsis]];
+    [fSaveButton sizeToFit];
+    
+    NSRect saveButtonFrame = [fSaveButton frame];
+    saveButtonFrame.size.width += 10.0;
+    [fSaveButton setFrame: saveButtonFrame];
+    
+    float oldClearButtonWidth = [fClearButton frame].size.width;
+    
+    [fClearButton setTitle: NSLocalizedString(@"Clear", "Message window -> save button")];
+    [fClearButton sizeToFit];
+    
+    NSRect clearButtonFrame = [fClearButton frame];
+    clearButtonFrame.size.width = MAX(clearButtonFrame.size.width + 10.0, saveButtonFrame.size.width);
+    clearButtonFrame.origin.x -= clearButtonFrame.size.width - oldClearButtonWidth;
+    [fClearButton setFrame: clearButtonFrame];
     
     //select proper level in popup button
     switch (tr_getMessageLevel())
@@ -94,6 +129,7 @@
     }
     
     fMessages = [[NSMutableArray alloc] init];
+    fIndex = 0;
 }
 
 - (void) windowDidBecomeKey: (NSNotification *) notification
@@ -116,21 +152,22 @@
     if ((messages = tr_getQueuedMessages()) == NULL)
         return;
     
-    NSMutableDictionary * message;
     for (currentMessage = messages; currentMessage != NULL; currentMessage = currentMessage->next)
     {
-        message  = [NSMutableDictionary dictionaryWithObjectsAndKeys:
-                    [NSString stringWithUTF8String: currentMessage->message], @"Message",
-                    [NSDate dateWithTimeIntervalSince1970: currentMessage->when], @"Date",
-                    [NSNumber numberWithInt: currentMessage->level], @"Level", nil];
+        NSString * name = currentMessage->name != NULL ? [NSString stringWithUTF8String: currentMessage->name]
+                            : [[NSProcessInfo processInfo] processName];
         
-        if (currentMessage->file != NULL)
-        {
-            [message setObject: [NSString stringWithUTF8String: currentMessage->file] forKey: @"File"];
-            [message setObject: [NSNumber numberWithInt: currentMessage->line] forKey: @"Line"];
-        }
+        NSDictionary * message  = [NSDictionary dictionaryWithObjectsAndKeys:
+                                    [NSString stringWithUTF8String: currentMessage->message], @"Message",
+                                    [NSDate dateWithTimeIntervalSince1970: currentMessage->when], @"Date",
+                                    [NSNumber numberWithUnsignedInt: fIndex], @"Index", //more accurate when sorting by date
+                                    [NSNumber numberWithInt: currentMessage->level], @"Level",
+                                    name, @"Name",
+                                    [NSString stringWithUTF8String: currentMessage->file], @"File",
+                                    [NSNumber numberWithInt: currentMessage->line], @"Line", nil];
                                 
         [fMessages addObject: message];
+        fIndex++;
     }
     
     tr_freeMessageList(messages);
@@ -142,14 +179,13 @@
     if (total > MAX_MESSAGES)
     {
         //remove the oldest
-        NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Date" ascending: YES] autorelease];
-        NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
-        [fMessages sortUsingDescriptors: descriptors];
-        [descriptors release];
+        NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
+        [fMessages sortUsingDescriptors: [NSArray arrayWithObject: descriptor]];
         
         [fMessages removeObjectsInRange: NSMakeRange(0, total-MAX_MESSAGES)];
         
         [fMessageTable noteHeightOfRowsWithIndexesChanged: [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, MAX_MESSAGES)]];
+        total = MAX_MESSAGES;
     }
     
     [fMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
@@ -185,6 +221,8 @@
                 return nil;
         }
     }
+    else if ([ident isEqualToString: @"Name"])
+        return [message objectForKey: @"Name"];
     else
         return [message objectForKey: @"Message"];
 }
@@ -210,7 +248,8 @@
 - (NSString *) tableView: (NSTableView *) tableView toolTipForCell: (NSCell *) cell rect: (NSRectPointer) rect
                 tableColumn: (NSTableColumn *) column row: (int) row mouseLocation: (NSPoint) mouseLocation
 {
-    return [self fileForMessage: [fMessages objectAtIndex: row]];
+    NSDictionary * message = [fMessages objectAtIndex: row];
+    return [NSString stringWithFormat: @"%@:%@", [[message objectForKey: @"File"] lastPathComponent], [message objectForKey: @"Line"]];
 }
 
 - (void) copy: (id) sender
@@ -270,7 +309,7 @@
 - (void) writeToFile: (id) sender
 {
     //make the array sorted by date
-    NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Date" ascending: YES] autorelease];
+    NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
     NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
     NSArray * sortedMessages = [fMessages sortedArrayUsingDescriptors: descriptors];
     [descriptors release];
@@ -332,35 +371,21 @@
     switch ([[message objectForKey: @"Level"] intValue])
     {
         case TR_MSG_ERR:
-            level = @"[Error]";
+            level = NSLocalizedString(@"Error", "Message window -> level");
             break;
         case TR_MSG_INF:
-            level = @"[Info]";
+            level = NSLocalizedString(@"Info", "Message window -> level");
             break;
         case TR_MSG_DBG:
-            level = @"[Debug]";
+            level = NSLocalizedString(@"Debug", "Message window -> level");
             break;
         default:
             level = @"";
     }
     
-    NSMutableArray * strings = [NSMutableArray arrayWithObjects: [message objectForKey: @"Date"], level,
-                                [message objectForKey: @"Message"], nil];
-    NSString * file;
-    if ((file = [self fileForMessage: message]))
-        [strings insertObject: file atIndex: 1];
-    
-    return [strings componentsJoinedByString: @" "];
-}
-
-- (NSString *) fileForMessage: (NSDictionary *) message
-{
-    NSString * file;
-    if ((file = [message objectForKey: @"File"]))
-        return [NSString stringWithFormat: @"%@:%@", [[message objectForKey: @"File"] lastPathComponent],
-                [message objectForKey: @"Line"]];
-    else
-        return nil;
+    return [NSString stringWithFormat: @"%@ %@:%@ [%@] %@: %@", [message objectForKey: @"Date"],
+            [[message objectForKey: @"File"] lastPathComponent], [message objectForKey: @"Line"], level,
+            [message objectForKey: @"Name"], [message objectForKey: @"Message"], nil];
 }
 
 @end
