@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.h 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: PrefsController.h 5633 2008-04-17 04:25:49Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -31,34 +31,35 @@
 {
     tr_handle * fHandle;
     
-    NSUserDefaults          * fDefaults;
-    BOOL                    fHasLoaded;
+    NSUserDefaults * fDefaults;
+    BOOL fHasLoaded;
     
-    IBOutlet NSView         * fGeneralView, * fTransfersView, * fBandwidthView, * fAdvancedView;
+    IBOutlet NSView * fGeneralView, * fTransfersView, * fBandwidthView, * fPeersView, * fNetworkView;
     
     NSString * fInitialString;
     
-    IBOutlet NSPopUpButton  * fFolderPopUp, * fIncompleteFolderPopUp, * fImportFolderPopUp,
-                            * fDownloadSoundPopUp, * fSeedingSoundPopUp;
-    IBOutlet NSTextField    * fRatioStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
+    IBOutlet NSPopUpButton * fFolderPopUp, * fIncompleteFolderPopUp, * fImportFolderPopUp;
+    IBOutlet NSTextField * fRatioStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
     
-    SUUpdater               * fUpdater;
+    SUUpdater * fUpdater;
 
-    IBOutlet NSTextField    * fUploadField, * fDownloadField,
-                            * fSpeedLimitUploadField, * fSpeedLimitDownloadField;
+    IBOutlet NSTextField * fUploadField, * fDownloadField,
+                        * fSpeedLimitUploadField, * fSpeedLimitDownloadField;
     
-    IBOutlet NSTextField    * fPeersGlobalField, * fPeersTorrentField;
+    IBOutlet NSTextField * fPeersGlobalField, * fPeersTorrentField, * fBlocklistMessageField;
+    IBOutlet NSButton * fBlocklistEnableCheck;
     
-    PortChecker                     * fPortChecker;
-    IBOutlet NSTextField            * fPortField, * fPortStatusField;
-    IBOutlet NSButton               * fNatCheck;
-    IBOutlet NSImageView            * fPortStatusImage;
-    IBOutlet NSProgressIndicator    * fPortStatusProgress;
-    NSTimer                         * fPortStatusTimer;
-    int                             fPublicPort, fNatStatus;
+    PortChecker * fPortChecker;
+    IBOutlet NSTextField * fPortField, * fPortStatusField;
+    IBOutlet NSButton * fNatCheck;
+    IBOutlet NSImageView * fPortStatusImage;
+    IBOutlet NSProgressIndicator * fPortStatusProgress;
+    NSTimer * fPortStatusTimer;
+    int fPublicPort, fNatStatus;
 }
 
 - (id) initWithHandle: (tr_handle *) handle;
+- (tr_handle *) handle;
 - (void) setUpdater: (SUUpdater *) updater;
 
 - (void) updatePortField;
@@ -77,6 +78,10 @@
 
 - (void) setEncryptionMode: (id) sender;
 
+- (void) setBlocklistEnabled: (id) sender;
+- (void) updateBlocklist: (id) sender;
+- (void) updateBlocklistFields;
+
 - (void) setBadge: (id) sender;
 - (void) resetWarnings: (id) sender;
 - (void) setCheckForUpdate: (id) sender;
@@ -108,6 +113,7 @@
 
 - (void) setAutoSize: (id) sender;
 
+- (void) helpForPeers: (id) sender;
 - (void) helpForNetwork: (id) sender;
 
 @end
