@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.m 4753 2008-01-20 04:50:52Z livings124 $
+ * $Id: PrefsController.m 5742 2008-05-04 19:45:51Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,20 +23,21 @@
  *****************************************************************************/
 
 #import "PrefsController.h"
+#import "BlocklistDownloader.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
 
 #define DOWNLOAD_FOLDER     0
 #define DOWNLOAD_TORRENT    2
-#define DOWNLOAD_ASK        3
 
 #define UPDATE_SECONDS 86400
 
 #define TOOLBAR_GENERAL     @"TOOLBAR_GENERAL"
 #define TOOLBAR_TRANSFERS   @"TOOLBAR_TRANSFERS"
 #define TOOLBAR_BANDWIDTH   @"TOOLBAR_BANDWIDTH"
-#define TOOLBAR_ADVANCED    @"TOOLBAR_ADVANCED"
+#define TOOLBAR_PEERS       @"TOOLBAR_PEERS"
+#define TOOLBAR_NETWORK     @"TOOLBAR_NETWORK"
 
 @interface PrefsController (Private)
 
@@ -57,7 +58,7 @@
         fDefaults = [NSUserDefaults standardUserDefaults];
         fHandle = handle;
         
-        //checks for old version upload speed of -1
+        //checks for old version speeds of -1
         if ([fDefaults integerForKey: @"UploadLimit"] < 0)
         {
             [fDefaults setInteger: 20 forKey: @"UploadLimit"];
@@ -69,6 +70,16 @@
             [fDefaults setBool: NO forKey: @"CheckDownload"];
         }
         
+        //check for old version download location (before 1.1)
+        NSString * choice;
+        if ((choice = [fDefaults stringForKey: @"DownloadChoice"]))
+        {
+            [fDefaults setBool: [choice isEqualToString: @"Constant"] forKey: @"DownloadLocationConstant"];
+            [fDefaults setBool: YES forKey: @"DownloadAsk"];
+            
+            [fDefaults removeObjectForKey: @"DownloadChoice"];
+        }
+        
         //set check for update to right value
         [self setCheckForUpdate: nil];
         
@@ -87,6 +98,11 @@
     return self;
 }
 
+- (tr_handle *) handle
+{
+    return fHandle;
+}
+
 - (void) dealloc
 {
     if (fPortStatusTimer)
@@ -116,13 +132,7 @@
     [self setPrefView: nil];
     
     //set download folder
-    NSString * downloadChoice = [fDefaults stringForKey: @"DownloadChoice"];
-    if ([downloadChoice isEqualToString: @"Constant"])
-        [fFolderPopUp selectItemAtIndex: DOWNLOAD_FOLDER];
-    else if ([downloadChoice isEqualToString: @"Torrent"])
-        [fFolderPopUp selectItemAtIndex: DOWNLOAD_TORRENT];
-    else
-        [fFolderPopUp selectItemAtIndex: DOWNLOAD_ASK];
+    [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
     
     //set stop ratio
     [self updateRatioStopField];
@@ -150,6 +160,9 @@
     [fQueueDownloadField setIntValue: [fDefaults integerForKey: @"QueueDownloadNumber"]];
     [fQueueSeedField setIntValue: [fDefaults integerForKey: @"QueueSeedNumber"]];
     [fStalledField setIntValue: [fDefaults integerForKey: @"StalledMinutes"]];
+    
+    //set blocklist
+    [self updateBlocklistFields];
 }
 
 - (void) setUpdater: (SUUpdater *) updater
@@ -159,12 +172,11 @@
 
 - (NSToolbarItem *) toolbar: (NSToolbar *) toolbar itemForItemIdentifier: (NSString *) ident willBeInsertedIntoToolbar: (BOOL) flag
 {
-    NSToolbarItem * item;
-    item = [[NSToolbarItem alloc] initWithItemIdentifier: ident];
+    NSToolbarItem * item = [[NSToolbarItem alloc] initWithItemIdentifier: ident];
 
     if ([ident isEqualToString: TOOLBAR_GENERAL])
     {
-        [item setLabel: NSLocalizedString(@"General", "Preferences -> General toolbar item title")];
+        [item setLabel: NSLocalizedString(@"General", "Preferences -> toolbar item title")];
         [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNamePreferencesGeneral : @"Preferences.png"]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
@@ -172,7 +184,7 @@
     }
     else if ([ident isEqualToString: TOOLBAR_TRANSFERS])
     {
-        [item setLabel: NSLocalizedString(@"Transfers", "Preferences -> Transfers toolbar item title")];
+        [item setLabel: NSLocalizedString(@"Transfers", "Preferences -> toolbar item title")];
         [item setImage: [NSImage imageNamed: @"Transfers.png"]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
@@ -180,16 +192,24 @@
     }
     else if ([ident isEqualToString: TOOLBAR_BANDWIDTH])
     {
-        [item setLabel: NSLocalizedString(@"Bandwidth", "Preferences -> Bandwidth toolbar item title")];
+        [item setLabel: NSLocalizedString(@"Bandwidth", "Preferences -> toolbar item title")];
         [item setImage: [NSImage imageNamed: @"Bandwidth.png"]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
     }
-    else if ([ident isEqualToString: TOOLBAR_ADVANCED])
+    else if ([ident isEqualToString: TOOLBAR_PEERS])
     {
-        [item setLabel: NSLocalizedString(@"Advanced", "Preferences -> Advanced toolbar item title")];
-        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNameAdvanced : @"Advanced.png"]];
+        [item setLabel: NSLocalizedString(@"Peers", "Preferences -> toolbar item title")];
+        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNameUserGroup : @"Peers.png"]];
+        [item setTarget: self];
+        [item setAction: @selector(setPrefView:)];
+        [item setAutovalidates: NO];
+    }
+    else if ([ident isEqualToString: TOOLBAR_NETWORK])
+    {
+        [item setLabel: NSLocalizedString(@"Network", "Preferences -> toolbar item title")];
+        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNameNetwork : @"Network.png"]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
@@ -215,7 +235,7 @@
 
 - (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
 {
-    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_BANDWIDTH, TOOLBAR_ADVANCED, nil];
+    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_BANDWIDTH, TOOLBAR_PEERS, TOOLBAR_NETWORK, nil];
 }
 
 //used by ipc
@@ -244,14 +264,14 @@
 
 - (void) updatePortStatus
 {
-    tr_handle_status * stat = tr_handleStatus(fHandle);
+    const tr_handle_status * stat = tr_handleStatus(fHandle);
     if (fNatStatus != stat->natTraversalStatus || fPublicPort != stat->publicPort)
     {
         fNatStatus = stat->natTraversalStatus;
         fPublicPort = stat->publicPort;
         
-        [fPortStatusField setStringValue: [NSLocalizedString(@"Checking port status",
-                                            "Preferences -> Advanced -> port status") stringByAppendingEllipsis]];
+        [fPortStatusField setStringValue: [NSLocalizedString(@"Checking port status", "Preferences -> Network -> port status")
+            stringByAppendingEllipsis]];
         [fPortStatusImage setImage: nil];
         [fPortStatusProgress startAnimation: self];
         
@@ -270,20 +290,20 @@
     switch ([fPortChecker status])
     {
         case PORT_STATUS_OPEN:
-            [fPortStatusField setStringValue: NSLocalizedString(@"Port is open", "Preferences -> Advanced -> port status")];
+            [fPortStatusField setStringValue: NSLocalizedString(@"Port is open", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"GreenDot.png"]];
             break;
         case PORT_STATUS_CLOSED:
-            [fPortStatusField setStringValue: NSLocalizedString(@"Port is closed", "Preferences -> Advanced -> port status")];
+            [fPortStatusField setStringValue: NSLocalizedString(@"Port is closed", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"RedDot.png"]];
             break;
         case PORT_STATUS_STEALTH:
-            [fPortStatusField setStringValue: NSLocalizedString(@"Port is stealth", "Preferences -> Advanced -> port status")];
+            [fPortStatusField setStringValue: NSLocalizedString(@"Port is stealth", "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"RedDot.png"]];
             break;
         case PORT_STATUS_ERROR:
             [fPortStatusField setStringValue: NSLocalizedString(@"Unable to check port status",
-                                                "Preferences -> Advanced -> port status")];
+                                                "Preferences -> Network -> port status")];
             [fPortStatusImage setImage: [NSImage imageNamed: @"YellowDot.png"]];
             break;
     }
@@ -295,25 +315,24 @@
 {
     NSMutableArray * sounds = [NSMutableArray array];
     
-    //until Apple can fix soundNamed to not crash on invalid sound files, don't use custom sounds
-    NSArray * directories = [NSArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds",
-                                /*[NSHomeDirectory() stringByAppendingPathComponent: @"Library/Sounds"],*/ nil];
-    BOOL isDirectory;
-    NSEnumerator * soundEnumerator;
-    NSString * sound;
+    NSMutableArray * directories = [NSMutableArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds", nil];
+    if ([NSApp isOnLeopardOrBetter])
+        [directories addObject: [NSHomeDirectory() stringByAppendingPathComponent: @"Library/Sounds"]];
     
+    BOOL isDirectory;
     NSString * directory;
     NSEnumerator * enumerator = [directories objectEnumerator];
     while ((directory = [enumerator nextObject]))
         if ([[NSFileManager defaultManager] fileExistsAtPath: directory isDirectory: &isDirectory] && isDirectory)
         {
-                soundEnumerator = [[[NSFileManager defaultManager] directoryContentsAtPath: directory] objectEnumerator];
-                while ((sound = [soundEnumerator nextObject]))
-                {
-                    sound = [sound stringByDeletingPathExtension];
-                    if ([NSSound soundNamed: sound])
-                        [sounds addObject: sound];
-                }
+            NSString * sound;
+            NSEnumerator * soundEnumerator = [[[NSFileManager defaultManager] directoryContentsAtPath: directory] objectEnumerator];
+            while ((sound = [soundEnumerator nextObject]))
+            {
+                sound = [sound stringByDeletingPathExtension];
+                if ([NSSound soundNamed: sound])
+                    [sounds addObject: sound];
+            }
         }
     
     return sounds;
@@ -351,6 +370,41 @@
         ([fDefaults boolForKey: @"EncryptionRequire"] ? TR_ENCRYPTION_REQUIRED : TR_ENCRYPTION_PREFERRED) : TR_PLAINTEXT_PREFERRED);
 }
 
+- (void) setBlocklistEnabled: (id) sender
+{
+    BOOL enable = [sender state] == NSOnState;
+    [fDefaults setBool: enable forKey: @"Blocklist"];
+    tr_blocklistSetEnabled(fHandle, enable);
+}
+
+- (void) updateBlocklist: (id) sender
+{
+    [BlocklistDownloader downloadWithPrefsController: self];
+}
+
+- (void) updateBlocklistFields
+{
+    BOOL exists = tr_blocklistExists(fHandle);
+    
+    if (exists)
+    {
+        NSNumberFormatter * numberFormatter = [[NSNumberFormatter alloc] init];
+        [numberFormatter setNumberStyle: NSNumberFormatterDecimalStyle];
+        [numberFormatter setMaximumFractionDigits: 0];
+        NSString * countString = [numberFormatter stringFromNumber: [NSNumber numberWithInt: tr_blocklistGetRuleCount(fHandle)]];
+        [numberFormatter release];
+        
+        [fBlocklistMessageField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ IP address rules in list",
+            "Prefs -> blocklist -> message"), countString]];
+    }
+    else 
+        [fBlocklistMessageField setStringValue: NSLocalizedString(@"A blocklist must first be downloaded",
+            "Prefs -> blocklist -> message")];
+    
+    [fBlocklistEnableCheck setEnabled: exists];
+    [fBlocklistEnableCheck setState: exists && [fDefaults boolForKey: @"Blocklist"]];
+}
+
 - (void) applySpeedSettings: (id) sender
 {
     if ([fDefaults boolForKey: @"SpeedLimit"])
@@ -440,7 +494,7 @@
 }
 
 - (void) setBadge: (id) sender
-{   
+{
     [[NSNotificationCenter defaultCenter] postNotificationName: @"DockBadgeChange" object: self];
 }
 
@@ -448,6 +502,8 @@
 {
     [fDefaults setBool: YES forKey: @"WarningDuplicate"];
     [fDefaults setBool: YES forKey: @"WarningRemainingSpace"];
+    [fDefaults setBool: YES forKey: @"WarningFolderDataSameName"];
+    [fDefaults setBool: YES forKey: @"WarningResetStats"];
 }
 
 - (void) setCheckForUpdate: (id) sender
@@ -482,18 +538,7 @@
 
 - (void) setDownloadLocation: (id) sender
 {
-    switch ([fFolderPopUp indexOfSelectedItem])
-    {
-        case DOWNLOAD_FOLDER:
-            [fDefaults setObject: @"Constant" forKey: @"DownloadChoice"];
-            break;
-        case DOWNLOAD_TORRENT:
-            [fDefaults setObject: @"Torrent" forKey: @"DownloadChoice"];
-            break;
-        case DOWNLOAD_ASK:
-            [fDefaults setObject: @"Ask" forKey: @"DownloadChoice"];
-            break;
-    }
+    [fDefaults setBool: [fFolderPopUp indexOfSelectedItem] == DOWNLOAD_FOLDER forKey: @"DownloadLocationConstant"];
 }
 
 - (void) folderSheetShow: (id) sender
@@ -563,9 +608,15 @@
     [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoSizeSettingChange" object: self];
 }
 
+- (void) helpForPeers: (id) sender
+{
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"PeersPrefs"
+        inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
+}
+
 - (void) helpForNetwork: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"PortForwarding"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"NetworkPrefs"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
@@ -583,9 +634,11 @@
             view = fTransfersView;
         else if ([identifier isEqualToString: TOOLBAR_BANDWIDTH])
             view = fBandwidthView;
-        else if ([identifier isEqualToString: TOOLBAR_ADVANCED])
-            view = fAdvancedView;
-        else;
+        else if ([identifier isEqualToString: TOOLBAR_PEERS])
+            view = fPeersView;
+        else if ([identifier isEqualToString: TOOLBAR_NETWORK])
+            view = fNetworkView;
+        else; //general view already selected
     }
     
     NSWindow * window = [self window];
@@ -619,8 +672,8 @@
             }
     }
     
-    //for advanced view make sure progress indicator hides itself
-    if (view == fAdvancedView && [fPortStatusImage image])
+    //for network view make sure progress indicator hides itself (get around a Tiger bug)
+    if (![NSApp isOnLeopardOrBetter] && view == fNetworkView && [fPortStatusImage image])
         [fPortStatusProgress setDisplayedWhenStopped: NO];
 }
 
@@ -635,13 +688,7 @@
     else
     {
         //reset if cancelled
-        NSString * downloadChoice = [fDefaults stringForKey: @"DownloadChoice"];
-        if ([downloadChoice isEqualToString: @"Constant"])
-            [fFolderPopUp selectItemAtIndex: DOWNLOAD_FOLDER];
-        else if ([downloadChoice isEqualToString: @"Torrent"])
-            [fFolderPopUp selectItemAtIndex: DOWNLOAD_TORRENT];
-        else
-            [fFolderPopUp selectItemAtIndex: DOWNLOAD_ASK];
+        [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
     }
 }
 
