@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: StatusBarView.h 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: StatusBarView.h 5421 2008-03-28 18:47:43Z livings124 $
  * 
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -24,15 +24,9 @@
 
 #import <Cocoa/Cocoa.h>
 
-@class CTGradient;
-
 @interface StatusBarView : NSView
 {
-    BOOL fShow;
-    
-    CTGradient * fGradient;
+    NSColor * fGrayBorderColor;
 }
 
-- (void) setShowOnTiger: (BOOL) show;
-
 @end
