@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: StatusBarView.m 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: StatusBarView.m 5421 2008-03-28 18:47:43Z livings124 $
  * 
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -23,8 +23,6 @@
  *****************************************************************************/
 
 #import "StatusBarView.h"
-#import "NSApplicationAdditions.h"
-#import "CTGradient.h"
 
 @implementation StatusBarView
 
@@ -32,35 +30,40 @@
 {
     if ((self = [super initWithFrame: rect]))
     {
-        fShow = [NSApp isOnLeopardOrBetter];
-            
-        NSColor * startingColor = [NSColor colorWithCalibratedRed: 208.0/255.0 green: 208.0/255.0 blue: 208.0/255.0 alpha: 1.0];
-        NSColor * endingColor = [NSColor colorWithCalibratedRed: 233.0/255.0 green: 233.0/255.0 blue: 233.0/255.0 alpha: 1.0];
-        fGradient = [[CTGradient gradientWithBeginningColor: startingColor endingColor: endingColor] retain];
+        fGrayBorderColor = [[NSColor colorWithCalibratedRed: 171.0/255.0 green: 171.0/255.0 blue: 171.0/255.0 alpha: 1.0] retain];
     }
     return self;
 }
 
 - (void) dealloc
 {
-    [fGradient release];
+    [fGrayBorderColor release];
     [super dealloc];
 }
 
-- (void) setShowOnTiger: (BOOL) show
-{
-    fShow = show || [NSApp isOnLeopardOrBetter];
-}
-
-- (BOOL) isOpaque
-{
-    return fShow;
-}
-
 - (void) drawRect: (NSRect) rect
 {
-    if (fShow)
-        [fGradient fillRect: rect angle: 90];
+    NSRect lineBorderRect = NSMakeRect(rect.origin.x, [self bounds].size.height - 1.0, rect.size.width, 1.0);
+    if (NSIntersectsRect(lineBorderRect, rect))
+    {
+        [[NSColor whiteColor] set];
+        NSRectFill(lineBorderRect);
+        
+        rect.size.height--;
+    }
+    
+    lineBorderRect.origin.y = 0.0;
+    if (NSIntersectsRect(lineBorderRect, rect))
+    {
+        [fGrayBorderColor set];
+        NSRectFill(lineBorderRect);
+        
+        rect.origin.y++;
+        rect.size.height--;
+    }
+    
+    [[NSColor controlColor] set];
+    NSRectFill(rect);
 }
 
 @end
