@@ -44,6 +44,7 @@
 #include <sys/socket.h>
 #include <sys/wait.h>
 #include <sys/signal.h>
+#include <assert.h>
 #include <unistd.h>
 #include <netdb.h>
 #endif
@@ -471,12 +472,14 @@ test_fork(void)
 			exit(1);
 		}
 
+		called = 0;
+
 		event_dispatch();
 
 		/* we do not send an EOF; simple_read_cb requires an EOF 
 		 * to set test_ok.  we just verify that the callback was
 		 * called. */
-		exit(test_ok != 0 || called != 2);
+		exit(test_ok != 0 || called != 2 ? -2 : 76);
 	}
 
 	/* wait for the child to read the data */
@@ -489,8 +492,8 @@ test_fork(void)
 		exit(1);
 	}
 	
-	if (WEXITSTATUS(status) != 0) {
-		fprintf(stderr, "FAILED (exit)\n");
+	if (WEXITSTATUS(status) != 76) {
+		fprintf(stderr, "FAILED (exit): %d\n", WEXITSTATUS(status));
 		exit(1);
 	}
 
@@ -905,6 +908,10 @@ test_evbuffer_find(void)
 	evbuffer_free(buf);
 }
 
+/*
+ * simple bufferevent test
+ */
+
 static void
 readcb(struct bufferevent *bev, void *arg)
 {
@@ -958,6 +965,74 @@ test_bufferevent(void)
 	cleanup_test();
 }
 
+/*
+ * test watermarks and bufferevent
+ */
+
+static void
+wm_readcb(struct bufferevent *bev, void *arg)
+{
+	int len = EVBUFFER_LENGTH(bev->input);
+	static int nread;
+
+	assert(len >= 10 && len <= 20);
+
+	evbuffer_drain(bev->input, len);
+
+	nread += len;
+	if (nread == 65000) {
+		bufferevent_disable(bev, EV_READ);
+		test_ok++;
+	}
+}
+
+static void
+wm_writecb(struct bufferevent *bev, void *arg)
+{
+	if (EVBUFFER_LENGTH(bev->output) == 0)
+		test_ok++;
+}
+
+static void
+wm_errorcb(struct bufferevent *bev, short what, void *arg)
+{
+	test_ok = -2;
+}
+
+static void
+test_bufferevent_watermarks(void)
+{
+	struct bufferevent *bev1, *bev2;
+	char buffer[65000];
+	int i;
+
+	setup_test("Bufferevent Watermarks: ");
+
+	bev1 = bufferevent_new(pair[0], NULL, wm_writecb, wm_errorcb, NULL);
+	bev2 = bufferevent_new(pair[1], wm_readcb, NULL, wm_errorcb, NULL);
+
+	bufferevent_disable(bev1, EV_READ);
+	bufferevent_enable(bev2, EV_READ);
+
+	for (i = 0; i < sizeof(buffer); i++)
+		buffer[i] = i;
+
+	bufferevent_write(bev1, buffer, sizeof(buffer));
+
+	/* limit the reading on the receiving bufferevent */
+	bufferevent_setwatermark(bev2, EV_READ, 10, 20);
+
+	event_dispatch();
+
+	bufferevent_free(bev1);
+	bufferevent_free(bev2);
+
+	if (test_ok != 2)
+		test_ok = 0;
+
+	cleanup_test();
+}
+
 struct test_pri_event {
 	struct event ev;
 	int count;
@@ -1391,6 +1466,7 @@ main (int argc, char **argv)
 	test_evbuffer_find();
 	
 	test_bufferevent();
+	test_bufferevent_watermarks();
 
 	test_free_active_base();
 
