@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: torrent-list.cc 3178 2007-09-26 01:55:04Z charles $
+ * $Id: torrent-list.cc 5457 2008-03-31 17:59:16Z charles $
  */
 
 #include <wx/intl.h>
@@ -303,12 +303,10 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
             case COL_STATE: /* FIXME: divine the meaning of these two columns */
             case COL_STATUS:
                 switch( s->status ) {
-                    case TR_STATUS_STOPPING:    xstr = _("Stopping"); break;
                     case TR_STATUS_STOPPED:     xstr = _("Stopped"); break;
                     case TR_STATUS_CHECK:       xstr = wxString::Format ( _("Checking Files (%.0f)"), s->recheckProgress );  break;
                     case TR_STATUS_CHECK_WAIT:  xstr = _("Waiting to Check"); break;
                     case TR_STATUS_DOWNLOAD:    xstr = _("Downloading"); break;
-                    case TR_STATUS_DONE:
                     case TR_STATUS_SEED:        xstr = _("Seeding"); break;
                     default: assert( 0 );
                 }
