@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: xmission.cc 4390 2007-12-31 01:19:35Z livings124 $
+ * $Id: xmission.cc 4495 2008-01-05 18:17:23Z charles $
  */
 
 #include <ctype.h>
@@ -282,16 +282,17 @@ MyFrame :: OnDeselectAllUpdate( wxUpdateUIEvent& event )
 void
 MyFrame :: OnStartUpdate( wxUpdateUIEvent& event )
 {
-    unsigned long l = 0;
+    bool enable = false;
     foreach( torrents_v, mySelectedTorrents, it )
-        l |= tr_torrentStat(*it)->status;
-    event.Enable( (l & TR_STATUS_INACTIVE)!=0 );
+        if( tr_torrentStatCached(*it)->status == TR_STATUS_STOPPED )
+            enable = true;
+    event.Enable( enable );
 }
 void
 MyFrame :: OnStart( wxCommandEvent& WXUNUSED(unused) )
 {
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStat(*it)->status & TR_STATUS_INACTIVE )
+        if( tr_torrentStatCached(*it)->status == TR_STATUS_STOPPED )
             tr_torrentStart( *it );
 }
 
@@ -301,16 +302,17 @@ MyFrame :: OnStart( wxCommandEvent& WXUNUSED(unused) )
 void
 MyFrame :: OnStopUpdate( wxUpdateUIEvent& event )
 {
-    unsigned long l = 0;
+    bool enable = false;
     foreach( torrents_v, mySelectedTorrents, it )
-        l |= tr_torrentStat(*it)->status;
-    event.Enable( (l & TR_STATUS_ACTIVE)!=0 );
+        if( tr_torrentStatCached(*it)->status != TR_STATUS_STOPPED )
+            enable = true;
+    event.Enable( enable );
 }
 void
 MyFrame :: OnStop( wxCommandEvent& WXUNUSED(unused) )
 {
     foreach( torrents_v, mySelectedTorrents, it )
-        if( tr_torrentStat(*it)->status & TR_STATUS_ACTIVE )
+        if( tr_torrentStat(*it)->status != TR_STATUS_STOPPED )
             tr_torrentStop( *it );
 }
 
@@ -368,10 +370,12 @@ void MyFrame :: OnOpen( wxCommandEvent& WXUNUSED(event) )
         for( size_t i=0; i<nPaths; ++i )
         {
             const std::string filename = toStr( paths[i] );
-            tr_torrent * tor = tr_torrentInit( handle,
-                                               filename.c_str(),
-                                               mySavePath.c_str(),
-                                               false, NULL );
+            tr_ctor * ctor = tr_ctorNew( handle );
+            tr_ctorSetMetainfoFromFile( ctor, filename.c_str() );
+            tr_ctorSetDestination( ctor, TR_FALLBACK, mySavePath.c_str() );
+            tr_torrent * tor = tr_torrentNew( handle, ctor, NULL );
+            tr_ctorFree( ctor );
+
             if( tor )
                 myTorrents.push_back( tor );
         }
@@ -384,7 +388,8 @@ void MyFrame :: OnOpen( wxCommandEvent& WXUNUSED(event) )
 }
 
 
-bool MyApp::OnInit()
+bool
+MyApp :: OnInit( )
 {
     handle = tr_init( "wx" );
 
@@ -674,25 +679,28 @@ MyFrame :: MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size
     **/
 
     int count = 0;
-    tr_torrent ** torrents = tr_loadTorrents ( handle, mySavePath.c_str(), paused, &count );
+    tr_ctor * ctor = tr_ctorNew( handle );
+    tr_ctorSetPaused( ctor, TR_FORCE, paused );
+    tr_ctorSetDestination( ctor, TR_FALLBACK, mySavePath.c_str() );
+    tr_torrent ** torrents = tr_loadTorrents ( handle, ctor, &count );
     myTorrents.insert( myTorrents.end(), torrents, torrents+count );
     tr_free( torrents );
+    tr_ctorFree( ctor );
 
     wxTimerEvent dummy;
     OnPulse( dummy );
 }
 
-void MyFrame::OnExit(wxCommandEvent& WXUNUSED(event))
+void
+MyFrame :: OnExit( wxCommandEvent& WXUNUSED( event ) )
 {
     Enable( false );
 
-    foreach( torrents_v, myTorrents, it )
-        tr_torrentClose( *it );
+    myTorrents.clear( );
+    mySelectedTorrents.clear( );
+    ApplyCurrentFilter( );
 
-    myTorrents.clear ();
-    mySelectedTorrents.clear ();
-
-    ApplyCurrentFilter ();
+    tr_close( handle );
 
     /* give the connections a max of 10 seconds to shut themselves down */
     myExitTime = time(0) + 10;
